
#include <string>
#include <isis/ivm/isasm.h>

// consts

const unsigned char ISE_MAJOR = 1;
const unsigned char ISE_MINOR = 0;


// declarations

int write_int_litend(int i, gzFile outfile);
int write_byte(unsigned char b, gzFile outfile);
int write_string(const std::string & s, gzFile outfile);


// definitions

void isasm_reset_global_state(void)
{
    reset_assembler_state();
    reset_instructions_state();
}


int save_isz_file(gzFile outfile)
{
    size_t bw;
    int temp;
    unsigned int i;
    std::string s;
    char isef_id[4] = {'I','S','E','F'};

    bw = gzwrite(outfile,isef_id,sizeof(char)*4);
    if ((4*sizeof(char)) != bw) return 0;

    if (!write_byte(ISE_MAJOR,outfile)) return 0;
    if (!write_byte(ISE_MINOR,outfile)) return 0;

    temp = count_global_size();
    if (!write_int_litend(temp,outfile)) return 0;

    //temp = InstructionTable.size();
    temp = instr_num;
    if (!write_int_litend(temp,outfile)) return 0;

    for (i = 0; i < InstructionTable.size(); i++)
    {
        if (!write_byte(InstructionTable[i], outfile)) return 0;
    }

    temp = (int)StringTable.size();
    if (!write_int_litend(temp,outfile)) return 0;

    for (i = 0; i < StringTable.size(); i++)
    {
        s = IsasmNameTable[StringTable[i]];
        if (!write_string(s,outfile)) return 0;
    }

    temp = (int)IsasmFunctionTable.size();
    if (!write_int_litend(temp,outfile)) return 0;

    for (i = 0; i < IsasmFunctionTable.size(); i++)
    {
        temp = IsasmFunctionTable[i].first_instr;
        if (!write_int_litend(temp,outfile)) return 0;

        temp = (int)IsasmFunctionTable[i].param_symbols.size();
        if (!write_int_litend(temp,outfile)) return 0;

        temp = IsasmFunctionTable[i].locals_size;
        if (!write_int_litend(temp,outfile)) return 0;

        s = IsasmNameTable[IsasmFunctionTable[i].name_index];
        if (!write_string(s,outfile)) return 0;
    }

    temp = (int)CallHostNameIndicies.size();
    if (!write_int_litend(temp,outfile)) return 0;

    for (i = 0; i < CallHostNameIndicies.size(); i++)
    {
        s = IsasmNameTable[CallHostNameIndicies[i]];
        if (!write_string(s,outfile)) return 0;
    }

    temp = (int)CallObjNameIndicies.size();
    if (!write_int_litend(temp,outfile)) return 0;

    for (i = 0; i < CallObjNameIndicies.size(); i++)
    {
        s = IsasmNameTable[CallObjNameIndicies[i]];
        if (!write_string(s,outfile)) return 0;
    }

    temp = (int)LinenoTable.size();

    if (NULL == getenv("ISISSCRIPT_DEBUG"))
        temp = 0;

    if (!write_int_litend(temp,outfile)) return 0;
    for (i = 0; i < (unsigned int)temp; i++)
    {
        if (!write_int_litend(LinenoTable[i].first,outfile)) return 0;
        if (!write_int_litend(LinenoTable[i].second,outfile)) return 0;
    }

    return 1;
}

int write_int_litend(int i, gzFile outfile)
{
    size_t bw;
    unsigned char bytes[4];

    bytes[0] = 0xFF & (i >> 0);
    bytes[1] = 0xFF & (i >> 8);
    bytes[2] = 0xFF & (i >> 16);
    bytes[3] = 0xFF & (i >> 24);

    bw = gzwrite(outfile,bytes,sizeof(unsigned char)*4);

    if ((4*sizeof(char)) != bw)
        return 0;

    return 1;
}

int write_byte(unsigned char b, gzFile outfile)
{
    size_t bw;
    bw = gzwrite(outfile,&b,sizeof(unsigned char));

    if (sizeof(char) != bw)
        return 0;

    return 1;
}

int write_string(const std::string & s, gzFile outfile)
{
    size_t bw;

    if (!write_int_litend((int)s.length(),outfile)) return 0;
    
    bw = gzwrite(outfile,(void*)s.c_str(),(unsigned int)(sizeof(char)*s.length()));

    if ((s.length()*sizeof(char)) != bw)
        return 0;

    return 1;
}

