
#include <stdio.h>
#include <string.h>

#include <isis/ivm/isasm.h>

int main(int argc, char * argv[])
{
    int result;

    if (argc < 2)
    {
        printf("%s version %s",PACKAGE,VERSION);
        printf("  --  IsisASM Assembler\n");
        printf("    usage: %s file1.ism [ file2.ism .. ]\n",argv[0]);

        return 0;
    }

	for (int i = 1; i < argc; i++)
	{
        isasm_reset_global_state();

		ISASM_in = fopen(argv[i],"rt");

		if (NULL == ISASM_in)
		{
			fprintf(stderr, "Error accessing file: %s\n",argv[i]);
			return 1;
		}

		result = ISASM_parse();
		fclose(ISASM_in);

        if (0 != result)
        {
            fprintf(stderr, "Error parsing file: %s\n",argv[i]);
            return 1;
        }
        else
        {
            int dot_index;
            char outfile_name[PATH_MAX];
            gzFile outfile;

            strncpy(outfile_name, argv[i], PATH_MAX);

			for (dot_index = (int)strlen(outfile_name)-1; 
				 dot_index >= 0; 
				 dot_index--)
			{
				if (outfile_name[dot_index] == '.')
					break;
			}

            if (-1 == dot_index)
            {
                dot_index = (int)strlen(outfile_name);
            }

            strncpy(outfile_name + dot_index,".isz",PATH_MAX-dot_index);
            outfile = gzopen(outfile_name,"wb");

            if (NULL == outfile)
            {
                fprintf(stderr,"Error opening output file: %s\n",outfile_name);
                return 1;
            }

            result = save_isz_file(outfile);
            gzclose(outfile);

            if (0 == result)
            {
                fprintf(stderr,"Error saving output file: %s\n",outfile_name);
                return 1;
            }
        }
	}

	return 0;
}

