
%{

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

#include <isis/ivm/isasm.h>

#define quit ISASM_quit

%}

%union
{
    int int_val;
    float float_val;
    int name_index;
    RVAL rval;
    LVAL lval;
}

%token VAR FUNC PARAM RETVAL

%token <name_index> IDENT
%token <int_val> INTEGER
%token <float_val> FLOAT
%token <name_index> STRING

%token MOV
%token ADD SUB MUL DIV MOD NEG INC DEC
%token LOGICAND LOGICOR BITAND BITOR XOR NOT SHL SHR
%token CONCAT GETCHAR SETCHAR
%token CMPE CMPNE CMPG CMPGE CMPL CMPLE
%token JMP JZ JNZ
%token PUSH POP
%token CALL CALLHOST CALLOBJ RET THIS

%type <rval> rvalue
%type <lval> lvalue

%%

asmfile: globallist functionlist  { resolve_call_targets(); 
                                    resolve_callhost_targets();
                                    resolve_callobj_targets(); }
       ;

globallist: 
          | globallist globaldec
          ;

globaldec: globalscalar
         | globalarray
         ;

globalscalar: VAR IDENT ';'  { symbol_table_add($2,SYM_GLOBAL_SCALAR); }
            ;

globalarray: VAR IDENT '[' INTEGER ']' ';'  { int sym_index = 
                                      symbol_table_add($2, SYM_GLOBAL_ARRAY);
                                      set_symbol_extra_info(sym_index,$4); }
           ;

functionlist: functionlist function
            | function
            ;

function: FUNC funcname '{' funcheader instrlist ret '}'  {
             instr_num++; resolve_label_jumps(); isasm_clear_func_symbols(); }
        ;

funcname: IDENT  { isasm_func_num = isasm_new_func($1); }
        ;

funcheader: paramlist locallist   { isasm_calc_func_locals_size(); }
          ;

paramlist: 
         | paramlist param
         ;

param: PARAM IDENT ';'  { isasm_add_param($2); }
     ;

locallist: 
       | locallist localdec
       ;

localdec: localscalar
      | localarray
      ;

localscalar: VAR IDENT ';'  { isasm_add_local_scalar($2); }
           ;

localarray: VAR IDENT '[' INTEGER ']' ';'  { add_local_array($2,$4); }
        ;

instrlist: 
         | instrlist label
         | instrlist lineno
         | instrlist instr      { instr_num++; }
         ;

label: IDENT ':'  { label_table_set(label_table_add($1),instr_num); }
     ;

lineno: '(' INTEGER ')'  { lineno_table_add($2,instr_num); }
      ;

instr: mov
     | add
     | sub
     | mul
     | div
     | mod
     | neg
     | inc
     | dec
     | logicand
     | logicor
     | bitand
     | bitor
     | xor
     | not
     | shl
     | shr
     | concat
     | getchar
     | setchar
     | cmpe
     | cmpne
     | cmpg
     | cmpge
     | cmpl
     | cmple
     | jmp
     | jz
     | jnz
     | push
     | pop
     | call
     | callhost
     | callobj
     | this
     | ret
     ;


mov: MOV lvalue ',' rvalue ';'    { emit_instr_lvalrval(INSTR_MOV,&$2,&$4); }
   ;


add: ADD lvalue ',' rvalue ',' rvalue ';'    
                            { emit_instr_lvalrvalrval(INSTR_ADD,&$2,&$4,&$6); }
   ;

sub: SUB lvalue ',' rvalue ',' rvalue ';'    
                            { emit_instr_lvalrvalrval(INSTR_SUB,&$2,&$4,&$6); }
   ;

mul: MUL lvalue ',' rvalue ',' rvalue ';'     
                            { emit_instr_lvalrvalrval(INSTR_MUL,&$2,&$4,&$6); }
   ;

div: DIV lvalue ',' rvalue ',' rvalue ';'
                            { emit_instr_lvalrvalrval(INSTR_DIV,&$2,&$4,&$6); }
   ;

mod: MOD lvalue ',' rvalue ',' rvalue ';'    
                            { emit_instr_lvalrvalrval(INSTR_MOD,&$2,&$4,&$6); }
   ;

neg: NEG lvalue ',' rvalue ';'    { emit_instr_lvalrval(INSTR_NEG,&$2,&$4); }
   ;

inc: INC lvalue ',' rvalue ';'    { emit_instr_lvalrval(INSTR_INC,&$2,&$4); }
   ;

dec: DEC lvalue ',' rvalue ';'    { emit_instr_lvalrval(INSTR_DEC,&$2,&$4); }
   ;


logicand: LOGICAND lvalue ',' rvalue ',' rvalue ';'    
                       { emit_instr_lvalrvalrval(INSTR_LOGICAND,&$2,&$4,&$6); }
   ;

logicor: LOGICOR lvalue ',' rvalue ',' rvalue ';'      
                        { emit_instr_lvalrvalrval(INSTR_LOGICOR,&$2,&$4,&$6); }
  ;

bitand: BITAND lvalue ',' rvalue ',' rvalue ';'    
                       { emit_instr_lvalrvalrval(INSTR_BITAND,&$2,&$4,&$6); }
   ;

bitor: BITOR lvalue ',' rvalue ',' rvalue ';'      
                        { emit_instr_lvalrvalrval(INSTR_BITOR,&$2,&$4,&$6); }
  ;

xor: XOR lvalue ',' rvalue ',' rvalue ';'    
                            { emit_instr_lvalrvalrval(INSTR_XOR,&$2,&$4,&$6); }
   ;

not: NOT lvalue ',' rvalue ';'    { emit_instr_lvalrval(INSTR_NOT,&$2,&$4); }
   ;

shl: SHL lvalue ',' rvalue ',' rvalue ';'    
                            { emit_instr_lvalrvalrval(INSTR_SHL,&$2,&$4,&$6); }
   ;

shr: SHR lvalue ',' rvalue ',' rvalue ';'    
                            { emit_instr_lvalrvalrval(INSTR_SHR,&$2,&$4,&$6); }
   ;


concat: CONCAT lvalue ',' rvalue ',' rvalue ';'
                         { emit_instr_lvalrvalrval(INSTR_CONCAT,&$2,&$4,&$6); }
      ;

getchar: GETCHAR lvalue ',' rvalue ',' rvalue ';'  { 
                          emit_instr_lvalrvalrval(INSTR_GETCHAR,&$2,&$4,&$6); }
       ;

setchar: SETCHAR lvalue ',' rvalue ',' rvalue ';'  { 
                          emit_instr_lvalrvalrval(INSTR_SETCHAR,&$2,&$4,&$6); }
       ;


cmpe: CMPE lvalue ',' rvalue ',' rvalue ';'   
                           { emit_instr_lvalrvalrval(INSTR_CMPE,&$2,&$4,&$6); }
    ;

cmpne: CMPNE lvalue ',' rvalue ',' rvalue ';' 
                          { emit_instr_lvalrvalrval(INSTR_CMPNE,&$2,&$4,&$6); }
     ;

cmpg: CMPG lvalue ',' rvalue ',' rvalue ';'   
                           { emit_instr_lvalrvalrval(INSTR_CMPG,&$2,&$4,&$6); }
    ;

cmpge: CMPGE lvalue ',' rvalue ',' rvalue ';' 
                          { emit_instr_lvalrvalrval(INSTR_CMPGE,&$2,&$4,&$6); }
     ;

cmpl: CMPL lvalue ',' rvalue ',' rvalue ';'   
                           { emit_instr_lvalrvalrval(INSTR_CMPL,&$2,&$4,&$6); }
    ;

cmple: CMPLE lvalue ',' rvalue ',' rvalue ';' 
                          { emit_instr_lvalrvalrval(INSTR_CMPLE,&$2,&$4,&$6); }
     ;


jmp: JMP IDENT ';'               { emit_instr_jmp($2); }
   ;

jz: JZ rvalue ',' IDENT ';'      { emit_instr_jz($4,&$2); }
  ;

jnz: JNZ rvalue ',' IDENT ';'    { emit_instr_jnz($4,&$2); }
  ;


push: PUSH rvalue ';'                { emit_instr_rval(INSTR_PUSH,&$2); }
    ;

pop: POP lvalue ';'                  { emit_instr_lval(INSTR_POP,&$2); }
   ;


call: CALL IDENT ',' rvalue ';'         {  emit_instr_call($2,&$4); }
    ;

callhost: CALLHOST IDENT ',' rvalue ';' { emit_instr_callhost($2,&$4); }
        ;

callobj: CALLOBJ lvalue ',' IDENT ',' rvalue ';' { 
                                              emit_instr_callobj(&$2,$4,&$6); }
        ;

this: THIS ';'                       { emit_instr_this(); }
   ;

ret: RET ';'                         { emit_instr_ret(); }
   ;

lvalue: IDENT   { $$.symbol_index = find_closest_symbol($1);
                  if (-1 == $$.symbol_index) 
                      quit("Unknown symbol: %s\n",isasm_name_table_get($1));
                  $$.type = LVAL_SCALAR; $$.offset.literal = 0; }
      | IDENT '[' INTEGER ']' { $$.symbol_index = find_closest_symbol($1);
                                if (-1 == $$.symbol_index) 
                         quit("Unknown symbol: %s\n",isasm_name_table_get($1));
                            $$.type = LVAL_ARRAY_LIT; $$.offset.literal = $3; }
      | IDENT '[' IDENT ']' { $$.symbol_index = find_closest_symbol($1);
                              if (-1 == $$.symbol_index) 
                         quit("Unknown symbol: %s\n",isasm_name_table_get($1));
                              $$.type = LVAL_ARRAY_SYM; 
                              $$.offset.symbol_index = find_closest_symbol($3);
                            if (-1 == $$.offset.symbol_index) 
                       quit("Unknown symbol: %s\n",isasm_name_table_get($3)); }
      | RETVAL  { $$.type = LVAL_RETVAL; }
      ;

rvalue: IDENT   { $$.val.lval.symbol_index = find_closest_symbol($1);
                  if (-1 == $$.val.lval.symbol_index) 
                      quit("Unknown symbol: %s\n",isasm_name_table_get($1));
                  $$.type = RVAL_LVAL; $$.val.lval.type = LVAL_SCALAR; 
                  $$.val.lval.offset.literal= 0; }
      | IDENT '[' INTEGER ']' { $$.val.lval.symbol_index = 
                                               find_closest_symbol($1);
                              if (-1 == $$.val.lval.symbol_index) 
                         quit("Unknown symbol: %s\n",isasm_name_table_get($1));
                              $$.type = RVAL_LVAL; 
                              $$.val.lval.type = LVAL_ARRAY_LIT; 
                              $$.val.lval.offset.literal= $3; }
      | IDENT '[' IDENT ']' { $$.val.lval.symbol_index = 
                                    find_closest_symbol($1);
                              if (-1 == $$.val.lval.symbol_index) 
                         quit("Unknown symbol: %s\n",isasm_name_table_get($1));
                              $$.type = RVAL_LVAL;
                              $$.val.lval.type = LVAL_ARRAY_SYM; 
                     $$.val.lval.offset.symbol_index = find_closest_symbol($3);
                            if (-1 == $$.val.lval.offset.symbol_index) 
                       quit("Unknown symbol: %s\n",isasm_name_table_get($3)); }
      | RETVAL                  { $$.type = RVAL_LVAL; 
                                  $$.val.lval.type = LVAL_RETVAL; }
      | INTEGER                 { $$.type = RVAL_INTEGER;
                                  $$.val.int_val = $1; }
      | FLOAT                   { $$.type = RVAL_FLOAT;
                                  $$.val.float_val = $1; }
      | STRING                  { $$.type = RVAL_STRINGIND;
                                  $$.val.string_ind = string_table_add($1); }
      ;

%%

int ISASM_wrap(void)
{
    return 1;
}

void ISASM_error(char * text)
{
    printf("Line %d:  %s near token: %s\n",ISASM_lineno,text,ISASM_text);
}

void quit(char * format, ... )
{
    va_list args;
    char buffer[4096];

    va_start(args,format);
    vsprintf(buffer,format,args);
    ISASM_error(buffer);
    exit(1);
}
