
#include <stdlib.h>

#include <isis/ivm/isc.h>

void optimize_ism()
{
    if (!getenv("ISC_NO_OPTIMIZATION"))
    {
        optimize_naked_expressions();
        optimize_temp_movs();
    }
}

bool isreg(const std::string & name)
{
    if (name.length() < 3)
        return false;

    if (name[0] == '_' && name[1] == 'r')
        return true;

    return false;
}

void replace_rvalue(LINE_STRUCT & line, const std::string & name1,
                                        const std::string & name2)
{
    switch (line.instr)
    {
    case INSTR_MOV:
    case INSTR_NEG:
    case INSTR_INC:
    case INSTR_DEC:
    case INSTR_NOT:
        {
            if (line.ops[1] == name1)
                line.ops[1] = name2;
        } break;
    case INSTR_ADD:
    case INSTR_SUB:
    case INSTR_MUL:
    case INSTR_DIV:
    case INSTR_MOD:
    case INSTR_LOGICAND:
    case INSTR_LOGICOR:
    case INSTR_BITAND:
    case INSTR_BITOR:
    case INSTR_XOR:
    case INSTR_SHL:
    case INSTR_SHR:
    case INSTR_CONCAT:
    case INSTR_GETCHAR:
    case INSTR_SETCHAR:
    case INSTR_CMPE:
    case INSTR_CMPNE:
    case INSTR_CMPG:
    case INSTR_CMPGE:
    case INSTR_CMPL:
    case INSTR_CMPLE:
        {
            if (line.ops[1] == name1)
                line.ops[1] = name2;

            if (line.ops[2] == name1)
                line.ops[2] = name2;
        } break;
    case INSTR_PUSH:
        {
            if (line.ops[0] == name1)
                line.ops[0] = name2;
        } break;
    case INSTR_CALL:
    case INSTR_CALLHOST:
        {
            if (line.ops[line.ops.size()-1] == name1)
                line.ops[line.ops.size()-1] = name2;
        } break;
    case INSTR_CALLOBJ:
        {
            if (line.ops[line.ops.size()-1] == name1)
                line.ops[line.ops.size()-1] = name2;

            if (line.ops[0] == name1)
                line.ops[0] = name2;
        } break;
    case INSTR_JZ:
    case INSTR_JNZ:
        {
            if (line.ops[0] == name1)
                line.ops[0] = name2;
        } break;
    case INSTR_JMP:
    case INSTR_POP:
    case INSTR_RET:
    case INSTR_THIS:
    default:
        break;
    }

    return;
}

bool isrvalue(LINE_STRUCT * line, const std::string & name)
{
    switch (line->instr)
    {
    case INSTR_MOV:
    case INSTR_NEG:
    case INSTR_INC:
    case INSTR_DEC:
    case INSTR_NOT:
        {
            if (line->ops[1] == name)
                return true;
        } break;
    case INSTR_ADD:
    case INSTR_SUB:
    case INSTR_MUL:
    case INSTR_DIV:
    case INSTR_MOD:
    case INSTR_LOGICAND:
    case INSTR_LOGICOR:
    case INSTR_BITAND:
    case INSTR_BITOR:
    case INSTR_XOR:
    case INSTR_SHL:
    case INSTR_SHR:
    case INSTR_CONCAT:
    case INSTR_GETCHAR:
    case INSTR_SETCHAR:
    case INSTR_CMPE:
    case INSTR_CMPNE:
    case INSTR_CMPG:
    case INSTR_CMPGE:
    case INSTR_CMPL:
    case INSTR_CMPLE:
        {
            if (line->ops[1] == name || line->ops[2] == name)
                return true;
        } break;
    case INSTR_PUSH:
        {
            if (line->ops[0] == name)
                return true;
        } break;
    case INSTR_CALL:
    case INSTR_CALLHOST:
        {
            if (line->ops[line->ops.size()-1] == name)
                return true;
        } break;
    case INSTR_CALLOBJ:
        {
            if (line->ops[0] == name || line->ops[line->ops.size()-1] == name)
                return true;
        } break;
    case INSTR_JZ:
    case INSTR_JNZ:
        {
            if (line->ops[0] == name)
                return true;
        } break;
    case INSTR_JMP:
    case INSTR_POP:
    case INSTR_RET:
    case INSTR_THIS:
    default:
        break;
    }

    return false;
}

bool islvalue(LINE_STRUCT * line, const std::string & name)
{
    switch (line->instr)
    {
    case INSTR_MOV:
    case INSTR_NEG:
    case INSTR_INC:
    case INSTR_DEC:
    case INSTR_NOT:
    case INSTR_ADD:
    case INSTR_SUB:
    case INSTR_MUL:
    case INSTR_DIV:
    case INSTR_MOD:
    case INSTR_LOGICAND:
    case INSTR_LOGICOR:
    case INSTR_BITAND:
    case INSTR_BITOR:
    case INSTR_XOR:
    case INSTR_SHL:
    case INSTR_SHR:
    case INSTR_CONCAT:
    case INSTR_GETCHAR:
    case INSTR_SETCHAR:
    case INSTR_CMPE:
    case INSTR_CMPNE:
    case INSTR_CMPG:
    case INSTR_CMPGE:
    case INSTR_CMPL:
    case INSTR_CMPLE:
    case INSTR_POP:
        {
            if (line->ops[0] == name)
                return true;
        } break;
    case INSTR_CALL:
    case INSTR_CALLHOST:
    case INSTR_CALLOBJ:
    case INSTR_JMP:
    case INSTR_JZ:
    case INSTR_JNZ:
    case INSTR_RET:
    case INSTR_THIS:
    case INSTR_PUSH:
    default:
        break;
    }

    return false;
}

void optimize_naked_expressions()
{
    for (unsigned int i = 0; i < IscFunctionTable.size(); i++)
    {
        std::list<LINE_STRUCT>::iterator iter1;
        std::list<LINE_STRUCT>::iterator iter2;

        for (iter1 = IscFunctionTable[i].body.begin();
             iter1 != IscFunctionTable[i].body.end(); )
        {
            bool del = false;

            if (LINE_INSTR == iter1->type && INSTR_MOV == iter1->instr && 
                isreg(iter1->ops[0]))
            {
                del = true;

                for (iter2 = iter1, iter2++;
                     iter2 != IscFunctionTable[i].body.end();
                     iter2++)
                {
                    if (LINE_INSTR == iter2->type)
                    {
                        if (isrvalue(&(*iter2),iter1->ops[0]))
                            del = false;

                        if (islvalue(&(*iter2),iter1->ops[0]))
                            break;
                    }
                }
            }

            iter2 = iter1++;

            if (del)
                IscFunctionTable[i].body.erase(iter2);
        }
    }
}

void optimize_temp_movs()
{
    for (unsigned int i = 0; i < IscFunctionTable.size(); i++)
    {
        std::list<LINE_STRUCT>::iterator iter1;
        std::list<LINE_STRUCT>::iterator iter2;
        std::list<LINE_STRUCT>::iterator iter3;
        std::list<LINE_STRUCT>::iterator iter4;

        for (iter1 = IscFunctionTable[i].body.begin();
             iter1 != IscFunctionTable[i].body.end(); )
        {
            if (LINE_INSTR == iter1->type && INSTR_MOV == iter1->instr && 
                isreg(iter1->ops[0]))
            {
                int used = 0;
                bool overwritten = false;

                std::string name1 = iter1->ops[0];
                std::string name2 = iter1->ops[1];

                for (iter2 = iter1, iter2++;
                     iter2 != IscFunctionTable[i].body.end();
                     iter2++)
                {
                    if (LINE_INSTR == iter2->type)
                    { 
                        if (islvalue(&(*iter2),name1))
                            break;

                        if (isrvalue(&(*iter2),name1))
                            used++;

                        if (islvalue(&(*iter2),name2))
                            overwritten = true;
                    }
                }

                iter2 = iter1++;

                if (used <= 1 && !overwritten)
                {
                    bool replace = true;

                    if (1 == used && "RetVal" == name2)
                    {
                        for (iter4 = iter3 = iter1;
                             iter3 != IscFunctionTable[i].body.end();
                             iter3++)
                        {
                            if (LINE_INSTR == iter3->type)
                            { 
                                if (islvalue(&(*iter3),name1))
                                    break;

                                if (isrvalue(&(*iter3),name1))
                                    iter4 = iter3;
                            }
                        }

                        for (iter3 = iter1;
                             iter3 != iter4;
                             iter3++)
                        {
                            if (LINE_INSTR == iter3->type)
                            {
                                if (islvalue(&(*iter3),name1))
                                    break;

                                if (iter3->instr == INSTR_CALL || 
                                    iter3->instr == INSTR_CALLHOST ||
                                    iter3->instr == INSTR_CALLOBJ ||
                                    iter3->instr == INSTR_THIS)
                                {
                                    replace = false;
                                }
                            }
                        }
                    }

                    if (replace)
                    {
                        IscFunctionTable[i].body.erase(iter2);

                        for (iter3 = iter1;
                            iter3 != IscFunctionTable[i].body.end();
                            iter3++)
                        {
                            if (LINE_INSTR == iter3->type)
                            {
                                if (islvalue(&(*iter3),name1))
                                    break;

                                replace_rvalue(*iter3,name1,name2);
                            }
                        }
                    }
                }
            }
            else
            {
                iter1++;
            }
        }
    }
}

