
%{

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

#include <isis/ivm/isc.h>

%}

%union
{
    int int_val;
    float float_val;
    int name_index;
    EXPR_STRUCT expr;
    LOOP_STRUCT loop;
}

%token VAR
%token RETURN NEW THIS
%token IF ELSE WHILE FOR BREAK CONTINUE

%token PLUSPLUS MINUSMINUS NEG
%token SHL SHR
%token LOGICAND LOGICOR
%token PLUSEQUALS MINUSEQUALS TIMESEQUALS DIVEQUALS MODEQUALS
%token CMPE CMPNE CMPGE CMPLE

%token <name_index> IDENT
%token <int_val> INTEGER
%token <float_val> FLOAT
%token <name_index> STRING

%right '=' PLUSEQUALS MINUSEQUALS TIMESEQUALS DIVEQUALS MODEQUALS

%left LOGICAND
%left LOGICOR

%left '|' '^'
%left '&'

%left CMPE CMPNE 
%left '<' '>' CMPGE CMPLE

%left SHL SHR

%left '+' '-'
%left '*' '/' '%'

%left '$'

%nonassoc PLUSPLUS MINUSMINUS NEG '!'

%left '['
%left '.'

%nonassoc LOWER_THAN_ELSE
%nonassoc ELSE

%type <int_val> exprs;
%type <int_val> exprlist;

%type <expr> expression rvalue lstruct vecarr assignment funccall methcall 
%type <expr> incdec arithmeticop stringop logicalop comparisonop

%type <int_val> whilestart for1 for3 ifstart iftop ifelse iffirst
%type <loop> whiletop fortop for2

%%

issfile: declist               { fixup_ism(); optimize_ism(); }
       ;

declist: dec
       | declist dec
       ;

dec: globalscalar
   | function
   ;

globalscalar: VAR IDENT ';'                  { add_global($2,0); }
            ;

function: funcname '(' paramlist ')' block   { isc_clear_func_symbols(); }
        ;

funcname: IDENT { isc_func_num = isc_new_func($1); }
        ;

paramlist: 
         | params
         ;

params: param  
      | params ',' param
      ;

param: IDENT    { isc_add_param($1); }
     ;

block: '{' statements '}'
     ;

statements: 
          | statements statement
          ;

statement: block
         | localscalar
         | whileloop
         | forloop
         | ifexp
         | RETURN ';'            { emit_return(NULL); }
         | RETURN expression ';' { emit_return(&$2); done_expr(&$2); }
         | BREAK ';'             { insert_jmp(get_break_target()); }
         | CONTINUE ';'          { insert_jmp(get_continue_target()); }
         | expression ';'        { done_expr(&$1); }
         ;

localscalar: VAR IDENT ';'                 { isc_add_local($2,0); }
           ;

whileloop: whiletop block  { insert_jmp($1.begin_label);
                             insert_label($1.end_label); 
                             pop_loop(); }
         ;

whiletop: whilestart '(' expression ')' { $$.end_label = make_label();
                                          insert_jz(&$3,$$.end_label);
                                          done_expr(&$3); $$.begin_label = 
                                      $$.continue_label = $1; push_loop(&$$); }
        ;

whilestart: WHILE   { $$ = make_label(); insert_label($$); }
          ;

forloop: fortop statement { pop_for(); insert_jmp($1.begin_label); 
                            insert_label($1.end_label); pop_loop(); }
       ;

fortop: FOR '(' for1 ';' for2 ';' for3 ')' { $$.begin_label = $3;
                                         $$.continue_label = $5.continue_label; 
                                             $$.end_label = $5.end_label; 
                                             push_loop(&$$); }
      ;

for1: expression { done_expr(&$1); $$ = make_label(); insert_label($$); }
    ;

for2: expression { done_expr(&$1); $$.end_label = make_label(); 
                $$.continue_label = make_label(); insert_jz(&$1,$$.end_label); 
                   push_for(); insert_label($$.continue_label); }
    ;

for3: expression { done_expr(&$1); end_forstart(); }
    ;

ifexp: iftop %prec LOWER_THAN_ELSE  { insert_label($1); }
     | ifelse                       { insert_label($1); }
     ; 

ifelse: iffirst statement   { $$ = $1; }
      ;

iffirst: iftop ELSE { $$ = make_label(); insert_jmp($$); insert_label($1); }
       ;

iftop: ifstart statement { $$ = $1; }
    ;

ifstart: IF '(' expression ')' { $$ = make_label(); insert_jz(&$3,$$); 
                                 done_expr(&$3); }
       ; 

expression: '(' expression ')' { $$ = $2; }
          | assignment 
          | incdec 
          | funccall
          | methcall
          | rvalue
          | lstruct
	      | vecarr
          | arithmeticop
          | logicalop
          | stringop
          | comparisonop
          | THIS                          { $$.type = EXPR_REGISTER;
	                                $$.val.reg_num = emit_this_call(); }
          | NEW IDENT                     { $$.type = EXPR_REGISTER; 
                                        $$.val.reg_num = emit_new_call($2,0); }
          | NEW IDENT '(' exprlist ')'    { $$.type = EXPR_REGISTER; 
                                       $$.val.reg_num = emit_new_call($2,$4); }
          ;

assignment: rvalue '=' expression        { $$ = $1; emit_assignment(&$1,&$3);
                                           done_expr(&$3); 
                                           if (EXPR_RVAL_SCALAR == $1.type)
                                               isc_add_assigned($1.val.name); }
          | expression '.' IDENT '=' expression  { EXPR_STRUCT expr; $$ = $5;
                                      expr.type = EXPR_LIT_STRING;
                                      expr.val.string_ind = $3;
                                      emit_push_expr(&expr);
                                      emit_push_expr(&$5);
                                      expr.type = EXPR_REGISTER;
              expr.val.reg_num = emit_callobj(&$1,isc_name_table_add("Set"),2); 
                                      done_expr(&$1); done_expr(&expr); }
          | expression '[' expression ']' '=' expression  { EXPR_STRUCT expr;
	                              $$ = $6;
                                      emit_push_expr(&$3);
                                      emit_push_expr(&$6);
                                      expr.type = EXPR_REGISTER;
              expr.val.reg_num = emit_callobj(&$1,isc_name_table_add("Set"),2); 
                                      done_expr(&$1); done_expr(&$3); }
          | rvalue PLUSEQUALS expression { $$ = $1; 
                            emit_opequals(INSTR_ADD,&$1,&$3); done_expr(&$3); }
          | rvalue MINUSEQUALS expression { $$ = $1; 
                            emit_opequals(INSTR_SUB,&$1,&$3); done_expr(&$3); }
          | rvalue TIMESEQUALS expression { $$ = $1; 
                            emit_opequals(INSTR_MUL,&$1,&$3); done_expr(&$3); }
          | rvalue DIVEQUALS expression { $$ = $1; 
                            emit_opequals(INSTR_DIV,&$1,&$3); done_expr(&$3); }
          | rvalue MODEQUALS expression { $$ = $1; 
                            emit_opequals(INSTR_MOD,&$1,&$3); done_expr(&$3); }
          ;

incdec: PLUSPLUS rvalue    { $$ = $2; emit_inc(&$2); }
      | rvalue PLUSPLUS    { $$.type = EXPR_REGISTER; 
                             $$.val.reg_num = emit_reg_expression(&$1); 
                             emit_inc(&$1); done_expr(&$1); }
      | MINUSMINUS rvalue  { $$ = $2; emit_dec(&$2); }
      | rvalue MINUSMINUS  { $$.type = EXPR_REGISTER; 
                             $$.val.reg_num = emit_reg_expression(&$1); 
                             emit_dec(&$1); done_expr(&$1); }
      ;

funccall: IDENT '(' exprlist ')'           { $$.type = EXPR_REGISTER;
                                          $$.val.reg_num = emit_call($1,$3); }
        ;

methcall: expression '.' IDENT '(' exprlist ')' { $$.type = EXPR_REGISTER;
                                    $$.val.reg_num = emit_callobj(&$1,$3,$5); }
        ; 

lstruct: expression '.' IDENT {  EXPR_STRUCT expr; expr.type = EXPR_LIT_STRING; 
                   expr.val.string_ind = $3;
                   emit_push_expr(&expr);
                   $$.type = EXPR_REGISTER;
               $$.val.reg_num = emit_callobj(&$1,isc_name_table_add("Get"),1); 
                  done_expr(&$1); }
       ; 

vecarr: expression '[' expression ']' {  
                   emit_push_expr(&$3);
                   $$.type = EXPR_REGISTER;
                $$.val.reg_num = emit_callobj(&$1,isc_name_table_add("Get"),1); 
                  done_expr(&$1); done_expr(&$3); }
       ; 

exprlist:                     { $$ = 0; }
        | exprs               { $$ = $1; }
        ;

exprs: expression             { $$ = 1; emit_push_expr(&$1); done_expr(&$1); }
     | exprs ',' expression   { $$ = $1 + 1; emit_push_expr(&$3); 
                                done_expr(&$3); }
     ;

arithmeticop: expression '+' expression { $$.type = EXPR_REGISTER;
                                 $$.val.reg_num = emit_2op(INSTR_ADD,&$1,&$3);
                                    done_expr(&$1); done_expr(&$3); }
            | expression '-' expression { $$.type = EXPR_REGISTER;
                                 $$.val.reg_num = emit_2op(INSTR_SUB,&$1,&$3);
                                    done_expr(&$1); done_expr(&$3); }
            | expression '*' expression { $$.type = EXPR_REGISTER;
                                 $$.val.reg_num = emit_2op(INSTR_MUL,&$1,&$3);
                                    done_expr(&$1); done_expr(&$3); }
            | expression '/' expression { $$.type = EXPR_REGISTER;
                                  $$.val.reg_num = emit_2op(INSTR_DIV,&$1,&$3);
                                    done_expr(&$1); done_expr(&$3); }
            | expression '%' expression { $$.type = EXPR_REGISTER;
                                  $$.val.reg_num = emit_2op(INSTR_MOD,&$1,&$3);
                                    done_expr(&$1); done_expr(&$3); }
            | '-' expression %prec NEG  { $$.type = EXPR_REGISTER;
                                   $$.val.reg_num = emit_1op(INSTR_NEG,&$2);
                                    done_expr(&$2); }
            ;

logicalop: expression '&' expression { $$.type = EXPR_REGISTER;
                               $$.val.reg_num = emit_2op(INSTR_BITAND,&$1,&$3);
                                    done_expr(&$1); done_expr(&$3); }
         | expression '|' expression { $$.type = EXPR_REGISTER;
                                $$.val.reg_num = emit_2op(INSTR_BITOR,&$1,&$3);
                                    done_expr(&$1); done_expr(&$3); }
         | expression LOGICAND expression { $$.type = EXPR_REGISTER;
                             $$.val.reg_num = emit_2op(INSTR_LOGICAND,&$1,&$3);
                                    done_expr(&$1); done_expr(&$3); }
         | expression LOGICOR expression { $$.type = EXPR_REGISTER;
                              $$.val.reg_num = emit_2op(INSTR_LOGICOR,&$1,&$3);
                                    done_expr(&$1); done_expr(&$3); }
         | expression '^' expression { $$.type = EXPR_REGISTER;
                                  $$.val.reg_num = emit_2op(INSTR_XOR,&$1,&$3);
                                    done_expr(&$1); done_expr(&$3); }
         | '!' expression            { $$.type = EXPR_REGISTER;
                                    $$.val.reg_num = emit_1op(INSTR_NOT,&$2);
                                    done_expr(&$2); }
         | expression SHL expression { $$.type = EXPR_REGISTER;
                                  $$.val.reg_num = emit_2op(INSTR_SHL,&$1,&$3);
                                    done_expr(&$1); done_expr(&$3); }
         | expression SHR expression { $$.type = EXPR_REGISTER;
                                  $$.val.reg_num = emit_2op(INSTR_SHR,&$1,&$3);
                                    done_expr(&$1); done_expr(&$3); }
         ;

stringop: expression '$' expression  { $$.type = EXPR_REGISTER;
                               $$.val.reg_num = emit_2op(INSTR_CONCAT,&$1,&$3);
                                    done_expr(&$1); done_expr(&$3); }
        ;

comparisonop: expression CMPE expression  { $$.type = EXPR_REGISTER;
                                 $$.val.reg_num = emit_2op(INSTR_CMPE,&$1,&$3);
                                    done_expr(&$1); done_expr(&$3); }
            | expression CMPNE expression { $$.type = EXPR_REGISTER;
                                $$.val.reg_num = emit_2op(INSTR_CMPNE,&$1,&$3);
                                    done_expr(&$1); done_expr(&$3); }
            | expression '>' expression   { $$.type = EXPR_REGISTER;
                                 $$.val.reg_num = emit_2op(INSTR_CMPG,&$1,&$3);
                                    done_expr(&$1); done_expr(&$3); }
            | expression CMPGE expression { $$.type = EXPR_REGISTER;
                                $$.val.reg_num = emit_2op(INSTR_CMPGE,&$1,&$3);
                                    done_expr(&$1); done_expr(&$3); }
            | expression '<' expression   { $$.type = EXPR_REGISTER;
                                 $$.val.reg_num = emit_2op(INSTR_CMPL,&$1,&$3);
                                    done_expr(&$1); done_expr(&$3); }
            | expression CMPLE expression { $$.type = EXPR_REGISTER;
                                $$.val.reg_num = emit_2op(INSTR_CMPLE,&$1,&$3);
                                    done_expr(&$1); done_expr(&$3); }
            ;

rvalue: IDENT                   { $$.type = EXPR_RVAL_SCALAR; $$.val.name = $1;
                                  if (!is_local_scalar($1) && !is_param($1) &&
                              !is_global_scalar($1)) { isc_add_local($1,0); } }
      | INTEGER            { $$.type = EXPR_LIT_INTEGER; $$.val.integer = $1; }
      | FLOAT               { $$.type = EXPR_LIT_FLOAT; $$.val.floating = $1; }
      | STRING           { $$.type = EXPR_LIT_STRING; $$.val.string_ind = $1; }
      ;

%%

int ISC_wrap(void)
{
    return 1;
}

void ISC_error(char * text)
{
    printf("Line %d: %s near token: %s\n",ISC_lineno,text,ISC_text);
}

void ISC_quit(char * format, ... )
{
    va_list args;
    char buffer[4096];

    va_start(args,format);
    vsprintf(buffer,format,args);
    ISC_error(buffer);
    exit(1);
}

