
%{ 

#define YY_NO_UNPUT

#include <math.h>
#include <stdlib.h>

#include <isis/ivm/isc.h>
#include "parser.h"

int ISC_lineno = 1;

#define yylval ISC_lval
#define yylex ISC_lex
#define yywrap ISC_wrap
#define yyin ISC_in
#define yytext ISC_text

%}

%%

\/\/.*      ;
#.*         ;
[ \t]       ; 
\n	        { ISC_lineno++; }

var         { return VAR; }
return      { return RETURN; }
new         { return NEW; }
this        { return THIS; }
if          { return IF; }
else        { return ELSE; }
while       { return WHILE; }
for         { return FOR; }
break       { return BREAK; }
continue    { return CONTINUE; }
\+\+        { return PLUSPLUS; }
--          { return MINUSMINUS; }
\+=         { return PLUSEQUALS; }
-=          { return MINUSEQUALS; }
\*=         { return TIMESEQUALS; }
\/=         { return DIVEQUALS; }
%=          { return MODEQUALS; }
\<\<        { return SHL; }
\>\>        { return SHR; }
==          { return CMPE; }
!=          { return CMPNE; }
\>=         { return CMPGE; }
\<=         { return CMPLE; }
&&          { return LOGICAND; }
\|\|        { return LOGICOR; }

[0-9]+	    { yylval.int_val = atoi(yytext); return INTEGER; }
true        { yylval.int_val = 1; return INTEGER; }
false       { yylval.int_val = 0; return INTEGER; }
null        { yylval.int_val = 0; return INTEGER; }

((([0-9]+)|([0-9]*\.[0-9]+))([eE][-+]?[0-9]+)?)    {
              yylval.float_val = (float)atof(yytext); return FLOAT; }

[a-zA-Z_][a-zA-Z0-9_]*  { yylval.name_index = isc_name_table_add(yytext); 
						return IDENT; }

\"[^"\n]*["\n] { yytext[yyleng-1] = '\0'; 
             yylval.name_index = isc_name_table_add(yytext+1); return STRING; }

.			{ return yytext[0]; }

%%

