
#ifndef ISIS_GAME_CELLIMAGE_H_INCLUDED
#define ISIS_GAME_CELLIMAGE_H_INCLUDED

#include <string>
#include <map>
#include <vector>

#include <isis/ivm.h>

#include <isis/game/Image2D.h>

namespace Isis {

class GraphicsEngine;
	
class CellImage : public Image2D
{
public:
		
	CellImage();
	virtual ~CellImage();

	void Init(Image2D * pSourceImage, int iCellX, int iCellY, 
			  int iCellWidth, int iCellHeight, 
			  Image2D::Flags DrawFlags = Image2D::None);

	virtual void Draw(int iX = 0, int iY = 0, 
		              int iWidth = 0, int iHeight = 0,
					  Flags DrawFlags = None,
					  unsigned int modColor = 0xFFFFFFFF,
					  int iCellX = 0, int iCellY = 0, 
					  int iCellWidth = 0, int iCellHeight = 0);

	virtual void Draw3D(int iX = 0, int iY = 0, 
		                int iWidth = 0, int iHeight = 0,
					    Flags DrawFlags = None,
					    unsigned int modColor = 0xFFFFFFFF,
					    int iCellX = 0, int iCellY = 0, 
					    int iCellWidth = 0, int iCellHeight = 0);

	virtual int GetWidth();
	virtual int GetHeight();

	virtual void IVMFillExportTable(std::vector<std::string> & table);
	virtual bool IVMContainsMethod(const std::string & funcname);
	virtual IVM::Value IVMCallMethod(const std::string & funcname,
							 const std::vector<IVM::Value> & parameters);

private:

	Image2D * m_pParentImage;

    int m_iWidth;
    int m_iHeight;

	int m_iCellX;
	int m_iCellY;

	Image2D::Flags m_DrawFlags;

	enum FuncType
    {
		FUNC_INIT,
		FUNC_DRAW,
		FUNC_DRAW3D,
		FUNC_GETWIDTH,
		FUNC_GETHEIGHT
    };
	
    static std::map<std::string,FuncType> m_ExportTable;
    static void FillExportTable();
};

} // namespace Isis

#endif // ISIS_GAME_CELLIMAGE_H_INCLUDED
