
#ifndef ISIS_GAME_FONT2D_H_INCLUDED
#define ISIS_GAME_FONT2D_H_INCLUDED 1

#include <vector>
#include <string>

#include <isis/ivm.h>

namespace Isis {

class Image2D;
class GraphicsEngine;

class Font2D : public IVM::IVMObject
{
public:

	Font2D(GraphicsEngine * pGraphicsEngine);
	virtual ~Font2D();

	void Init(const std::string & sIGZFileName);

	void Init(const std::vector<Image2D *> & vecCharacters);

	void DrawString(const std::string & sString, int x, int y,
					unsigned int modColor = 0xFFFFFFFF);

	void DrawString(const std::string & sString, int x, int y,
					int iBoundWidth, int iBoundHeight,
					unsigned int modColor = 0xFFFFFFFF);

	void DrawStringUp(const std::string & sString, int x, int y,
					  unsigned int modColor = 0xFFFFFFFF);
	void DrawStringDown(const std::string & sString, int x, int y,
					    unsigned int modColor = 0xFFFFFFFF);


	void CalcTextSize(const std::string & sString, int & iWidth, 
					  int & iHeight);
	int CalcNumLines(const std::string & sString, int iLineWidth);
	std::string ChopLines(const std::string & sString, int NumLines,
				   int iLineWidth);

	virtual void IVMFillExportTable(std::vector<std::string> & table);
	virtual bool IVMContainsMethod(const std::string & funcname);
    virtual IVM::Value IVMCallMethod(const std::string & funcname, 
		const std::vector<IVM::Value> & parameters);

#ifdef _DEBUG
	static int GetGlobalRefCount();
#endif // _DEBUG

private:

	std::vector<Image2D *> m_Characters;
	GraphicsEngine * m_pGraphicsEngine;

	enum FuncType
    {
		FUNC_INIT,
		FUNC_DRAWSTRING,
		FUNC_DRAWSTRINGUP,
		FUNC_DRAWSTRINGDOWN,
		FUNC_CALCTEXTSIZE,
		FUNC_CALCNUMLINES,
		FUNC_CHOPLINES
    };
	
    static std::map<std::string,FuncType> m_ExportTable;
    static void FillExportTable();

#ifdef _DEBUG
	static int m_iGlobalRefCount;
#endif // _DEBUG

};

} // namespace Isis

#endif // ISIS_GAME_FONT2D_H_INCLUDED

