
#ifndef ISIS_GAME_GUIGAME_H_INCLUDED
#define ISIS_GAME_GUIGAME_H_INCLUDED

#include <string>
#include <map>
#include <vector>
#include <list>

#include <isis/ivm.h>

#include <isis/game/GameApp.h>
#include <isis/game/IWindowContainer.h>

namespace Isis {

class Font2D;
class GUIWindow;

class GUIGame : public GameApp, 
				public IVM::ObjectProvider,
				public IWindowContainer
{
public:

    GUIGame();
    virtual ~GUIGame();

	int GetMouseX();
	int GetMouseY();

	void PushFont(Font2D * pFont);
	void PopFont();
	Font2D * GetDefaultFont();

	virtual GUIWindow * MakeWindow(int x, int y,
								   int width = 0, int height = 0);
	virtual GUIWindow * WindowFromPoint(int x, int y,
										bool bCountTransparent = false);
	virtual void RemoveWindow(GUIWindow * pWindow);
	virtual void DestroyAllWindows();
	virtual void WindowToScreen(int & x, int & y);
	virtual void ScreenToWindow(int & x, int & y);

	void SetKeyboardFocus(GUIWindow * pWindow);
	GUIWindow * GetKeyboardFocus();
	void SetMouseCapture(GUIWindow * pWindow);
	GUIWindow * GetMouseCapture();

	void AddExclusiveWindow(GUIWindow * pWindow);
	void RemoveExclusiveWindow(GUIWindow * pWindow);
	GUIWindow * GetExclusiveWindow();

	void AddFrameUpdater(GUIWindow * pWindow);
	void RemoveFrameUpdater(GUIWindow * pWindow);

	virtual void IVMFillExportTable(std::vector<std::string> & table);
    virtual bool IVMContainsMethod(const std::string & funcname);
    virtual IVM::Value IVMCallMethod(const std::string & funcname, 
		const std::vector<IVM::Value> & parameters);

	virtual void FillObjectTable(std::vector<std::string> & table);
    virtual IVM::IVMObject * CreateObject(const std::string & objname,
                                  const std::vector<IVM::Value> & InitParams);

#ifdef _DEBUG
	static int GetGlobalRefCount();
#endif // _DEBUG

protected:

    virtual void GameInit();
    virtual bool GameMain();
    virtual void GameShutdown();

	virtual void GUIInit() = 0;
	virtual bool GUIMain() = 0;
	virtual void GUIShutdown() = 0;

	virtual bool UnhandledException(IsisException & ex);

	virtual void KeyPressed(int key, ModifierKeys modifiers);
	virtual void KeyReleased(int key, ModifierKeys modifiers);

	virtual void MouseMoved(int x, int y);
	virtual void MouseClicked(ClickType click, int x, int y);
	virtual void EmptyClick(ClickType click, int x, int y);

	struct Mouse
	{
		int iX;
		int iY;

		int iOldX;
		int iOldY;
	} m_Mouse;

	std::list<Font2D *> m_FontStack;

	std::list<GUIWindow *> m_WindowList;
	std::list<GUIWindow *> m_ExclusiveStack;
	std::list<GUIWindow *> m_FrameUpdaters;

	GUIWindow * m_pKeyboardFocusWindow;
	GUIWindow * m_pMouseCaptureWindow;

    enum FuncType
    {
		FUNC_MAKEWINDOW,
		FUNC_REMOVEWINDOW,
		FUNC_DESTROYALLWINDOWS,
		FUNC_WINDOWTOSCREEN,
		FUNC_SCREENTOWINDOW,
		FUNC_WINDOWFROMPOINT,
		FUNC_ADDEXCLUSIVEWINDOW,
		FUNC_REMOVEEXCLUSIVEWINDOW,
		FUNC_GETEXCLUSIVEWINDOW,
		FUNC_ADDFRAMEUPDATER,
		FUNC_REMOVEFRAMEUPDATER,

		FUNC_SETKEYBOARDFOCUS,
		FUNC_GETKEYBOARDFOCUS,
		FUNC_GETMOUSECAPTURE,
		FUNC_SETMOUSECAPTURE,
        FUNC_GETMOUSEX,
        FUNC_GETMOUSEY,
		FUNC_PUSHFONT,
		FUNC_POPFONT,
		FUNC_GETDEFAULTFONT,
		
		FUNC_GETGUIGAME
    };

    static std::map<std::string,FuncType> m_ExportTable2;
    static void FillExportTable2();

    enum ObjectType
    {
        OBJECT_GUIWINDOW
    };

    static std::map<std::string,ObjectType> m_ObjectTable;
    static void FillObjectTable();

private:

#ifdef _DEBUG
	static int m_iGlobalRefCount;
#endif // _DEBUG

};

} // namepsace IGA

#endif // ISIS_GAME_GUIGAME_H_INCLUDED

