
#ifndef ISIS_GAME_GUIWINDOW_H_INCLUDED
#define ISIS_GAME_GUIWINDOW_H_INCLUDED 1

#include <isis/ivm.h>

#include <isis/game/Image2D.h>

#include <isis/game/IWindowContainer.h>

namespace Isis {

class Image2D;
class Font2D;

class GUIGame;

class IMouseHandler;
class IKeyHandler;
class ICustomDraw;
class IDestroyCallback;
class IFrameUpdate;

class GUIWindow : public IWindowContainer
{
public:

	GUIWindow(GUIGame * pGUIGame, IWindowContainer * pContainer);
	virtual ~GUIWindow();

	void Init(int x, int y, int width = 0, int height = 0);
	void Destroy();

	void Draw();
	void FrameUpdate();

	void SetX(int x);
	int GetX();
	void SetY(int y);
	int GetY();
	void SetWidth(int width);
	int GetWidth();
	void SetHeight(int height);
	int GetHeight();

	void SetTransparent(bool bTransparent);
	bool IsTransparent();
	void SetHidden(bool bHidden);
	bool IsHidden();
	void SetExclusive(bool bExclusive);
	bool IsExclusive();

	void SetColor(unsigned int color);
	unsigned int GetColor();
	void SetImage(Image2D * pImage, bool bChangeSize = false, 
				  Image2D::Flags flags = Image2D::None);
	Image2D * GetImage();
	void SetText(const std::string & sText, 
				 unsigned int modColor = 0xFF000000);
	std::string GetText();
	void SetFont(Font2D * pFont);
	Font2D * GetFont();
	void SetTextDirection(int dir);
	int GetTextDirection();
	void SetTextColor(unsigned int color);
	unsigned int GetTextColor();

	void SetMouseHandler(IMouseHandler * pMouseHandler);
	IMouseHandler * GetMouseHandler();
	void SetKeyHandler(IKeyHandler * pKeyHandler);
	IKeyHandler * GetKeyHandler();
	void SetCustomDraw(ICustomDraw * pCustomDraw);
	ICustomDraw * GetCustomDraw();
	void SetDestroyCallback(IDestroyCallback * pDestroyCallback);
	IDestroyCallback * GetDestroyCallback();
	void SetFrameUpdate(IFrameUpdate * pFrameUpdate);
	IFrameUpdate * GetFrameUpdate();
	IWindowContainer * GetContainer();
	GUIGame * GetGUIGame();

	virtual GUIWindow * MakeWindow(int x, int y, 
								   int width = 0, int height = 0);
	virtual GUIWindow * WindowFromPoint(int x, int y,
										bool bCountTransparent = false);
	virtual void RemoveWindow(GUIWindow * pWindow);
	virtual void WindowToScreen(int & x, int & y);
	virtual void ScreenToWindow(int & x, int & y);

	virtual void IVMFillExportTable(std::vector<std::string> & table);
	virtual bool IVMContainsMethod(const std::string & funcname);
    virtual IVM::Value IVMCallMethod(const std::string & funcname, 
		const std::vector<IVM::Value> & parameters);

#ifdef _DEBUG
	static int GetGlobalRefCount();
#endif // _DEBUG

private:

	GUIGame * m_pGUIGame;
	IWindowContainer * m_pContainer;

	std::list<GUIWindow *> m_WindowList;

	int m_iX;
	int m_iY;
	int m_iWidth;
	int m_iHeight;

	bool m_bTransparent;
	bool m_bHidden;
	bool m_bExclusive;

	unsigned int m_Color;
	
	Image2D * m_pImage;
	Image2D::Flags m_ImageFlags;

	std::string m_sText;
	unsigned int m_TextColor;
	Font2D * m_pFont;
	int m_iTextDirection;

	IMouseHandler * m_pMouseHandler;
	IKeyHandler * m_pKeyHandler;
	ICustomDraw * m_pCustomDraw;
	IDestroyCallback * m_pDestroyCallback;
	IFrameUpdate * m_pFrameUpdate;

	bool m_bDestroyed;

	enum FuncType
    {
		FUNC_INIT,
		FUNC_DESTROY,

		FUNC_DRAW,
		FUNC_FRAMEUPDATE,

		FUNC_MAKEWINDOW,
		FUNC_REMOVEWINDOW,
		FUNC_WINDOWTOSCREEN,
		FUNC_SCREENTOWINDOW,
		FUNC_WINDOWFROMPOINT,

		FUNC_SETTRANSPARENT,
		FUNC_ISTRANSPARENT,
		FUNC_SETHIDDEN,
		FUNC_ISHIDDEN,
		FUNC_SETEXCLUSIVE,
		FUNC_ISEXCLUSIVE,

		FUNC_SETCOLOR,
		FUNC_GETCOLOR,
		FUNC_SETIMAGE,
		FUNC_GETIMAGE,
		FUNC_SETTEXT,
		FUNC_GETTEXT,
		FUNC_SETTEXTCOLOR,
		FUNC_GETTEXTCOLOR,
		FUNC_SETFONT,
		FUNC_GETFONT,
		FUNC_SETTEXTDIRECTION,
		FUNC_GETTEXTDIRECTION,

		FUNC_SETX,
		FUNC_GETX,
		FUNC_SETY,
		FUNC_GETY,
		FUNC_GETWIDTH,
		FUNC_SETWIDTH,
		FUNC_GETHEIGHT,
		FUNC_SETHEIGHT,
		FUNC_GETCONTAINER,

		FUNC_SETMOUSEHANDLER,
		FUNC_GETMOUSEHANDLER,
		FUNC_SETKEYHANDLER,
		FUNC_GETKEYHANDLER,
		FUNC_GETCUSTOMDRAW,
		FUNC_SETCUSTOMDRAW,
		FUNC_GETDESTROYCALLBACK,
		FUNC_SETDESTROYCALLBACK,
		FUNC_GETFRAMEUPDATE,
		FUNC_SETFRAMEUPDATE
	};
	
    static std::map<std::string,FuncType> m_ExportTable;
    static void FillExportTable();

#ifdef _DEBUG
	static int m_iGlobalRefCount;
#endif // _DEBUG

};

} // namespace Isis

#endif // ISIS_GAME_GUIWINDOW_H_INCLUDED

