
#ifndef ISIS_GAME_GAMEAPP_H_INCLUDED
#define ISIS_GAME_GAMEAPP_H_INCLUDED

#include <string>
#include <map>
#include <vector>

#include <isis/ivm.h>

namespace Isis {

class GraphicsEngine;
class SoundEngine;

class GameApp : public virtual IVM::IVMObject
{
public:

	enum ModifierKeys
	{
		NoKey	= 0x0,
		Alt		= 0x1,
		Control	= 0x2,
		Shift	= 0x4,
		Meta	= 0x8
	};

	enum ClickType
	{
		NonClick	= 0x00,
		LeftDown	= 0x01,
		MiddleDown	= 0x02,
		RightDown	= 0x04,
		LeftUp		= 0x08,
		MiddleUp	= 0x10,
		RightUp		= 0x20
	};

    GameApp();
    virtual ~GameApp();

    void Create();
    void Run();

	GraphicsEngine * GetGraphicsEngine();
	void SetWindowTitle(const std::string & sWindowTitle);

	bool CheckModifier(ModifierKeys key);

    virtual void IVMFillExportTable(std::vector<std::string> & table);
    virtual bool IVMContainsMethod(const std::string & funcname);
    virtual IVM::Value IVMCallMethod(const std::string & funcname, 
		const std::vector<IVM::Value> & parameters);

protected:

    virtual void GameInit() = 0;
    virtual bool GameMain() = 0;
    virtual void GameShutdown() = 0;

	virtual bool UnhandledException(IsisException & ex);

    GraphicsEngine * m_pGraphicsEngine;
	SoundEngine * m_pSoundEngine;

    int m_iFrameRate;
    bool m_bUseFrameRate;

	std::string m_sWindowTitle;

	ModifierKeys m_CurModifiers;

	virtual void KeyPressed(int key, ModifierKeys modifiers);
	virtual void KeyReleased(int key, ModifierKeys modifiers);

	virtual void MouseMoved(int x, int y);
	virtual void MouseClicked(ClickType click, int x, int y);

    enum FuncType
    {
		FUNC_INIT,
        FUNC_CREATE,
        FUNC_RUN,
		FUNC_CHECKMODIFIER,
		FUNC_GETGRAPHICSENGINE,
		FUNC_SETWINDOWTITLE
    };

    static std::map<std::string,FuncType> m_ExportTable;
    static void FillExportTable();

};

} // namepsace IGA

#endif // ISIS_GAME_GAMEAPP_H_INCLUDED

