
#ifndef ISIS_GAME_PARTICLEMANAGER_H_INCLUDED
#define ISIS_GAME_PARTICLEMANAGER_H_INCLUDED 1

#include <set>

#include <isis/ivm.h>
#include <isis/game.h>

namespace Isis {

class ParticleSource;

class ParticleManager : public IVM::IVMObject
{
public:

	ParticleManager();
	virtual ~ParticleManager();

	void Init();
	void AddSource(ParticleSource * pSource);

	void Update();
	void Draw(Isis::GraphicsEngine * pGraphicsEngine);

	virtual void IVMFillExportTable(std::vector<std::string> & table);
	virtual bool IVMContainsMethod(const std::string & funcname);
    virtual IVM::Value IVMCallMethod(const std::string & funcname, 
		const std::vector<IVM::Value> & parameters);

#ifdef _DEBUG
	static int GetGlobalRefCount();
#endif // _DEBUG

private:

	struct Particle
	{
		ParticleSource * source;
		int lifetime;
		int counter;
		float size;
		float x, y;
		float xv, yv;
		float xa, ya;
	};

	std::set<ParticleSource *> m_Sources;
	std::set<Particle *> m_Particles;

	unsigned int m_uiListID;

	void NewParticle(ParticleSource * source);

	enum FuncType
    {
		FUNC_INIT,
		FUNC_ADDSOURCE,
		FUNC_UPDATE,
		FUNC_DRAW
    };
	
    static std::map<std::string,FuncType> m_ExportTable;
    static void FillExportTable();

#ifdef _DEBUG
	static int m_iGlobalRefCount;
#endif // _DEBUG

};

} // namespace Isis

#endif // ISIS_GAME_PARTICLEMANAGER_H_INCLUDED

