
#ifndef ISIS_GAME_PARTICLESOURCE_H_INCLUDED
#define ISIS_GAME_PARTICLESOURCE_H_INCLUDED 1

#include <isis/ivm.h>

namespace Isis {

class ParticleSource : public IVM::IVMObject
{
public:

	ParticleSource();
	virtual ~ParticleSource();

	struct Desc
	{
		int expire_time;
		bool active;
		float x, y;
		float xv, yv;
		float xa, ya;
		float p_prob;
		int p_life_min, p_life_max;
		float p_size_min, p_size_max;
		float p_xv_min, p_xv_max;
		float p_yv_min, p_yv_max;
		float p_xa, p_ya;
		std::vector<std::pair<unsigned int, int> > colors;
	};

	void Init(const Desc & desc);

	Desc & GetDesc();

	void Stop();
	void Start();

	unsigned int CalcColor(int time);

	void Destroy();

	virtual void IVMFillExportTable(std::vector<std::string> & table);
	virtual bool IVMContainsMethod(const std::string & funcname);
    virtual IVM::Value IVMCallMethod(const std::string & funcname, 
		const std::vector<IVM::Value> & parameters);

#ifdef _DEBUG
	static int GetGlobalRefCount();
#endif // _DEBUG

private:
	
	Desc m_Desc;

	enum FuncType
    {
		FUNC_INIT,
		FUNC_STOP,
		FUNC_START,
		FUNC_DESTROY,
		FUNC_CALCCOLOR,
		FUNC_SETX,
		FUNC_SETY,
		FUNC_GETX,
		FUNC_GETY,
		FUNC_SETACTIVE
    };
	
    static std::map<std::string,FuncType> m_ExportTable;
    static void FillExportTable();

#ifdef _DEBUG
	static int m_iGlobalRefCount;
#endif // _DEBUG

};

} // namespace Isis

#endif // ISIS_GAME_PARTICLESOURCE_H_INCLUDED

