
#ifndef ISIS_GAME_SOUND_H_INCLUDED
#define ISIS_GAME_SOUND_H_INCLUDED 1
#include <vector>
#include <string>

#include <isis/ivm.h>
#include <SDL/SDL_mixer.h>
namespace Isis {

class SoundEngine;

class Sound : public IVM::IVMObject
{

public:

	Sound(SoundEngine * pSoundEngine);
	virtual ~Sound();

	void Init(const std::string & sWAVEFileName);
	void Play();
	void SetVolume(int Volume=MIX_MAX_VOLUME/2);
	int GetVolume();

	Mix_Chunk * GetChunk();
	std::string GetName();
	virtual void IVMFillExportTable(std::vector<std::string> & table);
	virtual bool IVMContainsMethod(const std::string & funcname);
    virtual IVM::Value IVMCallMethod(const std::string & funcname, 
						const std::vector<IVM::Value> & parameters);

#ifdef _DEBUG
	static int GetGlobalRefCount();
#endif // _DEBUG

private:
	std::string m_sFileName;
	SoundEngine * m_pSoundEngine;
	Mix_Chunk *m_pSound;

	int m_iVolume;

	enum FuncType
    {
		FUNC_INIT,
		FUNC_PLAY,
		FUNC_SETVOLUME,
		FUNC_GETVOLUME
    };
	
    static std::map<std::string,FuncType> m_ExportTable;
    static void FillExportTable();

#ifdef _DEBUG
	static int m_iGlobalRefCount;
#endif // _DEBUG

};

} // namespace Isis

#endif //end ISIS_GAME_SOUND_H_INCLUDED

