
#ifndef ISIS_GAME_SOUNDENGINE_H_INCLUDED
#define ISIS_GAME_SOUNDENGINE_H_INCLUDED 1

#include <map>
#include <string>
#include <list>
#include <vector>

#include <isis/ivm.h>
#include <isis/game/Sound.h>

namespace Isis {
 
 class SoundEngine : public IVM::IVMObject, 
					   public IVM::ObjectProvider
{
public:

    SoundEngine();
    virtual ~SoundEngine();

	void Destroy();

    virtual void IVMFillExportTable(std::vector<std::string> & table);
    virtual bool IVMContainsMethod(const std::string & funcname);
    virtual IVM::Value IVMCallMethod(const std::string & funcname, 
		const std::vector<IVM::Value> & parameters);

	virtual void FillObjectTable(std::vector<std::string> & table);
    virtual IVM::IVMObject * CreateObject
		(const std::string & objname,
        const std::vector<IVM::Value> & InitParams);
	//Initialize Audio device
    void InitAudio(); 
	//Load+convert sound for playing
	Sound * LoadSound(const std::string  & sWAVEFilename); 
	
	//Play previously loaded sound Repeat+1 times -1 is infinate
	void Play(Isis::Sound *pSound,int iRepeat=0);
	
	void SetSoundVolume(int Volume=MIX_MAX_VOLUME/2);
	int GetSoundVolume();

	void SetMusicVolume(int Volume=MIX_MAX_VOLUME/2);
	int GetMusicVolume();

	//Load and play a music file with hook when finished
	void PlayMusic(const std::string  & sWAVEFilename,
					void (*p_MusicFinished)()=NULL);
	
	//Run this function on completion of music NULL disables
	void HookMusicFinished(void (*p_MusicFinished)()); 

	void FreeChunk(Mix_Chunk *p_Chunk);
#ifdef _DEBUG
	static int GetGlobalRefCount();
#endif // _DEBUG

private:

    enum FuncType
    {
        FUNC_INITAUDIO,
		FUNC_LOADSOUND,
		FUNC_PLAY,
		FUNC_PLAYMUSIC,
		FUNC_HOOKMUSICFINISHED,
		FUNC_SETSOUNDVOLUME,
		FUNC_GETSOUNDVOLUME,
		FUNC_SETMUSICVOLUME,
		FUNC_GETMUSICVOLUME
		
    };



    static std::map<std::string,FuncType> m_ExportTable;
    static void FillExportTable();


    enum ObjectType
    {
        OBJECT_SOUND
		
    };

	static std::map<std::string,ObjectType> m_ObjectTable;
    static void FillObjectTable();

	bool m_bDestroyed;
	
	std::vector<std::string> m_SoundPrefixes;
	
	int m_iAudioRate;
	Uint16 m_AudioFormat; 
	int m_iAudioChannels;
	int m_iAudioBuffers;
	int m_iMixingChannels;

	int m_iMusicVolume;
	int m_iSoundVolume;

	static void m_MusicFinished();
	
	//Keep an list of pointers to what's on what channel
	//We could do with with GetPlaying, but this 
	//std::list<Sound *> *m_ChannelsList; 

	Sound *m_Music;
	// std::vector<Sound> *m_Sounds;
	


#ifdef _DEBUG
	static int m_iGlobalRefCount;
#endif // _DEBUG

};

} // namespace Isis

#endif // ISIS_GAME_SOUNDENGINE_H_INCLUDED

