
#ifndef ISIS_GAME_TEXTURE_H_INCLUDED
#define ISIS_GAME_TEXTURE_H_INCLUDED

#include <string>
#include <map>
#include <vector>

#include <isis/ivm.h>

#include <isis/game/Image2D.h>
#include <isis/game/VRAMObject.h>

typedef void * gzFile;

namespace Isis {

class GraphicsEngine;

class Texture : public Image2D, public VRAMObject
{
public:

	Texture(GraphicsEngine * pGraphicsEngine);
	virtual ~Texture();

	void Init(const std::string & sFilename,
			  std::vector<Image2D*> & SubImages);
	void Init(unsigned char * pData, int width, int height);

    void LoadVRAM();
    void UnloadVRAM();

	unsigned int GetID();

	unsigned int GetData(int x, int y);

	virtual void Draw(int iX = 0, int iY = 0, 
		              int iWidth = 0, int iHeight = 0,
					  Flags DrawFlags = None,
					  unsigned int modColor = 0xFFFFFFFF,
					  int iCellX = 0, int iCellY = 0, 
					  int iCellWidth = 0, int iCellHeight = 0);

	virtual void Draw3D(int iX = 0, int iY = 0, 
		                int iWidth = 0, int iHeight = 0,
					    Flags DrawFlags = None,
					    unsigned int modColor = 0xFFFFFFFF,
					    int iCellX = 0, int iCellY = 0, 
					   int iCellWidth = 0, int iCellHeight = 0);

	virtual int GetWidth();
	virtual int GetHeight();

	virtual void IVMFillExportTable(std::vector<std::string> & table);
	virtual bool IVMContainsMethod(const std::string & funcname);
    virtual IVM::Value IVMCallMethod(const std::string & funcname, 
		const std::vector<IVM::Value> & parameters);

#ifdef _DEBUG
	static int GetGlobalRefCount();
#endif // _DEBUG

private:

	unsigned int m_uiID;
	GraphicsEngine * m_pGraphicsEngine;

    unsigned char * m_pData;
	int m_iWidth;
	int m_iHeight;

	enum FuncType
    {
		FUNC_INIT,
		FUNC_LOADVRAM,
		FUNC_UNLOADVRAM,
		FUNC_GETID,
		FUNC_DRAW,
		FUNC_DRAW3D,
		FUNC_GETWIDTH,
		FUNC_GETHEIGHT
    };
	
    static std::map<std::string,FuncType> m_ExportTable;
    static void FillExportTable();

	int ReadInt(gzFile File);

	std::string m_sFilename;

#ifdef _DEBUG
	static int m_iGlobalRefCount;
#endif // _DEBUG
};

} // namespace Isis

#endif // ISIS_GAME_TEXTURE_H_INCLUDED
