
#ifndef ISIS_GAME_VECTORANIMATION_H_INCLUDED
#define ISIS_GAME_VECTORANIMATION_H_INCLUDED

#include <string>
#include <map>
#include <vector>

#include <isis/ivm.h>

typedef void * gzFile;

namespace Isis {

class GraphicsEngine;
class VectorImage;

class VectorAnimation : public IVM::IVMObject
{
public:

	VectorAnimation();
	virtual ~VectorAnimation();

	static void LoadVGZ(GraphicsEngine * pGraphicsEngine,
						const std::string & sImageFilename,
						std::vector<VectorAnimation*> & animations);

	int GetNumFrames();
	VectorImage * GetFrame(int index);
	float GetFramerate();
	std::string GetName();

	virtual void IVMFillExportTable(std::vector<std::string> & table);
	virtual bool IVMContainsMethod(const std::string & funcname);
    virtual IVM::Value IVMCallMethod(const std::string & funcname, 
		const std::vector<IVM::Value> & parameters);

#ifdef _DEBUG
	static int GetGlobalRefCount();
#endif // _DEBUG

private:

	std::vector<VectorImage*> m_Frames;
	std::string m_sName;
	float m_fFramerate;

	enum FuncType
    {
		FUNC_GETNUMFRAMES,
		FUNC_GET,
		FUNC_GETFRAME,
		FUNC_GETFRAMERATE,
		FUNC_GETNAME
    };
	
    static std::map<std::string,FuncType> m_ExportTable;
    static void FillExportTable();

	void Init(GraphicsEngine * pGraphicsEngine, gzFile file);

	static int ReadInt(gzFile File);
	static float ReadFloat(gzFile File);
	static std::string ReadString(gzFile File);

#ifdef _DEBUG
	static int m_iGlobalRefCount;
#endif // _DEBUG
};

} // namespace Isis

#endif // ISIS_GAME_VECTORANIMATION_H_INCLUDED
