
#ifndef ISIS_GAME_VECTORIMAGE_H_INCLUDED
#define ISIS_GAME_VECTORIMAGE_H_INCLUDED

#include <string>
#include <map>
#include <vector>

#include <isis/ivm.h>

#include <isis/game/VRAMObject.h>

typedef void * gzFile;

namespace Isis {

class GraphicsEngine;

class VectorImage : public VRAMObject, public IVM::IVMObject
{
public:

	VectorImage(GraphicsEngine * pGraphicsEngine);
	virtual ~VectorImage();

	void Init(gzFile file);

    void LoadVRAM();
    void UnloadVRAM();

	void Draw(float x = 0, float y = 0, float z = 0, float angle = 0,
		      float scale_x = 1, float scale_y = 0);
	float GetLeft();
	float GetRight();
	float GetTop();
	float GetBottom();

	virtual void IVMFillExportTable(std::vector<std::string> & table);
	virtual bool IVMContainsMethod(const std::string & funcname);
    virtual IVM::Value IVMCallMethod(const std::string & funcname, 
		const std::vector<IVM::Value> & parameters);

#ifdef _DEBUG
	static int GetGlobalRefCount();
#endif // _DEBUG

private:

	GraphicsEngine * m_pGraphicsEngine;

	unsigned int m_uiID;

	enum RecordType
	{
		ColorRecord = 0,
		PolyRecord = 1,
		EndRecord = 2,
		PolyPolyRecord = 3
	};

	struct Record
	{
		RecordType type;
		int index;
	};

	struct Color
	{
		unsigned char red;
		unsigned char green;
		unsigned char blue;
	};

	struct Point
	{
		float x,y;
	};

	struct Polygon
	{
		std::vector<Point> points;
	};

	struct PolyPolygon
	{
		std::vector<Polygon> polygons;
	};

	std::vector<Record> m_Records;
	std::vector<Polygon> m_Polygons;
	std::vector<PolyPolygon> m_PolyPolygons;
	std::vector<Color> m_Colors;

	float m_fLeft;
	float m_fRight;
	float m_fTop;
	float m_fBottom;

	enum FuncType
    {
		FUNC_DRAW,
		FUNC_GETLEFT,
		FUNC_GETRIGHT,
		FUNC_GETTOP,
		FUNC_GETBOTTOM
    };

    static std::map<std::string,FuncType> m_ExportTable;
    static void FillExportTable();

	static int ReadInt(gzFile File);
	static unsigned char ReadByte(gzFile File);
	static float ReadFloat(gzFile File);

#ifdef _DEBUG
	static int m_iGlobalRefCount;
#endif // _DEBUG
};

} // namespace Isis

#endif // ISIS_GAME_VECTORIMAGE_H_INCLUDED
