
#ifndef IVM_DATASET_H_INCLUDED
#define IVM_DATASET_H_INCLUDED 1

#include <map>

#include <isis/ivm/Value.h>
#include <isis/ivm/IVMObject.h>
#include <isis/ivm/ISaveLoad.h>

namespace IVM {

class DataSet : public IVMObject
{
public:

    DataSet();
    ~DataSet();

    virtual void IVMFillExportTable(std::vector<std::string> & table);
    virtual bool IVMContainsMethod(const std::string & funcname);
    virtual IVM::Value IVMCallMethod(const std::string & funcname, 
                               const std::vector<Value> & parameters);

	Value Get(const std::string & sFieldName);
	void Set(const std::string & sFieldName, Value value);
    void Clear();
    int Size();

    void Save(ISaveLoad * pSaveLoad);
    void Load(ISaveLoad * pSaveLoad);

#ifdef _DEBUG
	static int GetGlobalRefCount();
#endif // _DEBUG

private:

    enum FuncType
    {
        FUNC_INIT,
        FUNC_GET,
		FUNC_SET,
		FUNC_SIZE,
        FUNC_CLEAR,
        FUNC_SAVE,
        FUNC_LOAD
    };

    static std::map<std::string,FuncType> m_ExportTable;
    static void FillExportTable();

	std::map<std::string,Value> m_Data;

#ifdef _DEBUG
	static int m_iGlobalRefCount;
#endif // _DEBUG

};

} // namespace IVM

#endif // IVM_DATASET_H_INCLUDED 

