
#ifndef IVM_IVMVECTOR_H_INCLUDED
#define IVM_IVMVECTOR_H_INCLUDED 1

#include <vector>

#include <isis/ivm/Value.h>
#include <isis/ivm/IVMObject.h>
#include <isis/ivm/ISaveLoad.h>

namespace IVM {

class IVMVector : public IVMObject
{
public:

    IVMVector();
    ~IVMVector();

    virtual void IVMFillExportTable(std::vector<std::string> & table);
    virtual bool IVMContainsMethod(const std::string & funcname);
    virtual IVM::Value IVMCallMethod(const std::string & funcname, 
                               const std::vector<Value> & parameters);

    void Add(Value value);
    Value Get(int index);
	void Set(int index, Value val);
    void DelValue(Value value);
    void DelIndex(int index);
    void Clear();
    int Size();

    void Save(ISaveLoad * pSaveLoad);
    void Load(ISaveLoad * pSaveLoad);

#ifdef _DEBUG
	static int GetGlobalRefCount();
#endif // _DEBUG

private:

    enum FuncType
    {
        FUNC_INIT,
        FUNC_ADD,
        FUNC_GET,
        FUNC_SET,
        FUNC_DELVALUE,
        FUNC_DELINDEX,
        FUNC_CLEAR,
        FUNC_SIZE,
        FUNC_SAVE,
        FUNC_LOAD
    };

    static std::map<std::string,FuncType> m_ExportTable;
    static void FillExportTable();

    std::vector<Value> m_Vector;

#ifdef _DEBUG
	static int m_iGlobalRefCount;
#endif // _DEBUG

};

} // namespace IVM

#endif // IVM_IVMVECTOR_H_INCLUDED 

