
#ifndef IVM_SCRIPT_H_INCLUDED
#define IVM_SCRIPT_H_INCLUDED

#include <string>
#include <vector>
#include <map>

#include <isis/util.h>

#include <isis/ivm/IVMObject.h>
#include <isis/ivm/ObjectProvider.h>
#include <isis/ivm/Value.h>

namespace IVM {

class IsisVM;
struct ISEStruct;

class Script : public IVMObject, public ObjectProvider
{
public:

    Script();
    virtual ~Script();

    void Load(IsisVM * pParentVM, const std::string & filename);
    IsisVM * GetParentVM();

    bool IsLoaded();

    bool ContainsFunction(const std::string & funcname);

    void ExecSync(const std::string & funcname, 
                  const std::vector<Value> & parameters);
    bool ExecNextInstruction();

    void SetRetVal(const Value & retval);
    Value GetRetVal();

    virtual void IVMFillExportTable(std::vector<std::string> & table);
    virtual bool IVMContainsMethod(const std::string & funcname);
    virtual IVM::Value IVMCallMethod(const std::string & funcname, 
                               const std::vector<Value> & parameters);

    virtual void FillObjectTable(std::vector<std::string> & table);
    virtual IVMObject * CreateObject(const std::string & objname,
                                     const std::vector<Value> & InitParams);

	void DebugPrint();
    void DebugPrintInstructionTable();
    void DebugPrintStringTable();
    void DebugPrintFunctionTable();
    void DebugPrintHostAPITable();
    void DebugPrintHostObjTable();

#ifdef _DEBUG
	static int GetGlobalRefCount();
#endif // _DEBUG

private:

    enum FuncType
    {
        FUNC_ISLOADED,
        FUNC_CONTAINSFUNCTION,
        FUNC_EXECNEXTINSTRUCTION,
        FUNC_SETRETVAL,
        FUNC_GETRETVAL,
        FUNC_GETPARENTVM,
        FUNC_DEBUGPRINT,
		FUNC_CALL
    };

    bool m_bLoaded;

    ISEStruct * m_pISE;

    int m_iCurrStackFrame;
    int m_iNextInstr;
    int m_iCurrInstr;
    std::vector<int> m_NextInstrStack;
    std::vector<Value> m_Stack;
    Value m_RetVal;

    IsisVM * m_pParentVM;

    static std::map<std::string,FuncType> m_ExportTable;
    static void FillExportTable();

    void Reset();
    int FindFuncIndex(const std::string & funcname);

    Value GetOpAsRValue(int opnum);
    void SetOpAsLValue(const Value & value, int opnum);
    int GetOpAsInstrNum(int opnum);
    int GetOpAsFuncIndex(int opnum);
    int GetOpAsHostFuncIndex(int opnum);
    int GetOpAsHostObjIndex(int opnum);
    IVMObject * GetOpAsHostObjPtr(int opnum);

    void CallFunction(int funcindex, const Value & num_params);
    void CallHostFunction(int funcindex, const Value & num_params);
    void CallHostObject(IVMObject * object, int funcindex, 
                        const Value & num_params);
    bool Return();

    void Push(const Value & val);
    Value Pop();

#ifdef _DEBUG
	static int m_iGlobalRefCount;
#endif // _DEBUG

};

} // namespace IVM

#endif // IVM_SCRIPT_H_INCLUDED

