
#ifndef IVM_VALUE_H_INCLUDED
#define IVM_VALUE_H_INCLUDED

#include <string>

namespace IVM {

class IVMObject;

class Value
{
public:

    enum ValueType
    {
        None,
        Integer,
        Float,
        String,
        ObjPtr
    };
    
    Value();
    Value(int value);
    Value(float value);
    Value(const std::string & value);
    Value(IVMObject * object);
    Value(const Value & v);

    ~Value();

    Value & operator=(const Value & v);

    ValueType GetType() const;

    void AssignNone();
    void AssignInteger(int value);
    void AssignFloat(float value);
    void AssignString(const std::string & value);
    void AssignObjPtr(IVMObject * value);

    int GetInteger() const;
    float GetFloat() const;
    std::string GetString() const;
    IVMObject * GetObjPtr() const;
    
    void Free();

    void DebugPrint() const;

#ifdef _DEBUG
	static int GetGlobalRefCount();
#endif // _DEBUG

private:

    ValueType Type;

    union
    {
        int iValue;
        float fValue;
        std::string * psValue;
        IVMObject * pObject;
    };

#ifdef _DEBUG
	static int m_iGlobalRefCount;
#endif // _DEBUG

};

bool operator==(const Value & left, const Value & right);
bool operator!=(const Value & left, const Value & right);
bool operator>(const Value & left, const Value & right);
bool operator>=(const Value & left, const Value & right);
bool operator<(const Value & left, const Value & right);
bool operator<=(const Value & left, const Value & right);

const Value operator+(const Value & left, const Value & right);
const Value operator-(const Value & left, const Value & right);
const Value operator*(const Value & left, const Value & right);
const Value operator/(const Value & left, const Value & right);
const Value operator%(const Value & left, const Value & right);
const Value operator-(const Value & value);

const Value operator&&(const Value & left, const Value & right);
const Value operator||(const Value & left, const Value & right);
const Value operator&(const Value & left, const Value & right);
const Value operator|(const Value & left, const Value & right);
const Value operator^(const Value & left, const Value & right);
const Value operator!(const Value & value);
const Value operator<<(const Value & left, const Value & right);
const Value operator>>(const Value & left, const Value & right);

} // namespace IVM

#endif // IVM_VALUE_H_INCLUDED

