
#ifndef ISASM_INSTRUCTIONS_H_INCLUDED
#define ISASM_INSTRUCTIONS_H_INCLUDED 1

#include "isasm_types.h"
#include "isasm_assembler.h"

#ifdef __cplusplus
extern "C" {
#endif // __cplusplus

// functions

void reset_instructions_state(void);


void emit_instr_lvalrval(INSTRUCTION_TYPE type, LVAL * lval, RVAL * rval);
void emit_instr_lval(INSTRUCTION_TYPE type, LVAL * lval);
void emit_instr_rval(INSTRUCTION_TYPE type, RVAL * rval);
void emit_instr_lvalrvalrval(INSTRUCTION_TYPE type, LVAL * lval, RVAL * rval1,
                              RVAL * rval2);

void emit_instr_jmp(int name_index);
void emit_instr_jz(int name_index, RVAL * rval);
void emit_instr_jnz(int name_index, RVAL * rval);

void emit_instr_call(int func_name_index, RVAL * num_params);
void emit_instr_callhost(int func_name_index, RVAL * num_params);
void emit_instr_callobj(LVAL * object, int func_name_index, RVAL * num_params);
void emit_instr_this(void);
void emit_instr_ret(void);
	

OPERAND_TYPE lval_op_type(LVAL * lval);
OPERAND_TYPE rval_op_type(RVAL * rval);

int calc_symbol_size(int symbol_num);
int calc_stack_index(int symbol_num);

void write_lval(LVAL * lval, unsigned char * dest, int * count);
void write_rval(RVAL * rval, unsigned char * dest, int * count);
void write_int(int val, unsigned char * dest, int * count);
void write_float(float val, unsigned char * dest, int * count);

int get_int_from_stream(int base_byte);
void set_int_to_stream(int base_byte, int value);
void add_instruction_bytes(unsigned char * bytes, int num_bytes);
int get_next_instr_byte(void);

void mark_jump_target(int offset);
void resolve_label_jumps(void);

void mark_call_target(int offset);
void mark_callhost_target(int offset, int name_index);
void mark_callobj_target(int offset, int name_index);
void resolve_call_targets(void);
void resolve_callhost_targets(void);
void resolve_callobj_targets(void);
void add_callhost_index(int name_index);
void add_callobj_index(int name_index);
int find_callhost_index(int name_index);
int find_callobj_index(int name_index);


#ifdef __cplusplus
}
#endif

#endif // INSTRUCTIONS_H_INCLUDED

