
#ifndef ISC_COMPILER_H_INCLUDED 
#define ISC_COMPILER_H_INCLUDED 1

#include <stdio.h>

#include "isc_types.h"

#ifdef __cplusplus
extern "C" {
#endif // __cplusplus

// functions

void reset_compiler_state(void);

int isc_name_table_find(const char * string);
int isc_name_table_add(const char * string);
int isc_name_table_contains(const char * string);
const char * isc_name_table_get(int name_index);

int isc_new_func(int name_index);
int isc_find_func(int name_index);
void isc_add_param(int name_index);
void isc_add_local(int name_index, int size);
void isc_add_assigned(int name_index);

void add_global(int name_index, int size);

int is_global_scalar(int name_index);
int is_param(int name_index);
int is_local_scalar(int name_index);

void done_expr(EXPR_STRUCT * expr);
void done_reg(int reg_num);
int find_free_reg();

void emit_push_expr(EXPR_STRUCT * expr);

int emit_reg_expression(EXPR_STRUCT * expr);
int emit_1op(INSTRUCTION_TYPE instr, EXPR_STRUCT * expr);
int emit_2op(INSTRUCTION_TYPE instr, EXPR_STRUCT * expr1, EXPR_STRUCT * expr2);
void emit_assignment(EXPR_STRUCT * lval, EXPR_STRUCT * expr);
void emit_opequals(INSTRUCTION_TYPE insr, EXPR_STRUCT * lval, 
                   EXPR_STRUCT * expr);
void emit_inc(EXPR_STRUCT * expr);
void emit_dec(EXPR_STRUCT * expr);

int emit_call(int name_index, int num_params);
int emit_callobj(EXPR_STRUCT * lval, int name_index, int num_params);

void emit_return(EXPR_STRUCT * expr);

int emit_this_call();
int emit_new_call(int name_index, int num_params);

int make_label();
void insert_label(int label_num);
void insert_jmp(int label_num);
void insert_jz(EXPR_STRUCT * expr, int label_num);
void insert_jnz(EXPR_STRUCT * expr, int label_num);
void push_loop(LOOP_STRUCT * loop);
void pop_loop();
int get_continue_target();
int get_break_target();
void push_for();
void end_forstart();
void pop_for();

void fixup_ism();
void isc_clear_func_symbols();

#ifdef __cplusplus

}

std::string instr_name(INSTRUCTION_TYPE type);
std::string reg_name(int reg_num);
std::string label_name(int reg_num);
std::string lineno_name(int reg_num);
std::string expr_text(EXPR_STRUCT * expr);

std::string line2string(const LINE_STRUCT & line);
void add_line_to_func(const LINE_STRUCT & line);
void add_line(const LINE_STRUCT & line);
void add_instr(const LINE_STRUCT & line);

void emit_instr_0op(INSTRUCTION_TYPE type);
void emit_instr_1op(INSTRUCTION_TYPE type, const std::string & op0);
void emit_instr_2op(INSTRUCTION_TYPE type, const std::string & op0, 
                    const std::string & op1);
void emit_instr_3op(INSTRUCTION_TYPE type, const std::string & op0, 
                    const std::string & op1, const std::string & op2);

#endif // __cplusplus

#endif // COMPILER_H_INCLUDED

