
#ifndef ISC_TYPES_H_INCLUDED
#define ISC_TYPES_H_INCLUDED 1

#ifdef __cplusplus

#include <vector>
#include <string>
#include <list>

extern "C" {

#endif // __cplusplus

#include <isis/ivm/iseformat.h>

// datatypes

typedef enum
{
    ASMT_EQUALS,
    ASMT_PLUSE,
    ASMT_MINUSE,
    ASMT_TIMESE,
    ASMT_DIVE,
    ASMT_MODE
} ASMT_TYPE;

typedef enum
{
    EXPR_REGISTER,
    EXPR_RVAL_SCALAR,
    EXPR_RVAL_ARRAY_INT,
    EXPR_RVAL_ARRAY_FLOAT,
    EXPR_RVAL_ARRAY_STRING,
    EXPR_RVAL_ARRAY_SCL,
    EXPR_RVAL_ARRAY_REG,
    EXPR_LIT_INTEGER,
    EXPR_LIT_FLOAT,
    EXPR_LIT_STRING
} EXPR_TYPE;

typedef struct
{
    EXPR_TYPE type;

    union
    {
        int reg_num;
        int name;
        int integer;
        float floating;
        int string_ind;
    } val;

    union
    {
        int reg;
        int name;
        int integer;
        float floating;
        int stringind;
    } extra;

} EXPR_STRUCT;

typedef struct
{
    int begin_label;
    int end_label;
    int continue_label;
} LOOP_STRUCT;

#ifdef __cplusplus

typedef enum
{
    LINE_INSTR,
    LINE_LABEL,
    LINE_LINENO
} LINE_TYPE;

typedef struct
{
    LINE_TYPE type;

    INSTRUCTION_TYPE instr;
    std::vector<std::string> ops;

    int num;
} LINE_STRUCT;

typedef struct
{
    int name_index;
    std::vector<int> param_symbols;
    std::vector<std::pair<int,int> > local_symbols;
    std::list<LINE_STRUCT> body;
} ISC_FUNCTION;

typedef struct
{
    bool start;
    std::vector<LINE_STRUCT> lines;
} FOR_STRUCT;

}
#endif // __cplusplus

#endif // TYPES_H_INCLUDED

