
#include <SDL/SDL.h>
#include <SDL/SDL_opengl.h>

#include <isis/game.h>

class IsisTest : public Isis::GameApp
{
    float m_fAngle;

public:

    IsisTest()
    {
        m_fAngle = 0.0f;

		m_sWindowTitle = "Isis Test Application";
        m_pGraphicsEngine->SetWindowed(true);
		m_pGraphicsEngine->SetWidth(800);
		m_pGraphicsEngine->SetHeight(600);
    }

    virtual ~IsisTest()
    {
    }

    void GameInit()
    {
        glMatrixMode(GL_PROJECTION);
        
        glLoadIdentity();
        gluPerspective(45,4.0f/3.0f,0.1,100);
    }

    void DrawCube() 
    {
        glBegin(GL_LINES);
        
        glColor3f(0,1,1);
        glVertex3f(-1,-1,1);
        glVertex3f(1,-1,1);
        
        glColor3f(1,1,0);
        glVertex3f(1,-1,1);
        glVertex3f(1,1,1);
        
        glColor3f(1,0,1);
        glVertex3f(1,1,1);
        glColor3f(1,0,0);
        glVertex3f(-1,1,1);
        
        glColor3f(1,0,0);
        glVertex3f(-1,1,1);
        glColor3f(1,1,0);
        glVertex3f(-1,-1,1);
        
        glColor3f(0,1,0);
        glVertex3f(-1,-1,-1);
        glVertex3f(1,-1,-1);
        
        glColor3f(1,1,0);
        glVertex3f(1,-1,-1);
        glVertex3f(1,1,-1);
        
        glColor3f(1,0,1);
        glVertex3f(1,1,-1);
        glColor3f(1,0,0);
        glVertex3f(-1,1,-1);
        
        glColor3f(1,0,0);
        glVertex3f(-1,1,-1);
        glColor3f(1,1,0);
        glVertex3f(-1,-1,-1);
        
        glColor3f(1,1,1);
        glVertex3f(-1,-1,-1);
        glVertex3f(-1,-1,1);
        
        glVertex3f(-1,1,-1);
        glVertex3f(-1,1,1);
        
        glVertex3f(1,-1,-1);
        glVertex3f(1,-1,1);
        
        glVertex3f(1,1,-1);
        glVertex3f(1,1,1);
        
        glEnd();
    }

    bool GameMain()
    {
        glClear(GL_COLOR_BUFFER_BIT);
        
        glMatrixMode(GL_MODELVIEW);
        glLoadIdentity();
        gluLookAt(10,0,0,0,0,0,0,1,0);

        glRotatef(m_fAngle,-0.2f,1.0f,0.3f);
        DrawCube();
        glRotatef(-m_fAngle,-0.2f,1.0f,0.3f);
        
        m_fAngle += 3;
        
        return true;
    }

    void GameShutdown()
    {
    }
};

int main(int argc, char * argv[])
{
    try
    {
        IsisTest it;

        it.Create();

        it.Run();
    }
    catch (Isis::IsisException & ie)
    {
        printf("Application aborted.\n");
        printf("An IsisException has occurred.\n");
        printf("\nMessage:\n%s\n",ie.GetMessage().c_str());
    }
    catch (std::exception & e)
    {
        printf("Application aborted.\n");
        printf("A C++ exception has occurred.\n");
        printf("\nMessage:\n%s\n",e.what());
    }
    catch (...)
    {
        printf("Application aborted.\n");
        printf("An unknown exception has occurred.\n");
    }

    return 0;
}

