
#include <SDL/SDL.h>
#include <SDL/SDL_opengl.h>

#ifdef WIN32
#include <windows.h>
#include <conio.h>
#endif // WIN32

#include <stdio.h>

#include <iostream>
#include <string>
#include <vector>

#include <isis/util.h>
#include <isis/ivm.h>
#include <isis/game.h>

int main(int argc, char * argv[])
{
	bool bError = false;

    try
    {
        IVM::IsisVM * vm;
        IVM::Script * script;

        vm = new IVM::IsisVM();
        vm->AddRef();

		vm->RegisterObjectProvider(new Isis::IsisGameProvider());

        std::string entry_point = "Main";
        std::vector<IVM::Value> parameters;

        if (argc < 2)
        {
            printf("%s version %s",PACKAGE,VERSION);
            printf("  --  libivm console test program\n");
            printf("    usage: %s file.ise [ entry_point [ arg1 arg2 ... ] ]\n",
               argv[0]);

            return 0;
        }

        script = vm->MakeScript(argv[1],parameters);
        script->AddRef();	
	
#ifdef INFO_MSG_ON
        script->DebugPrint();
#endif

        if (argc > 2)
        {
            entry_point = argv[2];
        }

        if (!script->ContainsFunction(entry_point))
        {
            fprintf(stderr, "Entry point %s not found in file %s\n",
                    entry_point.c_str(),argv[1]);

            script->Release();
		    vm->Release();

            return 1;
        }

        for (int i = 3; i < argc; i++)
        {
            parameters.push_back(std::string(argv[i]));
        }

        script->ExecSync(entry_point,parameters);

        script->Release();
        vm->Release();
    }
    catch (Isis::IsisException & ie)
    {
        printf("Application aborted - an IsisException has occurred.\n");
        printf("Message:\n%s\n\n",ie.GetMessage().c_str());

		bError = true;
    }
#ifndef _DEBUG
    catch (std::exception & e)
    {
        printf("Application aborted - a C++ exception has occurred.\n");
        printf("Message:\n%s\n\n",e.what());

		bError = true;
    }
    catch (...)
    {
        printf("Application aborted - an unknown exception has occurred.\n\n");

		bError = true;
	}
#endif // ndef _DEBUG

	if (bError)
	{
#ifdef WIN32
		printf("Press any key to continue...\n");
		while (!kbhit()) Sleep(10);
#endif // WIN32
	}

    return 0;
}

