
#include <isis/util.h>

#include <isis/game/GraphicsEngine.h>
#include <isis/game/CellImage.h>

namespace Isis {

std::map<std::string,CellImage::FuncType> CellImage::m_ExportTable;

CellImage::CellImage()
{
    if (m_ExportTable.empty())
        FillExportTable();

    m_iWidth = 0;
    m_iHeight = 0;
	m_iCellX = 0;
	m_iCellY = 0;
	m_DrawFlags = Image2D::None;

	m_pParentImage = NULL;
}

CellImage::~CellImage()
{
	if (m_pParentImage)
		m_pParentImage->Release();
}

void CellImage::Init(Image2D * pSourceImage, int iCellX, int iCellY, 
		  int iCellWidth, int iCellHeight, Image2D::Flags DrawFlags)
{
	if (NULL == pSourceImage)
		throw IsisException("NULL parent image");

	m_iCellX = iCellX;
	m_iCellY = iCellY;
	m_iWidth = iCellWidth;
	m_iHeight = iCellHeight;
	m_DrawFlags = DrawFlags;

	m_pParentImage = pSourceImage;
	m_pParentImage->AddRef();
}

void CellImage::Draw(int iX, int iY, 
					 int iWidth, int iHeight,
					 Image2D::Flags DrawFlags,
					 unsigned int modColor,
					 int iCellX, int iCellY,
					 int iCellWidth, int iCellHeight)
{
	if (NULL == m_pParentImage)
		throw IsisException("Not initialized in Draw()");

	m_pParentImage->Draw(iX,iY,
		(iWidth > 0 ? iWidth : m_iWidth),
		(iHeight > 0 ? iHeight : m_iHeight),
		(Image2D::Flags)((int)m_DrawFlags ^ (int)DrawFlags),
		modColor,
		m_iCellX + iCellX,
		m_iCellY + iCellY,
		(iCellWidth > 0 ? iCellWidth : m_iWidth),
		(iCellHeight > 0 ? iCellHeight : m_iHeight));
}

void CellImage::Draw3D(int iX, int iY, 
					   int iWidth, int iHeight,
					   Image2D::Flags DrawFlags,
					   unsigned int modColor,
					   int iCellX, int iCellY,
					   int iCellWidth, int iCellHeight)
{
	if (NULL == m_pParentImage)
		throw IsisException("Not initialized in Draw3D()");

	m_pParentImage->Draw3D(iX,iY,
		(iWidth > 0 ? iWidth : m_iWidth),
		(iHeight > 0 ? iHeight : m_iHeight),
		(Image2D::Flags)((int)m_DrawFlags ^ (int)DrawFlags),
		modColor,
		m_iCellX + iCellX,
		m_iCellY + iCellY,
		(iCellWidth > 0 ? iCellWidth : m_iWidth),
		(iCellHeight > 0 ? iCellHeight : m_iHeight));
}

} // namespace Isis
