
#include <isis/util.h>
#include <isis/ivm.h>

#include <isis/game/GraphicsEngine.h>
#include <isis/game/CellImage.h>

namespace Isis {

void CellImage::FillExportTable()
{
    m_ExportTable["Init"]      = FUNC_INIT;
    m_ExportTable["Draw"]      = FUNC_DRAW;
    m_ExportTable["Draw3D"]    = FUNC_DRAW3D;
    m_ExportTable["GetWidth"]  = FUNC_GETWIDTH;
    m_ExportTable["GetHeight"] = FUNC_GETHEIGHT;
}

void CellImage::IVMFillExportTable(std::vector<std::string> & table)
{
    std::map<std::string,FuncType>::iterator iter;

    for (iter = m_ExportTable.begin(); iter != m_ExportTable.end(); iter++)
    {
        table.push_back(iter->first);
    }
}

bool CellImage::IVMContainsMethod(const std::string & funcname)
{
    return m_ExportTable.end() != m_ExportTable.find(funcname);
}

IVM::Value CellImage::IVMCallMethod(const std::string & funcname,
                           const std::vector<IVM::Value> & parameters)
{   
    if (m_ExportTable.end() == m_ExportTable.find(funcname))
        throw IVM::IVMException("function not found in CellImage object",
								funcname);
    
    switch (m_ExportTable[funcname])
    {
    case FUNC_INIT:
        {
            if (parameters.size() == 5)
				Init(dynamic_cast<Image2D*>(parameters[0].GetObjPtr()),
					 parameters[1].GetInteger(),
					 parameters[2].GetInteger(),
					 parameters[3].GetInteger(),
					 parameters[4].GetInteger());
			else if (parameters.size() == 6)
				Init(dynamic_cast<Image2D*>(parameters[0].GetObjPtr()),
					 parameters[1].GetInteger(),
					 parameters[2].GetInteger(),
					 parameters[3].GetInteger(),
					 parameters[4].GetInteger(),
					 (Image2D::Flags)parameters[5].GetInteger());
			else
                throw IVM::IVMException(
                         "wrong number of parameters for Init");
        } break;
    case FUNC_GETWIDTH:
        {
            if (parameters.size() != 0)
                throw IVM::IVMException(
                         "wrong number of parameters for GetWidth");

            return GetWidth();

        } break;
    case FUNC_GETHEIGHT:
        {
            if (parameters.size() != 0)
                throw IVM::IVMException(
                         "wrong number of parameters for GetHeight");

            return GetHeight();

        } break;
    case FUNC_DRAW:
        {
            if (parameters.size() == 0)
				Draw();
			else if (parameters.size() == 2)
	            Draw(parameters[0].GetInteger(),parameters[1].GetInteger());
			else if (parameters.size() == 4)
	            Draw(parameters[0].GetInteger(),parameters[1].GetInteger(),
					 parameters[2].GetInteger(),parameters[3].GetInteger());
			else if (parameters.size() == 5)
	            Draw(parameters[0].GetInteger(),parameters[1].GetInteger(),
					 parameters[2].GetInteger(),parameters[3].GetInteger(),
					 (Image2D::Flags)parameters[4].GetInteger());
			else if (parameters.size() == 6)
	            Draw(parameters[0].GetInteger(),parameters[1].GetInteger(),
					 parameters[2].GetInteger(),parameters[3].GetInteger(),
					 (Image2D::Flags)parameters[4].GetInteger(),
					 parameters[5].GetInteger());
			else if (parameters.size() == 10)
	            Draw(parameters[0].GetInteger(),parameters[1].GetInteger(),
					 parameters[2].GetInteger(),parameters[3].GetInteger(),
					 (Image2D::Flags)parameters[4].GetInteger(),
					 parameters[5].GetInteger(),
					 parameters[6].GetInteger(),parameters[7].GetInteger(),
					 parameters[8].GetInteger(),parameters[9].GetInteger());
			else
                throw IVM::IVMException(
                         "wrong number of parameters for Draw");

            return true;

        } break;
    case FUNC_DRAW3D:
        {
            if (parameters.size() == 0)
				Draw3D();
			else if (parameters.size() == 2)
				Draw3D(parameters[0].GetInteger(),parameters[1].GetInteger());
			else if (parameters.size() == 4)
	            Draw3D(parameters[0].GetInteger(),parameters[1].GetInteger(),
					   parameters[2].GetInteger(),parameters[3].GetInteger());
			else if (parameters.size() == 5)
	            Draw3D(parameters[0].GetInteger(),parameters[1].GetInteger(),
					   parameters[2].GetInteger(),parameters[3].GetInteger(),
					   (Image2D::Flags)parameters[4].GetInteger());
			else if (parameters.size() == 6)
	            Draw3D(parameters[0].GetInteger(),parameters[1].GetInteger(),
					   parameters[2].GetInteger(),parameters[3].GetInteger(),
					   (Image2D::Flags)parameters[4].GetInteger(),
					   parameters[5].GetInteger());
			else if (parameters.size() == 10)
	            Draw3D(parameters[0].GetInteger(),parameters[1].GetInteger(),
					   parameters[2].GetInteger(),parameters[3].GetInteger(),
					   (Image2D::Flags)parameters[4].GetInteger(),
					   parameters[5].GetInteger(),
					   parameters[6].GetInteger(),parameters[7].GetInteger(),
					   parameters[8].GetInteger(),parameters[9].GetInteger());
			else
                throw IVM::IVMException(
                         "wrong number of parameters for Draw");

            return true;

        } break;
    }

	return true;
}

} // namespace Isis

