
#include <isis/util.h>
#include <isis/ivm.h>

#include <isis/game/Font2D.h>

#include <isis/game/GraphicsEngine.h>
#include <isis/game/Image2D.h>

namespace Isis {

void Font2D::FillExportTable()
{
    m_ExportTable["Init"]			= FUNC_INIT;
    m_ExportTable["DrawString"]		= FUNC_DRAWSTRING;
    m_ExportTable["DrawStringUp"]	= FUNC_DRAWSTRINGUP;
    m_ExportTable["DrawStringDown"]	= FUNC_DRAWSTRINGDOWN;
    m_ExportTable["CalcTextSize"]	= FUNC_CALCTEXTSIZE;
    m_ExportTable["CalcNumLines"]	= FUNC_CALCNUMLINES;
    m_ExportTable["ChopLines"]      = FUNC_CHOPLINES;
}

void Font2D::IVMFillExportTable(std::vector<std::string> & table)
{
    std::map<std::string,FuncType>::iterator iter;

    for (iter = m_ExportTable.begin(); iter != m_ExportTable.end(); iter++)
    {
        table.push_back(iter->first);
    }
}

bool Font2D::IVMContainsMethod(const std::string & funcname)
{
    return m_ExportTable.end() != m_ExportTable.find(funcname);
}

IVM::Value Font2D::IVMCallMethod(const std::string & funcname,
                           const std::vector<IVM::Value> & parameters)
{   
    if (m_ExportTable.end() == m_ExportTable.find(funcname))
        throw IVM::IVMException("function not found in Font2D object");
    
    switch (m_ExportTable[funcname])
    {
    case FUNC_INIT:
        {
            if (parameters.size() != 1)
                throw IVM::IVMException(
                       "wrong number of parameters for Init");

			if (parameters[0].GetType() == IVM::Value::String)
			{
	            Init(parameters[0].GetString());
			}
			else
			{
				std::vector<Image2D*> stlVector;
				IVM::IVMVector * pVector = dynamic_cast<IVM::IVMVector*>
									(parameters[0].GetObjPtr());

				if (NULL == pVector)
					throw IVM::IVMException("NULL vector for Load()");

				for (int i = 0; i < pVector->Size(); i++)
					stlVector.push_back(
						dynamic_cast<Image2D*>(pVector->Get(i).GetObjPtr()));

				Init(stlVector);
			}
		} break;
    case FUNC_DRAWSTRING:
        {
            if (parameters.size() != 3 && 
				parameters.size() != 4 && 
				parameters.size() != 5 && 
				parameters.size() != 6)
			{
                throw IVM::IVMException(
                         "wrong number of parameters for DrawString");
			}

			if (parameters.size() == 3)
			{
				DrawString(parameters[0].GetString(),
						   parameters[1].GetInteger(),
						   parameters[2].GetInteger());
			}
			else if (parameters.size() == 4)
			{
				DrawString(parameters[0].GetString(),
						   parameters[1].GetInteger(),
						   parameters[2].GetInteger(),
						   parameters[3].GetInteger());
			}
			else if (parameters.size() == 5)
			{
				DrawString(parameters[0].GetString(),
						   parameters[1].GetInteger(),
						   parameters[2].GetInteger(),
						   parameters[3].GetInteger(),
						   parameters[4].GetInteger());
			}
			else if (parameters.size() == 6)
			{
				DrawString(parameters[0].GetString(),
						   parameters[1].GetInteger(),
						   parameters[2].GetInteger(),
						   parameters[3].GetInteger(),
						   parameters[4].GetInteger(),
						   parameters[4].GetInteger());
			}

            return true;

        } break;
    case FUNC_DRAWSTRINGUP:
        {
            if (parameters.size() != 3 && 
				parameters.size() != 4)
			{
                throw IVM::IVMException(
                         "wrong number of parameters for DrawStringUp");
			}

			if (parameters.size() == 3)
			{
				DrawStringUp(parameters[0].GetString(),
						     parameters[1].GetInteger(),
						     parameters[2].GetInteger());
			}
			else if (parameters.size() == 4)
			{
				DrawStringUp(parameters[0].GetString(),
						     parameters[1].GetInteger(),
						     parameters[2].GetInteger(),
						     parameters[3].GetInteger());
			}

            return true;

        } break;
    case FUNC_DRAWSTRINGDOWN:
        {
            if (parameters.size() != 3 && 
				parameters.size() != 4)
			{
                throw IVM::IVMException(
                         "wrong number of parameters for DrawStringDown");
			}

			if (parameters.size() == 3)
			{
				DrawStringDown(parameters[0].GetString(),
						       parameters[1].GetInteger(),
						       parameters[2].GetInteger());
			}
			else if (parameters.size() == 4)
			{
				DrawStringDown(parameters[0].GetString(),
						       parameters[1].GetInteger(),
						       parameters[2].GetInteger(),
						       parameters[3].GetInteger());
			}

            return true;

        } break;
    case FUNC_CALCTEXTSIZE:
        {
            if (parameters.size() != 1)
                throw IVM::IVMException(
                         "wrong number of parameters for CalcTextSize");

			int iWidth;
			int iHeight;

			CalcTextSize(parameters[0].GetString(),iWidth,iHeight);

			IVM::IVMVector * pVector = new IVM::IVMVector();
			pVector->Add(iWidth);
			pVector->Add(iHeight);

            return pVector;

        } break;
    case FUNC_CALCNUMLINES:
        {
            if (parameters.size() != 2)
                throw IVM::IVMException(
                         "wrong number of parameters for CalcNumLines");

            return CalcNumLines(parameters[0].GetString(),
								parameters[1].GetInteger());

        } break;
    case FUNC_CHOPLINES:
        {
            if (parameters.size() != 3)
                throw IVM::IVMException(
                         "wrong number of parameters for CalcNumLines");

            return ChopLines(parameters[0].GetString(),
							 parameters[1].GetInteger(),
							 parameters[2].GetInteger());

        } break;
    }

	return true;
}

} // namespace Isis

