
#include <isis/game/Font2D.h>

#include <isis/game/GUIGame.h>
#include <isis/game/GUIWindow.h>

namespace Isis {

int GUIGame::GetMouseX()
{
	return m_Mouse.iX;
}

int GUIGame::GetMouseY()
{
	return m_Mouse.iY;
}

void GUIGame::PushFont(Font2D * pFont)
{
	if (pFont)
		pFont->AddRef();

	m_FontStack.push_back(pFont);
}

void GUIGame::PopFont()
{
	if (m_FontStack.size() > 0)
	{
		if (m_FontStack.back() != NULL)
			m_FontStack.back()->Release();

		m_FontStack.pop_back();
	}
}

Font2D * GUIGame::GetDefaultFont()
{
	if (m_FontStack.size() > 0)
		return m_FontStack.back();
	else
		return NULL;
}

void GUIGame::SetKeyboardFocus(GUIWindow * pWindow)
{
	if (pWindow == m_pKeyboardFocusWindow)
		return;

	if (m_pKeyboardFocusWindow)
		m_pKeyboardFocusWindow->Release();

	m_pKeyboardFocusWindow = pWindow;

	if (m_pKeyboardFocusWindow)
		m_pKeyboardFocusWindow->AddRef();
}

GUIWindow * GUIGame::GetKeyboardFocus()
{
	return m_pKeyboardFocusWindow;
}

void GUIGame::SetMouseCapture(GUIWindow * pWindow)
{
	if (pWindow == m_pMouseCaptureWindow)
		return;

	if (m_pMouseCaptureWindow)
		m_pMouseCaptureWindow->Release();

	m_pMouseCaptureWindow = pWindow;

	if (m_pMouseCaptureWindow)
		m_pMouseCaptureWindow->AddRef();
}

GUIWindow * GUIGame::GetMouseCapture()
{
	return m_pMouseCaptureWindow;
}

} // namespace Isis

