
#include <isis/game/Font2D.h>

#include <isis/game/GUIGame.h>
#include <isis/game/GUIWindow.h>

namespace Isis {

void GUIGame::FillExportTable2()
{
    m_ExportTable2["SetKeyboardFocus"]    = FUNC_SETKEYBOARDFOCUS;
    m_ExportTable2["GetKeyboardFocus"]    = FUNC_GETKEYBOARDFOCUS;
    m_ExportTable2["SetMouseCapture"]     = FUNC_SETMOUSECAPTURE;
    m_ExportTable2["GetMouseCapture"]     = FUNC_GETMOUSECAPTURE;
    m_ExportTable2["GetMouseX"]           = FUNC_GETMOUSEX;
    m_ExportTable2["GetMouseY"]           = FUNC_GETMOUSEY;
    m_ExportTable2["PushFont"]            = FUNC_PUSHFONT;
    m_ExportTable2["PopFont"]             = FUNC_POPFONT;
    m_ExportTable2["GetDefaultFont"]      = FUNC_GETDEFAULTFONT;
    m_ExportTable2["AddExclusiveWindow"]  = FUNC_ADDEXCLUSIVEWINDOW;
    m_ExportTable2["RemoveExclusiveWindow"] = FUNC_REMOVEEXCLUSIVEWINDOW;
    m_ExportTable2["GetExclusiveWindow"]  = FUNC_GETEXCLUSIVEWINDOW;
    m_ExportTable2["AddFrameUpdater"]     = FUNC_ADDFRAMEUPDATER;
    m_ExportTable2["RemoveFrameUpdater"]  = FUNC_REMOVEFRAMEUPDATER;

    m_ExportTable2["MakeWindow"]        = FUNC_MAKEWINDOW;
    m_ExportTable2["RemoveWindow"]      = FUNC_REMOVEWINDOW;
	m_ExportTable2["DestroyAllWindows"] = FUNC_DESTROYALLWINDOWS;
    m_ExportTable2["WindowToScreen"]    = FUNC_WINDOWTOSCREEN;
    m_ExportTable2["ScreenToWindow"]    = FUNC_SCREENTOWINDOW;
    m_ExportTable2["WindowFromPoint"]   = FUNC_WINDOWFROMPOINT;

	m_ExportTable2["GetGUIGame"]        = FUNC_GETGUIGAME;
}

void GUIGame::IVMFillExportTable(std::vector<std::string> & table)
{
	std::map<std::string,GameApp::FuncType>::iterator iter1;
	
    for (iter1 = m_ExportTable.begin(); iter1 != m_ExportTable.end(); iter1++)
    {
        table.push_back(iter1->first);
    }

	std::map<std::string,FuncType>::iterator iter2;
	
    for (iter2 = m_ExportTable2.begin(); iter2 != m_ExportTable2.end(); iter2++)
    {
        table.push_back(iter2->first);
    }
}

bool GUIGame::IVMContainsMethod(const std::string & funcname)
{
    return m_ExportTable.end() != m_ExportTable.find(funcname) ||
		   m_ExportTable2.end() != m_ExportTable2.find(funcname);
}

IVM::Value GUIGame::IVMCallMethod(const std::string & funcname,
							const std::vector<IVM::Value> & parameters)
{   
	if (m_ExportTable.end() != m_ExportTable.find(funcname))
		return GameApp::IVMCallMethod(funcname,parameters);
	
    if (m_ExportTable2.end() == m_ExportTable2.find(funcname))
        throw IVM::IVMException("Method not found in GUIGame");
	
    switch (m_ExportTable2[funcname])
    {
	case FUNC_MAKEWINDOW:
	    {
			switch (parameters.size())
			{
			case 2:
				{
					return MakeWindow(parameters[0].GetInteger(),
									  parameters[1].GetInteger());
				} break;
			case 4:
				{
					return MakeWindow(parameters[0].GetInteger(),
									  parameters[1].GetInteger(),
									  parameters[2].GetInteger(),
									  parameters[3].GetInteger());
				} break;
			default:
				{
					throw IVM::IVMException(
								"wrong number of parameters for MakeWindow");
				}
			}
		} break;	
	case FUNC_REMOVEWINDOW:
	    {
			if (parameters.size() != 1)
				throw IVM::IVMException(
							"wrong number of parameters for RemoveWindow");

			RemoveWindow(dynamic_cast<GUIWindow*>(parameters[0].GetObjPtr()));

			return true;

		} break;	
	case FUNC_DESTROYALLWINDOWS:
	    {
			if (parameters.size() != 0)
				throw IVM::IVMException(
						"wrong number of parameters for DestroyAllWindows");

			DestroyAllWindows();

			return true;

		} break;	
	case FUNC_WINDOWTOSCREEN:
	    {
			if (parameters.size() != 2)
				throw IVM::IVMException(
							"wrong number of parameters for WindowToScreen");

			int x = parameters[0].GetInteger();
			int y = parameters[1].GetInteger();

			WindowToScreen(x,y);

			IVM::IVMVector * pVector = new IVM::IVMVector();
			pVector->Add(x);
			pVector->Add(y);

			return pVector;

		} break;	
	case FUNC_SCREENTOWINDOW:
	    {
			if (parameters.size() != 2)
				throw IVM::IVMException(
							"wrong number of parameters for ScreenToWindow");

			int x = parameters[0].GetInteger();
			int y = parameters[1].GetInteger();

			ScreenToWindow(x,y);

			IVM::IVMVector * pVector = new IVM::IVMVector();
			pVector->Add(x);
			pVector->Add(y);

			return pVector;

		} break;	
	case FUNC_WINDOWFROMPOINT:
	    {
			switch (parameters.size())
			{
			case 2:
				{
					int x = parameters[0].GetInteger();
					int y = parameters[1].GetInteger();

					return WindowFromPoint(x,y);
				} break;
			case 3:
				{
					int x = parameters[0].GetInteger();
					int y = parameters[1].GetInteger();
					bool b = parameters[2].GetInteger() != 0;

					return WindowFromPoint(x,y,b);
				} break;
			default:
				{
					throw IVM::IVMException(
							"wrong number of parameters for WindowFromPoint");
				} break;
			}
		} break;	
	case FUNC_SETKEYBOARDFOCUS:
	    {
			if (parameters.size() != 1)
				throw IVM::IVMException(
						"wrong number of parameters for SetKeyboardFocus");
			
			SetKeyboardFocus(dynamic_cast<GUIWindow*>
							(parameters[0].GetObjPtr()));

			return true;
			
		} break;	
	case FUNC_GETKEYBOARDFOCUS:
	    {
			if (parameters.size() != 0)
				throw IVM::IVMException(
						"wrong number of parameters for GetKeyboardFocus");
			
			return GetKeyboardFocus();
			
		} break;	
	case FUNC_SETMOUSECAPTURE:
	    {
			if (parameters.size() != 1)
				throw IVM::IVMException(
						"wrong number of parameters for SetMouseCapture");
			
			SetMouseCapture(dynamic_cast<GUIWindow*>
							(parameters[0].GetObjPtr()));

			return true;
			
		} break;	
	case FUNC_GETMOUSECAPTURE:
	    {
			if (parameters.size() != 0)
				throw IVM::IVMException(
						"wrong number of parameters for GetMouseCapture");
			
			return GetMouseCapture();
			
		} break;	
	case FUNC_GETMOUSEX:
	    {
			if (parameters.size() != 0)
				throw IVM::IVMException(
								"wrong number of parameters for GetMouseX");
			
			return GetMouseX();
			
		} break;	
	case FUNC_GETMOUSEY:
	    {
			if (parameters.size() != 0)
				throw IVM::IVMException(
								"wrong number of parameters for GetMouseY");
			
			return GetMouseY();
			
		} break;	
	case FUNC_PUSHFONT:
	    {
			if (parameters.size() != 1)
				throw IVM::IVMException(
							"wrong number of parameters for PushFont");

			PushFont(dynamic_cast<Font2D*>(parameters[0].GetObjPtr()));

			return true;
			
		} break;	
	case FUNC_POPFONT:
	    {
			if (parameters.size() != 0)
				throw IVM::IVMException(
							"wrong number of parameters for PopFont");

			PopFont();

			return true;
			
		} break;	
	case FUNC_GETDEFAULTFONT:
	    {
			if (parameters.size() != 0)
				throw IVM::IVMException(
						"wrong number of parameters for GetDefaultFont");

			return GetDefaultFont();

		} break;	
	case FUNC_ADDEXCLUSIVEWINDOW:
	    {
			if (parameters.size() != 1)
				throw IVM::IVMException(
					"wrong number of parameters for AddExclusiveWindow");

			AddExclusiveWindow(dynamic_cast<GUIWindow*>(
							   parameters[0].GetObjPtr()));

			return true;

		} break;	
	case FUNC_REMOVEEXCLUSIVEWINDOW:
	    {
			if (parameters.size() != 1)
				throw IVM::IVMException(
					"wrong number of parameters for RemoveExclusiveWindow");

			RemoveExclusiveWindow(dynamic_cast<GUIWindow*>(
							   parameters[0].GetObjPtr()));

			return true;

		} break;	
	case FUNC_GETEXCLUSIVEWINDOW:
	    {
			if (parameters.size() != 0)
				throw IVM::IVMException(
					"wrong number of parameters for GetExclusiveWindow");

			return GetExclusiveWindow();

		} break;	
	case FUNC_ADDFRAMEUPDATER:
	    {
			if (parameters.size() != 1)
				throw IVM::IVMException(
					"wrong number of parameters for AddFrameUpdater");

			AddFrameUpdater(dynamic_cast<GUIWindow*>(
							   parameters[0].GetObjPtr()));

			return true;

		} break;	
	case FUNC_REMOVEFRAMEUPDATER:
	    {
			if (parameters.size() != 1)
				throw IVM::IVMException(
					"wrong number of parameters for RemoveFrameUpdater");

			RemoveFrameUpdater(dynamic_cast<GUIWindow*>(
							   parameters[0].GetObjPtr()));

			return true;

		} break;
	case FUNC_GETGUIGAME:
		{
			return this;
		} break;
    }

	return true;
}

} // namespace Isis

