
#include <isis/game/GUIGame.h>
#include <isis/game/GUIWindow.h>

namespace Isis {

void GUIGame::FillObjectTable()
{
    m_ObjectTable["GUIWindow"]   = OBJECT_GUIWINDOW;
}

void GUIGame::FillObjectTable(std::vector<std::string> & table)
{
    std::map<std::string,ObjectType>::iterator iter;

    for (iter = m_ObjectTable.begin(); iter != m_ObjectTable.end(); iter++)
    {
        table.push_back(iter->first);
    }
}

IVM::IVMObject * GUIGame::CreateObject(const std::string & objname,
                                   const std::vector<IVM::Value> & InitParams)
{
    if (m_ObjectTable.end() == m_ObjectTable.find(objname))
        throw IVM::IVMException("object type not found");

    IVM::IVMObject * pObject;

    switch (m_ObjectTable[objname])
    {
    case OBJECT_GUIWINDOW:
        {
			GUIWindow * pWindow = new GUIWindow(this,this);
			pWindow->AddRef();
			m_WindowList.push_back(pWindow);

			pObject = pWindow;

        } break;
    }

    if (pObject->IVMContainsMethod("Init"))
    {
        pObject->IVMCallMethod("Init",InitParams);
    } 
	else if (!InitParams.empty())
    {
        throw IVM::IVMException("Init() parameters given, but Init not found");
    }

    return pObject;
}

} // namespace Isis


