
#include <isis/game/Image2D.h>
#include <isis/game/Font2D.h>

#include <isis/game/GUIGame.h>
#include <isis/game/GUIWindow.h>

#include <isis/game/IMouseHandler.h>
#include <isis/game/IKeyHandler.h>
#include <isis/game/ICustomDraw.h>
#include <isis/game/IDestroyCallback.h>
#include <isis/game/IFrameUpdate.h>

#include <isis/game/IWindowContainer.h>

namespace Isis {

void GUIWindow::SetX(int x)
{
	m_iX = x;
}

int GUIWindow::GetX()
{
	return m_iX;
}

void GUIWindow::SetY(int y)
{
	m_iY = y;
}

int GUIWindow::GetY()
{
	return m_iY;
}

void GUIWindow::SetWidth(int width)
{
	m_iWidth = width;
}

int GUIWindow::GetWidth()
{
	return m_iWidth;
}

void GUIWindow::SetHeight(int height)
{
	m_iHeight = height;
}

int GUIWindow::GetHeight()
{
	return m_iHeight;
}

IWindowContainer * GUIWindow::GetContainer()
{
	return m_pContainer;
}

void GUIWindow::SetHidden(bool bHidden)
{
	m_bHidden = bHidden;
}

bool GUIWindow::IsHidden()
{
	return m_bHidden;
}

void GUIWindow::SetTransparent(bool bTransparent)
{
	m_bTransparent = bTransparent;
}

bool GUIWindow::IsTransparent()
{
	return m_bTransparent;
}

void GUIWindow::SetExclusive(bool bExclusive)
{
	if (m_bExclusive != bExclusive)
	{
		m_bExclusive = bExclusive;

		if (m_bExclusive)
			m_pGUIGame->AddExclusiveWindow(this);
		else
			m_pGUIGame->RemoveExclusiveWindow(this);
	}
}

bool GUIWindow::IsExclusive()
{
	return m_bExclusive;
}

void GUIWindow::SetColor(unsigned int color)
{
	m_Color = color;
}

unsigned int GUIWindow::GetColor()
{
	return m_Color;
}

void GUIWindow::SetImage(Image2D * pImage, bool bChangeSize, 
						 Image2D::Flags flags)
{
	if (m_bDestroyed)
		return;

	if (m_pImage)
		m_pImage->Release();

	m_pImage = pImage;
	m_ImageFlags = flags;

	if (m_pImage)
	{
		m_pImage->AddRef();

		if (bChangeSize)
		{
			m_iWidth = m_pImage->GetWidth();
			m_iHeight = m_pImage->GetHeight();
		}
	}
}

Image2D * GUIWindow::GetImage()
{
	return m_pImage;
}

void GUIWindow::SetText(const std::string & sText, unsigned int modColor)
{
	m_sText = sText;
	m_TextColor = modColor;
}

std::string GUIWindow::GetText()
{
	return m_sText;
}

void GUIWindow::SetTextColor(unsigned int modColor)
{
	m_TextColor = modColor;
}

unsigned int GUIWindow::GetTextColor()
{
	return m_TextColor;
}

void GUIWindow::SetFont(Font2D * pFont)
{
	if (m_bDestroyed)
		return;

	if (m_pFont)
		m_pFont->Release();

	m_pFont = pFont;

	if (m_pFont)
		m_pFont->AddRef();
}

Font2D * GUIWindow::GetFont()
{
	return m_pFont;
}

void GUIWindow::SetTextDirection(int dir)
{
	m_iTextDirection = dir;
}

int GUIWindow::GetTextDirection()
{
	return m_iTextDirection;
}


void GUIWindow::SetMouseHandler(IMouseHandler * pMouseHandler)
{
	if (m_bDestroyed)
		return;

	if (m_pMouseHandler)
		m_pMouseHandler->Release();

	m_pMouseHandler = pMouseHandler;

	if (m_pMouseHandler)
		m_pMouseHandler->AddRef();
}

IMouseHandler * GUIWindow::GetMouseHandler()
{
	return m_pMouseHandler;
}

void GUIWindow::SetKeyHandler(IKeyHandler * pKeyHandler)
{
	if (m_bDestroyed)
		return;

	if (m_pKeyHandler)
		m_pKeyHandler->Release();

	m_pKeyHandler = pKeyHandler;

	if (m_pKeyHandler)
		m_pKeyHandler->AddRef();
}

IKeyHandler * GUIWindow::GetKeyHandler()
{
	return m_pKeyHandler;
}

void GUIWindow::SetCustomDraw(ICustomDraw * pCustomDraw)
{
	if (m_bDestroyed)
		return;

	if (m_pCustomDraw)
		m_pCustomDraw->Release();

	m_pCustomDraw = pCustomDraw;

	if (m_pCustomDraw)
		m_pCustomDraw->AddRef();
}

ICustomDraw * GUIWindow::GetCustomDraw()
{
	return m_pCustomDraw;
}

void GUIWindow::SetDestroyCallback(IDestroyCallback * pDestroyCallback)
{
	if (m_bDestroyed)
		return;

	if (m_pDestroyCallback)
		m_pDestroyCallback->Release();

	m_pDestroyCallback = pDestroyCallback;

	if (m_pDestroyCallback)
		m_pDestroyCallback->AddRef();
}

IDestroyCallback * GUIWindow::GetDestroyCallback()
{
	return m_pDestroyCallback;
}

void GUIWindow::SetFrameUpdate(IFrameUpdate * pFrameUpdate)
{
	if (m_bDestroyed)
		return;

	if (m_pFrameUpdate)
    {
        m_pGUIGame->RemoveFrameUpdater(this);
		m_pFrameUpdate->Release();
    }

	m_pFrameUpdate = pFrameUpdate;

	if (m_pFrameUpdate)
    {
		m_pFrameUpdate->AddRef();
        m_pGUIGame->AddFrameUpdater(this);
    }
}

IFrameUpdate * GUIWindow::GetFrameUpdate()
{
	return m_pFrameUpdate;
}

GUIGame * GUIWindow::GetGUIGame()
{
	return m_pGUIGame;
}
} // namespace Isis

