
#include <isis/game/Image2D.h>
#include <isis/game/Font2D.h>

#include <isis/game/GUIWindow.h>

#include <isis/game/IMouseHandler.h>
#include <isis/game/IKeyHandler.h>
#include <isis/game/ICustomDraw.h>
#include <isis/game/IDestroyCallback.h>
#include <isis/game/IFrameUpdate.h>

#include <isis/game/IWindowContainer.h>

#include <isis/game/IVMMouseHandler.h>
#include <isis/game/IVMKeyHandler.h>
#include <isis/game/IVMCustomDraw.h>
#include <isis/game/IVMDestroyCallback.h>
#include <isis/game/IVMFrameUpdate.h>

namespace Isis {

void GUIWindow::FillExportTable()
{
    m_ExportTable["Init"]              = FUNC_INIT;
    m_ExportTable["Destroy"]           = FUNC_DESTROY;

	m_ExportTable["Draw"]              = FUNC_DRAW;
	m_ExportTable["FrameUpdate"]       = FUNC_FRAMEUPDATE;

	m_ExportTable["MakeWindow"]        = FUNC_MAKEWINDOW;
    m_ExportTable["RemoveWindow"]      = FUNC_REMOVEWINDOW;
    m_ExportTable["WindowToScreen"]    = FUNC_WINDOWTOSCREEN;
    m_ExportTable["ScreenToWindow"]    = FUNC_SCREENTOWINDOW;
    m_ExportTable["WindowFromPoint"]   = FUNC_WINDOWFROMPOINT;

	m_ExportTable["SetTransparent"]    = FUNC_SETTRANSPARENT;
    m_ExportTable["IsTransparent"]     = FUNC_ISTRANSPARENT;
	m_ExportTable["SetHidden"]         = FUNC_SETHIDDEN;
    m_ExportTable["IsHidden"]          = FUNC_ISHIDDEN;
    m_ExportTable["SetExclusive"]      = FUNC_SETEXCLUSIVE;
    m_ExportTable["IsExclusive"]       = FUNC_ISEXCLUSIVE;

	m_ExportTable["SetColor"]          = FUNC_SETCOLOR;
	m_ExportTable["GetColor"]          = FUNC_GETCOLOR;
	m_ExportTable["SetImage"]          = FUNC_SETIMAGE;
	m_ExportTable["GetImage"]          = FUNC_GETIMAGE;
	m_ExportTable["SetText"]           = FUNC_SETTEXT;
	m_ExportTable["GetText"]           = FUNC_GETTEXT;
	m_ExportTable["SetTextColor"]      = FUNC_SETTEXTCOLOR;
	m_ExportTable["GetTextColor"]      = FUNC_GETTEXTCOLOR;
	m_ExportTable["SetFont"]           = FUNC_SETFONT;
	m_ExportTable["GetFont"]           = FUNC_GETFONT;
	m_ExportTable["SetTextDirection"]  = FUNC_SETTEXTDIRECTION;
	m_ExportTable["GetTextDirection"]  = FUNC_GETTEXTDIRECTION;
	
	m_ExportTable["SetX"]              = FUNC_SETX;
	m_ExportTable["GetX"]              = FUNC_GETX;
	m_ExportTable["SetY"]              = FUNC_SETY;
	m_ExportTable["GetY"]              = FUNC_GETY;
	m_ExportTable["GetHeight"]         = FUNC_GETHEIGHT;
	m_ExportTable["GetWidth"]          = FUNC_GETWIDTH;
	m_ExportTable["SetWidth"]          = FUNC_SETWIDTH;
	m_ExportTable["GetHeight"]         = FUNC_GETHEIGHT;
	m_ExportTable["SetHeight"]         = FUNC_SETHEIGHT;
	m_ExportTable["GetContainer"]      = FUNC_GETCONTAINER;

    m_ExportTable["GetMouseHandler"]   = FUNC_GETMOUSEHANDLER;
    m_ExportTable["SetMouseHandler"]   = FUNC_SETMOUSEHANDLER;
    m_ExportTable["GetKeyHandler"]     = FUNC_GETKEYHANDLER;
    m_ExportTable["SetKeyHandler"]     = FUNC_SETKEYHANDLER;
    m_ExportTable["GetCustomDraw"]     = FUNC_GETCUSTOMDRAW;
    m_ExportTable["SetCustomDraw"]     = FUNC_SETCUSTOMDRAW;
    m_ExportTable["GetDestroyCallback"] = FUNC_GETDESTROYCALLBACK;
    m_ExportTable["SetDestroyCallback"] = FUNC_SETDESTROYCALLBACK;
    m_ExportTable["GetFrameUpdate"]    = FUNC_GETFRAMEUPDATE;
    m_ExportTable["SetFrameUpdate"]    = FUNC_SETFRAMEUPDATE;
}

void GUIWindow::IVMFillExportTable(std::vector<std::string> & table)
{
    std::map<std::string,FuncType>::iterator iter;

    for (iter = m_ExportTable.begin(); iter != m_ExportTable.end(); iter++)
    {
        table.push_back(iter->first);
    }
}

bool GUIWindow::IVMContainsMethod(const std::string & funcname)
{
    return m_ExportTable.end() != m_ExportTable.find(funcname);
}

IVM::Value GUIWindow::IVMCallMethod(const std::string & funcname,
                           const std::vector<IVM::Value> & parameters)
{   
    if (m_ExportTable.end() == m_ExportTable.find(funcname))
		throw IVM::IVMException("function not found in GUIWindow object:",
								funcname);
    
    switch (m_ExportTable[funcname])
    {
	case FUNC_INIT:
	    {
			switch (parameters.size())
			{
			case 2:
				{
					Init(parameters[0].GetInteger(),
						 parameters[1].GetInteger());
				} break;
			case 4:
				{
					Init(parameters[0].GetInteger(),
						 parameters[1].GetInteger(),
						 parameters[2].GetInteger(),
						 parameters[3].GetInteger());
				} break;
			default:
				{
					throw IVM::IVMException(
								"wrong number of parameters for Init");
				}
			}
		} break;	
    case FUNC_DESTROY:
        {
			if (parameters.size() != 0)
				throw IVM::IVMException(
				  "wrong number of parameters for Destroy");

			Destroy();

			return true;

        } break;
    case FUNC_DRAW:
        {
			if (parameters.size() != 0)
				throw IVM::IVMException(
				  "wrong number of parameters for Draw");

			Draw();

			return true;

        } break;
    case FUNC_FRAMEUPDATE:
        {
			if (parameters.size() != 0)
				throw IVM::IVMException(
				  "wrong number of parameters for FrameUpdate");

			FrameUpdate();

			return true;

        } break;
	case FUNC_MAKEWINDOW:
	    {
			switch (parameters.size())
			{
			case 2:
				{
					return MakeWindow(parameters[0].GetInteger(),
									  parameters[1].GetInteger());
				} break;
			case 4:
				{
					return MakeWindow(parameters[0].GetInteger(),
									  parameters[1].GetInteger(),
									  parameters[2].GetInteger(),
									  parameters[3].GetInteger());
				} break;
			default:
				{
					throw IVM::IVMException(
								"wrong number of parameters for MakeWindow");
				}
			}
			
		} break;	
	case FUNC_REMOVEWINDOW:
	    {
			if (parameters.size() != 1)
				throw IVM::IVMException(
							"wrong number of parameters for RemoveWindow");

			RemoveWindow(dynamic_cast<GUIWindow*>(parameters[0].GetObjPtr()));

			return true;

		} break;	
	case FUNC_WINDOWTOSCREEN:
	    {
			if (parameters.size() != 2)
				throw IVM::IVMException(
							"wrong number of parameters for WindowToScreen");

			int x = parameters[0].GetInteger();
			int y = parameters[1].GetInteger();

			WindowToScreen(x,y);

			IVM::IVMVector * pVector = new IVM::IVMVector();
			pVector->Add(x);
			pVector->Add(y);

			return pVector;

		} break;	
	case FUNC_SCREENTOWINDOW:
	    {
			if (parameters.size() != 2)
				throw IVM::IVMException(
							"wrong number of parameters for ScreenToWindow");

			int x = parameters[0].GetInteger();
			int y = parameters[1].GetInteger();

			ScreenToWindow(x,y);

			IVM::IVMVector * pVector = new IVM::IVMVector();
			pVector->Add(x);
			pVector->Add(y);

			return pVector;

		} break;	
	case FUNC_WINDOWFROMPOINT:
	    {
			switch (parameters.size())
			{
			case 2:
				{
					int x = parameters[0].GetInteger();
					int y = parameters[1].GetInteger();

					return WindowFromPoint(x,y);
				} break;
			case 3:
				{
					int x = parameters[0].GetInteger();
					int y = parameters[1].GetInteger();
					bool b = parameters[2].GetInteger() != 0;

					return WindowFromPoint(x,y,b);
				} break;
			default:
				{
					throw IVM::IVMException(
							"wrong number of parameters for WindowFromPoint");
				} break;
			}
		} break;	
    case FUNC_SETHIDDEN:
        {
			if (parameters.size() != 1)
				throw IVM::IVMException(
				  "wrong number of parameters for SetHidden");

			SetHidden(parameters[0].GetInteger() != 0);

            return true;

        } break;
    case FUNC_ISHIDDEN:
        {
			if (parameters.size() != 0)
				throw IVM::IVMException(
				  "wrong number of parameters for IsHidden");

            return IsHidden();

        } break;
    case FUNC_SETTRANSPARENT:
        {
			if (parameters.size() != 1)
				throw IVM::IVMException(
				  "wrong number of parameters for SetTransparent");

			SetTransparent(parameters[0].GetInteger() != 0);

            return true;

        } break;
    case FUNC_ISTRANSPARENT:
        {
			if (parameters.size() != 0)
				throw IVM::IVMException(
				  "wrong number of parameters for IsTransparent");

            return IsTransparent();

        } break;
    case FUNC_SETEXCLUSIVE:
        {
			if (parameters.size() != 1)
				throw IVM::IVMException(
				  "wrong number of parameters for SetExclusive");

			SetExclusive(parameters[0].GetInteger() != 0);

            return true;

        } break;
    case FUNC_ISEXCLUSIVE:
        {
			if (parameters.size() != 0)
				throw IVM::IVMException(
				  "wrong number of parameters for IsExclusive");

            return IsExclusive();

        } break;
    case FUNC_SETCOLOR:
        {
			if (parameters.size() != 1)
				throw IVM::IVMException(
				  "wrong number of parameters for SetColor");

			SetColor(parameters[0].GetInteger());

            return true;

        } break;
    case FUNC_GETCOLOR:
        {
			if (parameters.size() != 0)
				throw IVM::IVMException(
				  "wrong number of parameters for GetColor");

            return (int)GetColor();

        } break;
    case FUNC_SETIMAGE:
        {
			switch (parameters.size())
			{
			case 1:
				{
					SetImage(dynamic_cast<Image2D*>(
						parameters[0].GetObjPtr()));
				} break;
			case 2:
				{
					SetImage(dynamic_cast<Image2D*>(
						parameters[0].GetObjPtr()),
						parameters[1].GetInteger() != 0);
				} break;
			case 3:
				{
					SetImage(dynamic_cast<Image2D*>(
						parameters[0].GetObjPtr()),
						parameters[1].GetInteger() != 0,
						(Image2D::Flags)parameters[2].GetInteger());
				} break;
			default:
				{
				throw IVM::IVMException(
				  "wrong number of parameters for SetImage");
				}
			}

            return true;

        } break;
    case FUNC_GETIMAGE:
        {
			if (parameters.size() != 0)
				throw IVM::IVMException(
				  "wrong number of parameters for GetImage");

            return GetImage();

        } break;
    case FUNC_SETTEXT:
        {
			switch (parameters.size())
			{
			case 1:
				{
					SetText(parameters[0].GetString());
				} break;
			case 2:
				{
					SetText(parameters[0].GetString(),
							parameters[1].GetInteger());
				} break;
			default:
				{
					throw IVM::IVMException(
						"wrong number of parameters for SetText");
				}
			}

            return true;

        } break;
    case FUNC_GETTEXT:
        {
			if (parameters.size() != 0)
				throw IVM::IVMException(
				  "wrong number of parameters for GetText");

            return GetText();

        } break;
    case FUNC_SETTEXTCOLOR:
        {
			if (parameters.size() != 1)
				throw IVM::IVMException(
				  "wrong number of parameters for SetTextColor");

            SetTextColor((unsigned int)parameters[0].GetInteger());

        } break;
    case FUNC_GETTEXTCOLOR:
        {
			if (parameters.size() != 0)
				throw IVM::IVMException(
				  "wrong number of parameters for GetTextColor");

            return (int)GetTextColor();

        } break;
    case FUNC_SETFONT:
        {
			if (parameters.size() != 1)
				throw IVM::IVMException(
				  "wrong number of parameters for SetFont");

			SetFont(dynamic_cast<Font2D*>(parameters[0].GetObjPtr()));

            return true;

        } break;
    case FUNC_GETFONT:
        {
			if (parameters.size() != 0)
				throw IVM::IVMException(
				  "wrong number of parameters for GetFont");

            return GetFont();

        } break;
    case FUNC_SETTEXTDIRECTION:
        {
			if (parameters.size() != 1)
				throw IVM::IVMException(
				  "wrong number of parameters for SetTextDirection");

			SetTextDirection(parameters[0].GetInteger());

            return true;

        } break;
    case FUNC_GETTEXTDIRECTION:
        {
			if (parameters.size() != 0)
				throw IVM::IVMException(
				  "wrong number of parameters for GetTextDirection");

            return GetTextDirection();

        } break;
    case FUNC_GETCONTAINER:
        {
			if (parameters.size() != 0)
				throw IVM::IVMException(
				  "wrong number of parameters for GetContainer");

            return GetContainer();

        } break;
    case FUNC_SETX:
        {
			if (parameters.size() != 1)
				throw IVM::IVMException(
				  "wrong number of parameters for SetX");

			SetX(parameters[0].GetInteger());

            return true;

        } break;
    case FUNC_GETX:
        {
			if (parameters.size() != 0)
				throw IVM::IVMException(
				  "wrong number of parameters for GetX");

            return GetX();

        } break;
    case FUNC_SETY:
        {
			if (parameters.size() != 1)
				throw IVM::IVMException(
				  "wrong number of parameters for SetY");

			SetY(parameters[0].GetInteger());

            return true;

        } break;
    case FUNC_GETY:
        {
			if (parameters.size() != 0)
				throw IVM::IVMException(
				  "wrong number of parameters for GetY");

            return GetY();

        } break;
    case FUNC_GETWIDTH:
        {
			if (parameters.size() != 0)
				throw IVM::IVMException(
				  "wrong number of parameters for GetWidth");

            return GetWidth();

        } break;
    case FUNC_SETHEIGHT:
        {
			if (parameters.size() != 1)
				throw IVM::IVMException(
				  "wrong number of parameters for SetHeight");

			SetHeight(parameters[0].GetInteger());

            return true;

        } break;
    case FUNC_GETHEIGHT:
        {
			if (parameters.size() != 0)
				throw IVM::IVMException(
				  "wrong number of parameters for GetHeight");

            return GetHeight();

        } break;
    case FUNC_SETWIDTH:
        {
			if (parameters.size() != 1)
				throw IVM::IVMException(
				  "wrong number of parameters for SetWidth");

			SetWidth(parameters[0].GetInteger());

            return true;

        } break;
    case FUNC_SETMOUSEHANDLER:
        {
			switch (parameters.size())
			{
			case 2:
				{
					SetMouseHandler(new IVMMouseHandler(dynamic_cast<
						IVM::Script*>(parameters[0].GetObjPtr()),
						parameters[1].GetString()));
				} break;
			default:
				{
					throw IVM::IVMException(
					"wrong number of parameters for SetMouseHandler");
				}
			}

            return true;

        } break;
    case FUNC_GETMOUSEHANDLER:
        {
			if (parameters.size() != 0)
				throw IVM::IVMException(
				  "wrong number of parameters for GetMouseHandler");

			IVMMouseHandler * pHandler = dynamic_cast<IVMMouseHandler*>
				(GetMouseHandler());

			IVM::IVMVector * pVector = new IVM::IVMVector();
			pVector->Add(pHandler->GetScript());
			pVector->Add(pHandler->GetFuncname());

            return pVector;

        } break;
    case FUNC_SETKEYHANDLER:
        {
			switch (parameters.size())
			{
			case 2:
				{
					SetKeyHandler(new IVMKeyHandler(dynamic_cast<
						IVM::Script*>(parameters[0].GetObjPtr()),
						parameters[1].GetString()));
				} break;
			default:
				{
					throw IVM::IVMException(
					"wrong number of parameters for SetKeyHandler");
				}
			}

            return true;

        } break;
    case FUNC_GETKEYHANDLER:
        {
			if (parameters.size() != 0)
				throw IVM::IVMException(
				  "wrong number of parameters for GetKeyHandler");

			IVMKeyHandler * pHandler = dynamic_cast<IVMKeyHandler*>
				(GetKeyHandler());

			IVM::IVMVector * pVector = new IVM::IVMVector();
			pVector->Add(pHandler->GetScript());
			pVector->Add(pHandler->GetFuncname());

            return pVector;

        } break;
    case FUNC_SETCUSTOMDRAW:
        {
			switch (parameters.size())
			{
			case 2:
				{
					SetCustomDraw(new IVMCustomDraw(dynamic_cast<
						IVM::Script*>(parameters[0].GetObjPtr()),
						parameters[1].GetString()));
				} break;
			default:
				{
					throw IVM::IVMException(
					"wrong number of parameters for SetCustomDraw");
				}
			}

            return true;

        } break;
    case FUNC_GETCUSTOMDRAW:
        {
			if (parameters.size() != 0)
				throw IVM::IVMException(
				  "wrong number of parameters for GetCustomDraw");

			IVMCustomDraw * pHandler = dynamic_cast<IVMCustomDraw*>
				(GetCustomDraw());

			IVM::IVMVector * pVector = new IVM::IVMVector();
			pVector->Add(pHandler->GetScript());
			pVector->Add(pHandler->GetFuncname());

            return pVector;

        } break;
    case FUNC_SETDESTROYCALLBACK:
        {
			switch (parameters.size())
			{
			case 2:
				{
					SetDestroyCallback(new IVMDestroyCallback(dynamic_cast<
						IVM::Script*>(parameters[0].GetObjPtr()),
						parameters[1].GetString()));
				} break;
			default:
				{
					throw IVM::IVMException(
					"wrong number of parameters for SetDestroyCallback");
				}
			}

            return true;

        } break;
    case FUNC_GETDESTROYCALLBACK:
        {
			if (parameters.size() != 0)
				throw IVM::IVMException(
				  "wrong number of parameters for GetDestroyCallback");

			IVMDestroyCallback * pHandler = dynamic_cast<IVMDestroyCallback*>
				(GetDestroyCallback());

			IVM::IVMVector * pVector = new IVM::IVMVector();
			pVector->Add(pHandler->GetScript());
			pVector->Add(pHandler->GetFuncname());

            return pVector;

        } break;
    case FUNC_SETFRAMEUPDATE:
        {
			switch (parameters.size())
			{
			case 2:
				{
					SetFrameUpdate(new IVMFrameUpdate(dynamic_cast<
						IVM::Script*>(parameters[0].GetObjPtr()),
						parameters[1].GetString()));
				} break;
			default:
				{
					throw IVM::IVMException(
					"wrong number of parameters for SetFrameUpdate");
				}
			}

            return true;

        } break;
    case FUNC_GETFRAMEUPDATE:
        {
			if (parameters.size() != 0)
				throw IVM::IVMException(
				  "wrong number of parameters for GetFrameUpdate");

			IVMFrameUpdate * pHandler = dynamic_cast<IVMFrameUpdate*>
				(GetFrameUpdate());

			IVM::IVMVector * pVector = new IVM::IVMVector();
			pVector->Add(pHandler->GetScript());
			pVector->Add(pHandler->GetFuncname());

            return pVector;

        } break;
    }

	return true;
}

} // namespace Isis
