
#include <isis/util.h>
#include <isis/ivm.h>

#include <isis/game/GameApp.h>
#include <isis/game/GraphicsEngine.h>

namespace Isis {

void GameApp::FillExportTable()
{
    m_ExportTable["Init"]           = FUNC_INIT;
    m_ExportTable["Create"]         = FUNC_CREATE;
    m_ExportTable["Run"]            = FUNC_RUN;
    m_ExportTable["GetGraphicsEngine"] = FUNC_GETGRAPHICSENGINE;
    m_ExportTable["SetWindowTitle"] = FUNC_SETWINDOWTITLE;
    m_ExportTable["CheckModifier"]	= FUNC_CHECKMODIFIER;
}

void GameApp::IVMFillExportTable(std::vector<std::string> & table)
{
    std::map<std::string,FuncType>::iterator iter;

    for (iter = m_ExportTable.begin(); iter != m_ExportTable.end(); iter++)
    {
        table.push_back(iter->first);
    }
}

bool GameApp::IVMContainsMethod(const std::string & funcname)
{
    return m_ExportTable.end() != m_ExportTable.find(funcname);
}

IVM::Value GameApp::IVMCallMethod(const std::string & funcname,
                           const std::vector<IVM::Value> & parameters)
{   
    if (m_ExportTable.end() == m_ExportTable.find(funcname))
        throw IVM::IVMException("function not found in GameApp object");
    
    switch (m_ExportTable[funcname])
    {
    case FUNC_INIT:
        {
            if (parameters.size() >= 2)
			{
				m_pGraphicsEngine->SetWidth(parameters[0].GetInteger());
				m_pGraphicsEngine->SetHeight(parameters[1].GetInteger());
			}
			
			if (parameters.size() >= 3)
				m_pGraphicsEngine->SetWindowed(parameters[2].GetInteger() != 0);

			if (parameters.size() >= 4)
				SetWindowTitle(parameters[3].GetString());

        } break;
    case FUNC_CREATE:
        {
            if (parameters.size() != 0)
                throw IVM::IVMException(
                             "wrong number of parameters for Create");

            Create();

            return true;

        } break;
    case FUNC_RUN:
        {
            if (parameters.size() != 0)
                throw IVM::IVMException("wrong number of parameters for Run");

            Run();

            return true;

        } break;
    case FUNC_GETGRAPHICSENGINE:
        {
            if (parameters.size() != 0)
                throw IVM::IVMException(
				  "wrong number of parameters for GetGraphicsEngine");

            return GetGraphicsEngine();

        } break;
    case FUNC_SETWINDOWTITLE:
        {
            if (parameters.size() != 1)
                throw IVM::IVMException(
				  "wrong number of parameters for SetWindowTitle");

            SetWindowTitle(parameters[0].GetString());

            return true;

        } break;
    case FUNC_CHECKMODIFIER:
        {
            if (parameters.size() != 1)
                throw IVM::IVMException(
				  "wrong number of parameters for CheckModifier");

            return CheckModifier((ModifierKeys)parameters[0].GetInteger());

        } break;
    }

	return true;
}

} // namespace Isis

