
#include <SDL/SDL_opengl.h>

#include <isis/game/GraphicsEngine.h>

namespace Isis {

void GraphicsEngine::SetWidth(int iWidth)
{
    m_iWidth = iWidth;
}

int GraphicsEngine::GetWidth()
{
    return m_iWidth;
}

void GraphicsEngine::SetHeight(int iHeight)
{
    m_iHeight = iHeight;
}

int GraphicsEngine::GetHeight()
{
    return m_iHeight;
}

void GraphicsEngine::SetWindowed(bool bWindowed)
{
    m_bWindowed = bWindowed;
}

bool GraphicsEngine::GetWindowed()
{
    return m_bWindowed;
}

void GraphicsEngine::AddImagePrefix(const std::string & sImagePrefix)
{
	bool bFound = false;

	std::vector<std::string>::iterator iter;
	for (iter = m_ImagePrefixes.begin();
		 iter != m_ImagePrefixes.end();
		 iter++)
	{
		if (*iter == sImagePrefix)
		{
			bFound = true;
			break;
		}
	}

	if (!bFound)
		m_ImagePrefixes.push_back(sImagePrefix);
}

void GraphicsEngine::RemoveImagePrefix(const std::string & sImagePrefix)
{
	std::vector<std::string>::iterator iter;
	for (iter = m_ImagePrefixes.begin();
		 iter != m_ImagePrefixes.end();
		 iter++)
	{
		if (*iter == sImagePrefix)
		{
			m_ImagePrefixes.erase(iter);
			break;
		}
	}
}

std::vector<std::string> GraphicsEngine::GetImagePrefixes()
{
	return m_ImagePrefixes;
}

bool GraphicsEngine::IsTexMapEnabled()
{
    return m_bTexMapEnabled;
}

void GraphicsEngine::EnableTexMap(bool bEnable)
{
    if (m_bTexMapEnabled != bEnable)
    {
        if (bEnable)
            glEnable(GL_TEXTURE_2D);
        else
            glDisable(GL_TEXTURE_2D);

        m_bTexMapEnabled = bEnable;
    }
}

bool GraphicsEngine::IsOrtho()
{
    return m_bOrtho;
}

void GraphicsEngine::SetOrtho(bool bOrtho)
{
    if (m_bOrtho != bOrtho)
    {
        if (bOrtho)
        {
			glViewport(0,0,GetWidth(),GetHeight());
            glMatrixMode(GL_MODELVIEW);
            glLoadIdentity();
            glMatrixMode(GL_PROJECTION);
            glLoadIdentity();
            glOrtho(0,(double)m_iWidth,(double)m_iHeight,0,-1,1);
        }
        else
        {
            // nothing I guess
        }

        m_bOrtho = bOrtho;
    }
}

bool GraphicsEngine::IsDepth()
{
    return m_bDepth;
}

void GraphicsEngine::SetDepth(bool bDepth)
{
    if (m_bDepth != bDepth)
    {
        if (bDepth)
			glEnable(GL_DEPTH_TEST);
        else
			glDisable(GL_DEPTH_TEST);

        m_bDepth = bDepth;
    }
}

bool GraphicsEngine::IsBlend()
{
    return m_bBlend;
}

void GraphicsEngine::SetBlend(bool bBlend)
{
    if (m_bBlend != bBlend)
    {
        if (bBlend)
			glEnable(GL_BLEND);
        else
			glDisable(GL_BLEND);

        m_bBlend = bBlend;
    }
}

void GraphicsEngine::VRAMObjectCreated(VRAMObject * pObject)
{
    m_VRAMList.push_back(pObject);
}

void GraphicsEngine::VRAMObjectDestroyed(VRAMObject * pObject)
{
    m_VRAMList.remove(pObject);
}

} // namespace Isis

