
#include <SDL/SDL.h>
#include <SDL/SDL_opengl.h>

#include <isis/util.h>
#include <isis/ivm.h>

#include <isis/game/GraphicsEngine.h>

#include <isis/game/Image2D.h>
#include <isis/game/Texture.h>
#include <isis/game/Font2D.h>

#include <isis/game/VectorAnimation.h>

namespace Isis {

void GraphicsEngine::FillExportTable()
{
    m_ExportTable["CreateDisplay"]     = FUNC_CREATEDISPLAY;
    m_ExportTable["ToggleFullscreen"]  = FUNC_TOGGLEFULLSCREEN;
    m_ExportTable["Present"]           = FUNC_PRESENT;
    m_ExportTable["Clear"]             = FUNC_CLEAR;
	m_ExportTable["LoadIGZ"]		   = FUNC_LOADIGZ;
	m_ExportTable["LoadVGZ"]		   = FUNC_LOADVGZ;
	m_ExportTable["LoadFont"]		   = FUNC_LOADFONT;
    m_ExportTable["SetWidth"]          = FUNC_SETWIDTH;
    m_ExportTable["GetWidth"]          = FUNC_GETWIDTH;
    m_ExportTable["SetHeight"]         = FUNC_SETHEIGHT;
    m_ExportTable["GetHeight"]         = FUNC_GETHEIGHT;
    m_ExportTable["SetWindowed"]       = FUNC_SETWINDOWED;
    m_ExportTable["GetWindowed"]       = FUNC_GETWINDOWED;
	m_ExportTable["AddImagePrefix"]    = FUNC_ADDIMAGEPREFIX;
	m_ExportTable["RemoveImagePrefix"] = FUNC_REMOVEIMAGEPREFIX;
	m_ExportTable["GetImagePrefixes"]  = FUNC_GETIMAGEPREFIXES;
	m_ExportTable["IsTexMapEnabled"]   = FUNC_ISTEXMAPENABLED;
	m_ExportTable["EnableTexMap"]      = FUNC_ENABLETEXMAP;
	m_ExportTable["IsOrtho"]           = FUNC_ISORTHO;
	m_ExportTable["SetOrtho"]          = FUNC_SETORTHO;
	m_ExportTable["IsBlend"]           = FUNC_ISBLEND;
	m_ExportTable["SetBlend"]          = FUNC_SETBLEND;
	m_ExportTable["IsDepth"]           = FUNC_ISDEPTH;
	m_ExportTable["SetDepth"]          = FUNC_SETDEPTH;
	m_ExportTable["Color"]             = FUNC_COLOR;
	m_ExportTable["DrawRectangle"]     = FUNC_DRAWRECTANGLE;
	m_ExportTable["FillRectangle"]     = FUNC_FILLRECTANGLE;
	m_ExportTable["DrawLine"]          = FUNC_DRAWLINE;
	m_ExportTable["PushMatrix"]        = FUNC_PUSHMATRIX;
	m_ExportTable["PopMatrix"]         = FUNC_POPMATRIX;
	m_ExportTable["LookAt"]            = FUNC_LOOKAT;
	m_ExportTable["Perspective"]       = FUNC_PERSPECTIVE;
	m_ExportTable["Rotate"]            = FUNC_ROTATE;
	m_ExportTable["Translate"]         = FUNC_TRANSLATE;
	m_ExportTable["CullFront"]         = FUNC_CULLFRONT;
	m_ExportTable["CullBack"]          = FUNC_CULLBACK;
}

bool GraphicsEngine::IVMContainsMethod(const std::string & funcname)
{
    return m_ExportTable.end() != m_ExportTable.find(funcname);
}

void GraphicsEngine::IVMFillExportTable(std::vector<std::string> & table)
{
    std::map<std::string,FuncType>::iterator iter;

    for (iter = m_ExportTable.begin(); iter != m_ExportTable.end(); iter++)
    {
        table.push_back(iter->first);
    }
}

IVM::Value GraphicsEngine::IVMCallMethod(const std::string & funcname,
                           const std::vector<IVM::Value> & parameters)
{   
    if (m_ExportTable.end() == m_ExportTable.find(funcname))
        throw IVM::IVMException("function not found in GraphicsEngine object");
    
    switch (m_ExportTable[funcname])
    {
    case FUNC_CREATEDISPLAY:
        {
            if (parameters.size() != 0)
                throw IVM::IVMException(
                       "wrong number of parameters for CreateDisplay");

            CreateDisplay();

            return true;

        } break;
    case FUNC_TOGGLEFULLSCREEN:
        {
            if (parameters.size() != 0)
                throw IVM::IVMException(
                         "wrong number of parameters for ToggleFullscreen");

            ToggleFullscreen();

            return true;

        } break;
    case FUNC_PRESENT:
        {
            if (parameters.size() != 0)
                throw IVM::IVMException(
                               "wrong number of parameters for Present");

            Present();

            return true;

        } break;
    case FUNC_CLEAR:
	{
		if (parameters.size() > 1)
			throw IVM::IVMException(
						   "wrong number of parameters for Clear");
		
		if (parameters.size() == 1)
			Clear(parameters[0].GetInteger());
		else
			Clear();
		
		return true;
		
	} break;
    case FUNC_LOADIGZ:
	{
		if (parameters.size() != 2 && parameters.size() != 1)
			throw IVM::IVMException(
						   "wrong number of parameters for LoadIGZ");

		Texture * pTexture;

		if (parameters.size() == 2)
		{
			IVM::IVMVector * pVector = dynamic_cast<IVM::IVMVector*>
									(parameters[1].GetObjPtr());

			if (NULL == pVector)
				throw IVM::IVMException("NULL vector for LoadIGZ()");

			std::vector<Image2D*> stlVector;

			pTexture = LoadIGZ(parameters[0].GetString(),stlVector);

 			for (int i = 0; i < (int)stlVector.size(); i++)
				pVector->Add(stlVector[i]);
		}
		else
		{
			pTexture = LoadIGZ(parameters[0].GetString());
		}

		return pTexture;

	} break;
	case FUNC_LOADVGZ:
	{
		if (parameters.size() != 2 && parameters.size() != 1)
			throw IVM::IVMException(
						   "wrong number of parameters for LoadVGZ");

		if (parameters.size() == 2)
		{
			IVM::IVMVector * pVector = dynamic_cast<IVM::IVMVector*>
									(parameters[1].GetObjPtr());
			IVM::DataSet * pDataSet = new IVM::DataSet();

			if (NULL == pVector)
				throw IVM::IVMException("NULL vector for LoadVGZ()");

			std::vector<VectorAnimation*> stlVector;

			LoadVGZ(parameters[0].GetString(),stlVector);

 			for (int i = 0; i < (int)stlVector.size(); i++)
			{
				pVector->Add(stlVector[i]);
				pDataSet->Set(stlVector[i]->GetName(),stlVector[i]);
			}

			return pDataSet;
		}
		else
		{
			IVM::DataSet * pDataSet = new IVM::DataSet();
			std::vector<VectorAnimation*> stlVector;

			LoadVGZ(parameters[0].GetString(),stlVector);

 			for (int i = 0; i < (int)stlVector.size(); i++)
				pDataSet->Set(stlVector[i]->GetName(),stlVector[i]);

			return pDataSet;
		}

	} break;
    case FUNC_LOADFONT:
	{
		if (parameters.size() != 1)
			throw IVM::IVMException(
						   "wrong number of parameters for LoadFont");

		return LoadFont(parameters[0].GetString());

	} break;
	case FUNC_BINDTEXTURE:
	{
		if (parameters.size() != 1)
			throw IVM::IVMException(
						   "Wrong number of parameters for BindTexture");
		
		BindTexture(dynamic_cast<Texture *>(parameters[0].GetObjPtr()));
		
		return true;
		
	} break;
	case FUNC_REBINDCURRENTTEXTURE:
	{
		if (parameters.size() != 0)
			throw IVM::IVMException(
					"Wrong number of parameters for RebindCurrentTexture");
		
		RebindCurrentTexture();
		
		return true;

	} break;
    case FUNC_SETWIDTH:
        {
            if (parameters.size() != 1)
                throw IVM::IVMException(
                          "wrong number of parameters for SetWidth");

            int iWidth = parameters[0].GetInteger();

            SetWidth(iWidth);

            return true;

        } break;
    case FUNC_GETWIDTH:
        {
            if (parameters.size() != 0)
                throw IVM::IVMException(
                          "wrong number of parameters for GetWidth");

            return GetWidth();

        } break;
    case FUNC_SETHEIGHT:
        {
            if (parameters.size() != 1)
                throw IVM::IVMException(
                          "wrong number of parameters for SetHeight");

            int iHeight = parameters[0].GetInteger();

            SetWidth(iHeight);

            return true;

        } break;
    case FUNC_GETHEIGHT:
        {
            if (parameters.size() != 0)
                throw IVM::IVMException(
                         "wrong number of parameters for GetHeight");

            return GetHeight();

        } break;
    case FUNC_SETWINDOWED:
        {
            if (parameters.size() != 1)
                throw IVM::IVMException(
                         "wrong number of parameters for SetWindowed");

            bool bWindowed = (0 != parameters[0].GetInteger());

            SetWindowed(bWindowed);

            return true;

        } break;
    case FUNC_GETWINDOWED:
        {
            if (parameters.size() != 0)
                throw IVM::IVMException(
                             "wrong number of parameters for GetWindowed");

            return (int)GetWindowed();

        } break;
    case FUNC_ADDIMAGEPREFIX:
	    {
		    if (parameters.size() != 1)
			    throw IVM::IVMException(
						"wrong number of parameters for AddImagePrefix");
		
		    AddImagePrefix(parameters[0].GetString());
		
		    return true;
		
    	} break;
    case FUNC_REMOVEIMAGEPREFIX:
	    {
		    if (parameters.size() != 1)
			    throw IVM::IVMException(
						"wrong number of parameters for RemoveImagePrefix");
		
		    RemoveImagePrefix(parameters[0].GetString());
		
		    return true;
		
    	} break;
    case FUNC_GETIMAGEPREFIXES:
	    {
		    if (parameters.size() != 0)
			    throw IVM::IVMException(
						  "wrong number of parameters for GetImagePrefixes");

			IVM::IVMVector * pVector = new IVM::IVMVector();
			
			for (unsigned int i = 0; i < m_ImagePrefixes.size(); i++)
			{
				pVector->Add(m_ImagePrefixes[i]);
			}

		    return pVector;
		
	    } break;
    case FUNC_ISTEXMAPENABLED: {
            if (parameters.size() != 0)
                throw IVM::IVMException(
                     "wrong number of parameters for IsTexMapEnabled");

            return IsTexMapEnabled();

        } break;
    case FUNC_ENABLETEXMAP:
        {
            if (parameters.size() != 1)
                throw IVM::IVMException(
                     "wrong number of parameters for EnableTexMap");

            EnableTexMap(parameters[0].GetInteger() != 0);

            return true;

        } break;
    case FUNC_ISORTHO:
        {
            if (parameters.size() != 0)
                throw IVM::IVMException(
                     "wrong number of parameters for IsOrtho");

            return IsOrtho();

        } break;
    case FUNC_SETORTHO:
        {
            if (parameters.size() != 1)
                throw IVM::IVMException(
                     "wrong number of parameters for SetOrtho");

            SetOrtho(parameters[0].GetInteger() != 0);

            return true;

        } break;
    case FUNC_ISDEPTH:
        {
            if (parameters.size() != 0)
                throw IVM::IVMException(
                     "wrong number of parameters for IsDepth");

            return IsDepth();

        } break;
    case FUNC_SETDEPTH:
        {
            if (parameters.size() != 1)
                throw IVM::IVMException(
                     "wrong number of parameters for SetDepth");

            SetDepth(parameters[0].GetInteger() != 0);

            return true;

        } break;
    case FUNC_ISBLEND:
        {
            if (parameters.size() != 0)
                throw IVM::IVMException(
                     "wrong number of parameters for IsBlend");

            return IsBlend();

        } break;
    case FUNC_SETBLEND:
        {
            if (parameters.size() != 1)
                throw IVM::IVMException(
                     "wrong number of parameters for SetBlend");

            SetBlend(parameters[0].GetInteger() != 0);

            return true;

        } break;
    case FUNC_COLOR:
        {
			switch (parameters.size())
			{
			case 3:
				{
					return (int)Color(
						(unsigned int)parameters[0].GetInteger(),
						(unsigned int)parameters[1].GetInteger(),
						(unsigned int)parameters[2].GetInteger());
				} break;
			case 4:
				{
					return (int)Color(
						(unsigned int)parameters[0].GetInteger(),
						(unsigned int)parameters[1].GetInteger(),
						(unsigned int)parameters[2].GetInteger(),
						(unsigned int)parameters[3].GetInteger());
				} break;
			default:
				{
					throw IVM::IVMException(
						"wrong number of parameters for Color()");
				} break;
			}
        } break;
    case FUNC_DRAWRECTANGLE:
        {
            if (parameters.size() != 5)
                throw IVM::IVMException(
                     "wrong number of parameters for DrawRectangle");

            DrawRectangle(parameters[0].GetInteger(),
						  parameters[1].GetInteger(),
						  parameters[2].GetInteger(),
						  parameters[3].GetInteger(),
						  parameters[4].GetInteger());

            return true;

        } break;
    case FUNC_FILLRECTANGLE:
        {
            if (parameters.size() != 5)
                throw IVM::IVMException(
                     "wrong number of parameters for FillRectangle");

            FillRectangle(parameters[0].GetInteger(),
						  parameters[1].GetInteger(),
						  parameters[2].GetInteger(),
						  parameters[3].GetInteger(),
						  parameters[4].GetInteger());

            return true;

        } break;
    case FUNC_DRAWLINE:
        {
            if (parameters.size() != 5)
                throw IVM::IVMException(
                     "wrong number of parameters for DrawLine");

            DrawLine(parameters[0].GetInteger(),
					 parameters[1].GetInteger(),
					 parameters[2].GetInteger(),
					 parameters[3].GetInteger(),
					 parameters[4].GetInteger());

            return true;

        } break;
    case FUNC_PUSHMATRIX:
        {
            if (parameters.size() != 0)
                throw IVM::IVMException(
                     "wrong number of parameters for PushMatrix");

			glPushMatrix();

            return true;

        } break;
	case FUNC_POPMATRIX:
        {
            if (parameters.size() != 0)
                throw IVM::IVMException(
                     "wrong number of parameters for PopMatrix");

			glPopMatrix();

            return true;

        } break;
    case FUNC_PERSPECTIVE:
        {
            if (parameters.size() != 4)
                throw IVM::IVMException(
                     "wrong number of parameters for Perspective");

            glMatrixMode(GL_PROJECTION);
            glLoadIdentity();
			gluPerspective(parameters[0].GetFloat(),
						   parameters[1].GetFloat(),
						   parameters[2].GetFloat(),
						   parameters[3].GetFloat());
			glMatrixMode(GL_MODELVIEW);

            return true;

        } break;
    case FUNC_LOOKAT:
        {
            if (parameters.size() != 9)
                throw IVM::IVMException(
                     "wrong number of parameters for LookAt");

            glMatrixMode(GL_MODELVIEW);
            glLoadIdentity();
			gluLookAt(parameters[0].GetFloat(),
					  parameters[1].GetFloat(),
					  parameters[2].GetFloat(),
					  parameters[3].GetFloat(),
					  parameters[4].GetFloat(),
					  parameters[5].GetFloat(),
					  parameters[6].GetFloat(),
					  parameters[7].GetFloat(),
					  parameters[8].GetFloat());
			glMatrixMode(GL_MODELVIEW);

            return true;

        } break;
    case FUNC_ROTATE:
        {
            if (parameters.size() != 4)
                throw IVM::IVMException(
                     "wrong number of parameters for Rotate");

			glMatrixMode(GL_MODELVIEW);
			glRotatef(parameters[0].GetFloat(),
					  parameters[1].GetFloat(),
					  parameters[2].GetFloat(),
					  parameters[3].GetFloat());

            return true;

        } break;
    case FUNC_TRANSLATE:
        {
            if (parameters.size() != 3 && parameters.size() != 2)
                throw IVM::IVMException(
                     "wrong number of parameters for Translate");

			glMatrixMode(GL_MODELVIEW);

			if (parameters.size() == 3)
				glTranslatef(parameters[0].GetFloat(),
						    parameters[1].GetFloat(),
						    parameters[2].GetFloat());
			else if (parameters.size() == 2)
				glTranslatef(parameters[0].GetFloat(),
						    parameters[1].GetFloat(),
						    0);

            return true;

        } break;
    case FUNC_CULLFRONT:
        {
            if (parameters.size() != 0)
                throw IVM::IVMException(
                     "wrong number of parameters for PushMatrix");

			glCullFace(GL_FRONT);

            return true;

        } break;
	case FUNC_CULLBACK:
        {
            if (parameters.size() != 0)
                throw IVM::IVMException(
                     "wrong number of parameters for PopMatrix");

			glCullFace(GL_BACK);

            return true;

        } break;
    }

	return true;
}

} // namespace Isis

