
#include <isis/game/GraphicsEngine.h>
#include <isis/game/Texture.h>
#include <isis/game/CellImage.h>
#include <isis/game/Font2D.h>

namespace Isis {

void GraphicsEngine::FillObjectTable()
{
    m_ObjectTable["Texture"]   = OBJECT_TEXTURE;
    m_ObjectTable["CellImage"] = OBJECT_CELLIMAGE;
    m_ObjectTable["Font"]	   = OBJECT_FONT;
}

void GraphicsEngine::FillObjectTable(std::vector<std::string> & table)
{
    std::map<std::string,ObjectType>::iterator iter;

    for (iter = m_ObjectTable.begin(); iter != m_ObjectTable.end(); iter++)
    {
        table.push_back(iter->first);
    }
}

IVM::IVMObject * GraphicsEngine::CreateObject(const std::string & objname,
                                   const std::vector<IVM::Value> & InitParams)
{
    if (m_ObjectTable.end() == m_ObjectTable.find(objname))
        throw IVM::IVMException("object type not found");

    IVM::IVMObject * pObject = NULL;

    switch (m_ObjectTable[objname])
    {
    case OBJECT_TEXTURE:
        {
            pObject =  new Texture(this);
        } break;
    case OBJECT_CELLIMAGE:
        {
            pObject =  new CellImage();
        } break;
    case OBJECT_FONT:
        {
            pObject =  new Font2D(this);
        } break;
    }

	pObject->AddRef();

    if (pObject->IVMContainsMethod("Init"))
    {
        pObject->IVMCallMethod("Init",InitParams);
    } 
	else if (!InitParams.empty())
    {
        throw IVM::IVMException("Init() parameters given, but Init not found");
    }

	pObject->SafeRelease();
    return pObject;
}

} // namespace Isis


