
#include <vector>

#include <isis/game/GraphicsEngine.h>

#include <isis/game/IVMCustomDraw.h>

namespace Isis {

IVMCustomDraw::IVMCustomDraw(IVM::Script * pScript, 
								 const std::string & sFuncname)
{
	m_pScript = pScript;
	m_sFuncname = sFuncname;

	if (m_pScript)
		m_pScript->AddRef();
}

IVMCustomDraw::~IVMCustomDraw()
{
	if (m_pScript)
		m_pScript->Release();
}

void IVMCustomDraw::Draw(int x, int y, int width, int height,
						 GraphicsEngine * pGraphicsEngine)
{
	if (m_pScript)
	{
		std::vector<IVM::Value> params;
		params.push_back(x);
		params.push_back(y);
		params.push_back(width);
		params.push_back(height);
		params.push_back(pGraphicsEngine);

		m_pScript->ExecSync(m_sFuncname,params);
	}
}

IVM::Script * IVMCustomDraw::GetScript()
{
	return m_pScript;
}

std::string IVMCustomDraw::GetFuncname()
{
	return m_sFuncname;
}

} // namespace Isis

