
#include <SDL/SDL.h>

#include <isis/game/GraphicsEngine.h>
#include <isis/game/SoundEngine.h>

#include "IVMGUIGame.h"

namespace Isis {

std::map<std::string,IVMGUIGame::FuncType> IVMGUIGame::m_ExportTable3;

IVMGUIGame::IVMGUIGame()
{
	m_pCurScript = NULL;
	
	if (m_ExportTable3.empty())
		FillExportTable3();
}

IVMGUIGame::~IVMGUIGame()
{
}

void IVMGUIGame::FillExportTable3()
{
    m_ExportTable3["RunScript"]         = FUNC_RUNSCRIPT;
}

bool IVMGUIGame::IVMContainsMethod(const std::string & funcname)
{
    return m_ExportTable.end() != m_ExportTable.find(funcname) ||
		   m_ExportTable2.end() != m_ExportTable2.find(funcname) ||
		   m_ExportTable3.end() != m_ExportTable3.find(funcname);
}

void IVMGUIGame::IVMFillExportTable(std::vector<std::string> & table)
{
	std::map<std::string,GameApp::FuncType>::iterator iter1;
	
    for (iter1 = m_ExportTable.begin(); iter1 != m_ExportTable.end(); iter1++)
    {
        table.push_back(iter1->first);
    }

	std::map<std::string,GUIGame::FuncType>::iterator iter2;
	
    for (iter2 = m_ExportTable2.begin(); iter2 != m_ExportTable2.end(); iter2++)
    {
        table.push_back(iter2->first);
    }

	std::map<std::string,FuncType>::iterator iter3;
	
    for (iter3 = m_ExportTable3.begin(); iter3 != m_ExportTable3.end(); iter3++)
    {
        table.push_back(iter3->first);
    }
}

IVM::Value IVMGUIGame::IVMCallMethod(const std::string & funcname,
							const std::vector<IVM::Value> & parameters)
{
	if (m_ExportTable.end() != m_ExportTable.find(funcname))
		return GameApp::IVMCallMethod(funcname,parameters);
	
	if (m_ExportTable2.end() != m_ExportTable2.find(funcname))
		return GUIGame::IVMCallMethod(funcname,parameters);
	
    if (m_ExportTable3.end() == m_ExportTable3.find(funcname))
        throw IVM::IVMException("Method not found in IVMGUIGame");
	
    switch (m_ExportTable3[funcname])
    {
	case FUNC_RUNSCRIPT:
	    {
			if (parameters.size() != 1)
				throw IVM::IVMException(
								"wrong number of parameters for RunScript");
			
			RunScript(dynamic_cast<IVM::Script*>(parameters[0].GetObjPtr()));
			
			return true;
			
		} break;	
    }

	return true;
}

void IVMGUIGame::RunScript(IVM::Script * pScriptObject)
{
	m_pCurScript = pScriptObject;
	m_pCurScript->AddRef();

    IVM::IsisVM * pVM = pScriptObject->GetParentVM();
	pVM->RegisterHAPIProvider(this);
	pVM->RegisterObjectProvider(this);
    pVM->RegisterHAPIProvider(m_pGraphicsEngine);
	pVM->RegisterObjectProvider(m_pGraphicsEngine);

	pVM->RegisterHAPIProvider(m_pSoundEngine);
	pVM->RegisterObjectProvider(m_pSoundEngine);

	Create();
	Run();

	m_pCurScript->Release();
	m_pCurScript = NULL;
}

void IVMGUIGame::GUIInit()
{
	if (m_pCurScript->ContainsFunction("GUIInit"))
	{
	    std::vector<IVM::Value> params;
		m_pCurScript->ExecSync("GUIInit",params);
	}
}

bool IVMGUIGame::GUIMain()
{
	if (m_pCurScript->ContainsFunction("GUIMain"))
	{
		std::vector<IVM::Value> params;
		return m_pCurScript->IVMCallMethod("GUIMain",params).GetInteger()!=0;
	}
	else
		return true;
}

void IVMGUIGame::GUIShutdown()
{
	if (m_pCurScript->ContainsFunction("GUIShutdown"))
	{
		std::vector<IVM::Value> params;
		m_pCurScript->ExecSync("GUIShutdown",params);
	}
}

bool IVMGUIGame::UnhandledException(IsisException & ex)
{
	if (m_pCurScript->ContainsFunction("UnhandledException"))
	{
		std::vector<IVM::Value> params;
		params.push_back(ex.GetMessage());

		return (m_pCurScript->IVMCallMethod(
					"UnhandledException",params).GetInteger() != 0);
	}
	else 
		return false;
}

void IVMGUIGame::EmptyClick(GUIGame::ClickType click, int x, int y)
{
	if (m_pCurScript->ContainsFunction("EmptyClick"))
	{
		std::vector<IVM::Value> params;
		params.push_back((int)click);
		params.push_back(x);
		params.push_back(y);

		m_pCurScript->IVMCallMethod("EmptyClick",params);
	}
}

} // namespace Isis
