
#include <SDL/SDL.h>

#include <isis/game/GraphicsEngine.h>
#include <isis/game/SoundEngine.h>

#include "IVMGameApp.h"

namespace Isis {

std::map<std::string,IVMGameApp::FuncType> IVMGameApp::m_ExportTable2;

IVMGameApp::IVMGameApp()
{
	m_pCurScript = NULL;
	
	if (m_ExportTable2.empty())
		FillExportTable2();
}

IVMGameApp::~IVMGameApp()
{
}

void IVMGameApp::FillExportTable2()
{
    m_ExportTable2["RunScript"]         = FUNC_RUNSCRIPT;
}

bool IVMGameApp::IVMContainsMethod(const std::string & funcname)
{
    return m_ExportTable.end() != m_ExportTable.find(funcname) ||
		   m_ExportTable2.end() != m_ExportTable2.find(funcname);
}

void IVMGameApp::IVMFillExportTable(std::vector<std::string> & table)
{
	std::map<std::string,GameApp::FuncType>::iterator iter1;
	
    for (iter1 = m_ExportTable.begin(); iter1 != m_ExportTable.end(); iter1++)
    {
        table.push_back(iter1->first);
    }

	std::map<std::string,IVMGameApp::FuncType>::iterator iter2;
	
    for (iter2 = m_ExportTable2.begin(); iter2 != m_ExportTable2.end(); iter2++)
    {
        table.push_back(iter2->first);
    }
}

IVM::Value IVMGameApp::IVMCallMethod(const std::string & funcname,
							const std::vector<IVM::Value> & parameters)
{   
	if (m_ExportTable.end() != m_ExportTable.find(funcname))
		return GameApp::IVMCallMethod(funcname,parameters);

    if (m_ExportTable2.end() == m_ExportTable2.find(funcname))
        throw IVM::IVMException("Method not found in IVMGameApp");
	
    switch (m_ExportTable2[funcname])
    {
	case FUNC_RUNSCRIPT:
	    {
			if (parameters.size() != 1)
				throw IVM::IVMException(
								"wrong number of parameters for RunScript");
			
			RunScript(dynamic_cast<IVM::Script*>(parameters[0].GetObjPtr()));
			
			return true;
			
		} break;	
    }

	return true;
}

void IVMGameApp::RunScript(IVM::Script * pScriptObject)
{
	m_pCurScript = pScriptObject;
	m_pCurScript->AddRef();

    IVM::IsisVM * pVM = pScriptObject->GetParentVM();
    pVM->RegisterHAPIProvider(this);
    pVM->RegisterHAPIProvider(m_pGraphicsEngine);
	pVM->RegisterObjectProvider(m_pGraphicsEngine);
	
	pVM->RegisterHAPIProvider(m_pSoundEngine);
	pVM->RegisterObjectProvider(m_pSoundEngine);

	Create();
	Run();

	m_pCurScript->Release();
	m_pCurScript = NULL;
}

void IVMGameApp::GameInit()
{
	if (m_pCurScript->ContainsFunction("GameInit"))
	{
	    std::vector<IVM::Value> params;
		m_pCurScript->ExecSync("GameInit",params);
	}
}

bool IVMGameApp::GameMain()
{
	int retval = 1;

	if (m_pCurScript->ContainsFunction("GameMain"))
	{
		std::vector<IVM::Value> params;
		m_pCurScript->ExecSync("GameMain",params);
		retval = m_pCurScript->GetRetVal().GetInteger();
	}

	return (retval != 0);
}

void IVMGameApp::GameShutdown()
{
	if (m_pCurScript->ContainsFunction("GameShutdown"))
	{
		std::vector<IVM::Value> params;
		m_pCurScript->ExecSync("GameShutdown",params);
	}
}

void IVMGameApp::KeyPressed(int key, ModifierKeys modifiers)
{
	if (m_pCurScript->ContainsFunction("KeyPressed"))
	{
		std::vector<IVM::Value> params;
		char arr[2] = {(char)key,0};
		std::string str = arr;
		params.push_back(str);
		params.push_back((int)modifiers);
		m_pCurScript->ExecSync("KeyPressed",params);
	}
}

void IVMGameApp::KeyReleased(int key, ModifierKeys modifiers)
{
	if (m_pCurScript->ContainsFunction("KeyReleased"))
	{
		std::vector<IVM::Value> params;
		char arr[2] = {(char)key,0};
		std::string str = arr;
		params.push_back(str);
		params.push_back((int)modifiers);
		m_pCurScript->ExecSync("KeyReleased",params);
	}
}

void IVMGameApp::MouseMoved(int x, int y)
{
	if (m_pCurScript->ContainsFunction("MouseMoved"))
	{
		std::vector<IVM::Value> params;
		params.push_back(y);
		params.push_back(x);
		m_pCurScript->ExecSync("MouseMoved",params);
	}
}

void IVMGameApp::MouseClicked(ClickType click, int x, int y)
{
	if (m_pCurScript->ContainsFunction("MouseClicked"))
	{
		std::vector<IVM::Value> params;
		params.push_back(y);
		params.push_back(x);
		params.push_back((int)click);
		m_pCurScript->ExecSync("MouseClicked",params);
	}
}

} // namespace Isis
