
#include <vector>

#include <isis/game/IVMKeyHandler.h>

namespace Isis {

IVMKeyHandler::IVMKeyHandler(IVM::Script * pScript, 
								 const std::string & sFuncname)
{
	m_pScript = pScript;
	m_sFuncname = sFuncname;

	if (m_pScript)
		m_pScript->AddRef();
}

IVMKeyHandler::~IVMKeyHandler()
{
	if (m_pScript)
		m_pScript->Release();
}

void IVMKeyHandler::HandleKey(int key, GameApp::ModifierKeys modifiers,
							  bool pressed)
{
	if (m_pScript)
	{
		std::vector<IVM::Value> params;
		char arr[2] = {(char)key,0};
		params.push_back(std::string(arr));
		params.push_back(key);
		params.push_back((int)modifiers);
		params.push_back(pressed);

		m_pScript->ExecSync(m_sFuncname,params);
	}
}

IVM::Script * IVMKeyHandler::GetScript()
{
	return m_pScript;
}

std::string IVMKeyHandler::GetFuncname()
{
	return m_sFuncname;
}

} // namespace Isis

