
#include <isis/ivm/IVMException.h>
#include <isis/ivm/IsisVM.h>

#include <isis/game/GraphicsEngine.h>
#include <isis/game/SoundEngine.h>
#include <isis/game/Sound.h>
#include <isis/game/Texture.h>
#include <isis/game/CellImage.h>
#include <isis/game/Font2D.h>

#include <isis/game/ParticleManager.h>
#include <isis/game/ParticleSource.h>

#include <isis/game/IsisGameProvider.h>

#include "IVMGameApp.h"
#include "IVMGUIGame.h"

namespace Isis {

std::map<std::string,IsisGameProvider::ObjectType> 
							IsisGameProvider::m_ObjectTable;

IsisGameProvider::IsisGameProvider()
{
    if (m_ObjectTable.empty())
        FillTable();
}

void IsisGameProvider::FillTable()
{
    m_ObjectTable["GameApp"]			= OBJECT_IVMGAMEAPP;
    m_ObjectTable["GUIGame"]			= OBJECT_IVMGUIGAME;
    m_ObjectTable["GraphicsEngine"]		= OBJECT_GRAPHICSENGINE;
	m_ObjectTable["SoundEngine"]		= OBJECT_SOUNDENGINE;
	m_ObjectTable["ParticleManager"]	= OBJECT_PARTICLEMANAGER;
	m_ObjectTable["ParticleSource"]		= OBJECT_PARTICLESOURCE;
}

void IsisGameProvider::FillObjectTable(std::vector<std::string> & table)
{
    std::map<std::string,ObjectType>::iterator iter;

    for (iter = m_ObjectTable.begin(); iter != m_ObjectTable.end(); iter++)
    {
        table.push_back(iter->first);
    }
}

IVM::IVMObject * IsisGameProvider::CreateObject(const std::string & objname,
                                   const std::vector<IVM::Value> & InitParams)
{
    if (m_ObjectTable.end() == m_ObjectTable.find(objname))
        throw IVM::IVMException("object type not found IsisGameProvider");

    IVM::IVMObject * pObject;

    switch (m_ObjectTable[objname])
    {
    case OBJECT_IVMGAMEAPP:
        {
            pObject =  new IVMGameApp();
        } break;
    case OBJECT_IVMGUIGAME:
        {
			pObject =  new IVMGUIGame();
        } break;
    case OBJECT_GRAPHICSENGINE:
        {
            pObject =  new GraphicsEngine();
        } break;
	case OBJECT_SOUNDENGINE:
        {
            pObject =  new SoundEngine();
        } break;
    case OBJECT_PARTICLEMANAGER:
        {
            pObject = new ParticleManager();
        } break;
    case OBJECT_PARTICLESOURCE:
        {
            pObject = new ParticleSource();
        } break;
    default:
        {
			throw IVM::IVMException("object type not implemented: ",objname);
        }
    }

	pObject->AddRef();

    if (pObject->IVMContainsMethod("Init"))
    {
        pObject->IVMCallMethod("Init",InitParams);
    } 
	else if (!InitParams.empty())
    {
        throw IVM::IVMException("Init() parameters given, but Init not found");
    }

	pObject->SafeRelease();
    return pObject;
}

} // namespace Isis


