
#include <isis/game/ParticleManager.h>
#include <isis/game/ParticleSource.h>

namespace Isis {

void ParticleManager::FillExportTable()
{
    m_ExportTable["Init"]			= FUNC_INIT;
    m_ExportTable["AddSource"]		= FUNC_ADDSOURCE;
    m_ExportTable["Update"]			= FUNC_UPDATE;
    m_ExportTable["Draw"]			= FUNC_DRAW;
}

void ParticleManager::IVMFillExportTable(std::vector<std::string> & table)
{
    std::map<std::string,FuncType>::iterator iter;

    for (iter = m_ExportTable.begin(); iter != m_ExportTable.end(); iter++)
    {
        table.push_back(iter->first);
    }
}

bool ParticleManager::IVMContainsMethod(const std::string & funcname)
{
    return m_ExportTable.end() != m_ExportTable.find(funcname);
}

IVM::Value ParticleManager::IVMCallMethod(const std::string & funcname,
                           const std::vector<IVM::Value> & parameters)
{   
    if (m_ExportTable.end() == m_ExportTable.find(funcname))
		throw IVM::IVMException("function not found in ParticleManager object: ",
								funcname);
    
    switch (m_ExportTable[funcname])
    {
    case FUNC_INIT:
        {
            if (parameters.size() != 0)
                throw IVM::IVMException(
                       "wrong number of parameters for Init");

			Init();

			return true;

		} break;
    case FUNC_ADDSOURCE:
        {
            if (parameters.size() != 1)
                throw IVM::IVMException(
                         "wrong number of parameters for AddSource");

			AddSource(dynamic_cast<ParticleSource*>(parameters[0].GetObjPtr()));

            return true;

        } break;
    case FUNC_UPDATE:
        {
            if (parameters.size() != 0)
                throw IVM::IVMException(
                         "wrong number of parameters for Update");

			Update();

            return true;

        } break;
    case FUNC_DRAW:
        {
            if (parameters.size() != 1)
                throw IVM::IVMException(
                         "wrong number of parameters for Draw");

			Draw(dynamic_cast<Isis::GraphicsEngine*>(parameters[0].GetObjPtr()));

            return true;

        } break;
    }

	return true;
}

} // namespace Isis

