
#include <isis/game/ParticleSource.h>

#define ALPHA(c)	((unsigned char)((c)>>24))
#define RED(c)		((unsigned char)(((c)&0x00FF0000)>>16))
#define GREEN(c)	((unsigned char)(((c)&0x0000FF00)>>8))
#define BLUE(c)		((unsigned char)((c)&0x000000FF))

#define COLOR(a,r,g,b)	((unsigned int)(((a)<<24)|((r)<<16)|((g)<<8)|(b)))

namespace Isis {

#ifdef _DEBUG
int ParticleSource::m_iGlobalRefCount = 0;
#endif // _DEBUG

std::map<std::string,ParticleSource::FuncType> ParticleSource::m_ExportTable;

ParticleSource::ParticleSource()
{
    if (m_ExportTable.empty())
        FillExportTable();

#ifdef _DEBUG
	m_iGlobalRefCount++;
#endif // _DEBUG
}

ParticleSource::~ParticleSource()
{
#ifdef _DEBUG
	m_iGlobalRefCount--;
#endif // _DEBUG
}

#ifdef _DEBUG
int ParticleSource::GetGlobalRefCount()
{
	return m_iGlobalRefCount;
}
#endif // _DEBUG

void ParticleSource::Init(const Desc & desc)
{
	if (desc.colors.empty())
		throw Isis::IsisException(
				"Colors cannot be empty in ParticleSource::Init()");

	m_Desc.expire_time	= desc.expire_time;
	m_Desc.active		= desc.active;
	m_Desc.x			= desc.x;
	m_Desc.y			= desc.y;
	m_Desc.xv			= desc.xv;
	m_Desc.yv			= desc.yv;
	m_Desc.xa			= desc.xa;
	m_Desc.ya			= desc.ya;
	m_Desc.p_prob		= desc.p_prob;
	m_Desc.p_life_min	= desc.p_life_min;
	m_Desc.p_life_max	= desc.p_life_max;
	m_Desc.p_size_min	= desc.p_size_min;
	m_Desc.p_size_max	= desc.p_size_max;
	m_Desc.p_xv_min		= desc.p_xv_min;
	m_Desc.p_xv_max		= desc.p_xv_max;
	m_Desc.p_yv_min		= desc.p_yv_min;
	m_Desc.p_yv_max		= desc.p_yv_max;
	m_Desc.p_xa			= desc.p_xa;
	m_Desc.p_ya			= desc.p_ya;

	for (int i = 0; i < (int)desc.colors.size(); i++)
		m_Desc.colors.push_back(desc.colors[i]);
}

ParticleSource::Desc & ParticleSource::GetDesc()
{
	return m_Desc;
}

void ParticleSource::Stop()
{
	m_Desc.active = false;
}

void ParticleSource::Start()
{
	m_Desc.active = true;
}

void ParticleSource::Destroy()
{
	m_Desc.expire_time = 0;
}

unsigned int ParticleSource::CalcColor(int time)
{
	if (time <= 0)
		return m_Desc.colors[0].first;

	for (int sum = 0, i = 0; i < (int)m_Desc.colors.size()-1; i++)
	{
		if (time >= sum && time < sum + m_Desc.colors[i].second)
		{
			unsigned int color1 = m_Desc.colors[i].first;
			unsigned int color2 = m_Desc.colors[i+1].first;
			float frac1 = (float)(time-sum) / (float)m_Desc.colors[i].second;
			float frac2 = 1 - frac1;

			return COLOR(
					(unsigned char)(frac2*ALPHA(color1)+frac1*ALPHA(color2)),
					(unsigned char)(frac2*RED(color1)  +frac1*RED(color2)),
					(unsigned char)(frac2*GREEN(color1)+frac1*GREEN(color2)),
					(unsigned char)(frac2*BLUE(color1) +frac1*BLUE(color2))
					);
		}
		else
			sum += m_Desc.colors[i].second;
	}

	return m_Desc.colors[m_Desc.colors.size()-1].first;
}

} // namespace Isis
