
#include <isis/game/ParticleSource.h>

namespace Isis {

void ParticleSource::FillExportTable()
{
    m_ExportTable["Init"]			= FUNC_INIT;
    m_ExportTable["Stop"]			= FUNC_STOP;
    m_ExportTable["Start"]			= FUNC_START;
    m_ExportTable["Destroy"]		= FUNC_DESTROY;
    m_ExportTable["CalcColor"]		= FUNC_CALCCOLOR;
    m_ExportTable["SetX"]			= FUNC_SETX;
    m_ExportTable["SetY"]			= FUNC_SETY;
    m_ExportTable["GetX"]			= FUNC_GETX;
    m_ExportTable["GetY"]			= FUNC_GETY;
    m_ExportTable["SetActive"]		= FUNC_SETACTIVE;
}

void ParticleSource::IVMFillExportTable(std::vector<std::string> & table)
{
    std::map<std::string,FuncType>::iterator iter;

    for (iter = m_ExportTable.begin(); iter != m_ExportTable.end(); iter++)
    {
        table.push_back(iter->first);
    }
}

bool ParticleSource::IVMContainsMethod(const std::string & funcname)
{
    return m_ExportTable.end() != m_ExportTable.find(funcname);
}

IVM::Value ParticleSource::IVMCallMethod(const std::string & funcname,
                           const std::vector<IVM::Value> & parameters)
{   
    if (m_ExportTable.end() == m_ExportTable.find(funcname))
		throw IVM::IVMException("function not found in ParticleSource object: ",
								funcname);
    
    switch (m_ExportTable[funcname])
    {
    case FUNC_INIT:
        {
            if (parameters.size() != 20)
                throw IVM::IVMException(
                       "wrong number of parameters for Init");

			Desc desc;
			desc.expire_time	= parameters[0].GetInteger();
			desc.active			= parameters[1].GetInteger() != 0;
			desc.x				= parameters[2].GetFloat();
			desc.y				= parameters[3].GetFloat();
			desc.xv				= parameters[4].GetFloat();
			desc.yv				= parameters[5].GetFloat();
			desc.xa				= parameters[6].GetFloat();
			desc.ya				= parameters[7].GetFloat();
			desc.p_prob			= parameters[8].GetFloat();
			desc.p_life_min		= parameters[9].GetInteger();
			desc.p_life_max		= parameters[10].GetInteger();
			desc.p_size_min		= parameters[11].GetFloat();
			desc.p_size_max		= parameters[12].GetFloat();
			desc.p_xv_min		= parameters[13].GetFloat();
			desc.p_xv_max		= parameters[14].GetFloat();
			desc.p_yv_min		= parameters[15].GetFloat();
			desc.p_yv_max		= parameters[16].GetFloat();
			desc.p_xa			= parameters[17].GetFloat();
			desc.p_ya			= parameters[18].GetFloat();

			IVM::IVMVector * pVector = dynamic_cast<IVM::IVMVector*>
											(parameters[19].GetObjPtr());

			for (int i = 0; i < pVector->Size(); i++)
			{
				std::pair<unsigned int, int> p;

				IVM::IVMVector * pPair = dynamic_cast<IVM::IVMVector*>
											(pVector->Get(i).GetObjPtr());

				p.first = pPair->Get(0).GetInteger();
				p.second = pPair->Get(1).GetInteger();

				desc.colors.push_back(p);
			}

			Init(desc);

			return true;

		} break;
	case FUNC_STOP:
		{
            if (parameters.size() != 0)
                throw IVM::IVMException(
                       "wrong number of parameters for Stop");

			Stop();

			return true;

		} break;
	case FUNC_START:
		{
            if (parameters.size() != 0)
                throw IVM::IVMException(
                       "wrong number of parameters for Start");

			Start();

			return true;

		} break;
	case FUNC_DESTROY:
		{
            if (parameters.size() != 0)
                throw IVM::IVMException(
                       "wrong number of parameters for Destroy");

			Destroy();

			return true;

		} break;
	case FUNC_CALCCOLOR:
		{
            if (parameters.size() != 1)
                throw IVM::IVMException(
                       "wrong number of parameters for CalcColor");

			CalcColor(parameters[0].GetInteger());

			return true;

		} break;
	case FUNC_SETX:
		{
            if (parameters.size() != 1)
                throw IVM::IVMException(
                       "wrong number of parameters for SetX");

			GetDesc().x = parameters[0].GetFloat();

			return true;

		} break;
	case FUNC_SETY:
		{
            if (parameters.size() != 1)
                throw IVM::IVMException(
                       "wrong number of parameters for SetY");

			GetDesc().y = parameters[0].GetFloat();

			return true;

		} break;
	case FUNC_GETX:
		{
            if (parameters.size() != 0)
                throw IVM::IVMException(
                       "wrong number of parameters for GetX");

			return GetDesc().x;

		} break;
	case FUNC_GETY:
		{
            if (parameters.size() != 0)
                throw IVM::IVMException(
                       "wrong number of parameters for GetY");

			return GetDesc().y;

		} break;
	case FUNC_SETACTIVE:
		{
            if (parameters.size() != 1)
                throw IVM::IVMException(
                       "wrong number of parameters for SetActive");

			GetDesc().active = parameters[0].GetInteger() != 0;

			return true;

		} break;
    }

	return true;
}

} // namespace Isis

