
#include <SDL/SDL_mixer.h>
#include <SDL/SDL.h>
#include <ctype.h>

#include <isis/game/Sound.h>
#include <isis/game/SoundEngine.h>

namespace Isis {

#ifdef _DEBUG
int Sound::m_iGlobalRefCount = 0;
#endif // _DEBUG

std::map<std::string,Sound::FuncType> Sound::m_ExportTable;

Sound::Sound(SoundEngine * pSoundEngine)
{
	m_pSound=NULL;
	m_iVolume=MIX_MAX_VOLUME/2;
    if (m_ExportTable.empty())
        FillExportTable();

	m_pSoundEngine = pSoundEngine;
	m_pSoundEngine->AddRef();

#ifdef _DEBUG
	m_iGlobalRefCount++;
#endif // _DEBUG
}

Sound::~Sound()
{
	if(m_pSound)
		m_pSoundEngine->FreeChunk(m_pSound); 
	m_pSound=NULL;
	
	if (m_pSoundEngine)
		m_pSoundEngine->Release();

#ifdef _DEBUG
	m_iGlobalRefCount--;
#endif // _DEBUG
}

#ifdef _DEBUG
int Sound::GetGlobalRefCount()
{
	return m_iGlobalRefCount;
}
#endif // _DEBUG


void Sound::Init(const std::string & sWAVEFilename) 
{
	if (m_pSound)
		throw IsisException("Already loaded sound in Load");
	m_sFileName=sWAVEFilename;
	m_pSound=Mix_LoadWAV(sWAVEFilename.c_str());
	if(!m_pSound)
		throw IsisException
		("Couldn't load sound: \"",sWAVEFilename+"\""+SDL_GetError());
#ifdef _DEBUG
	else
		printf("loaded sound: %s\n",sWAVEFilename.c_str());
#endif
}
void Sound::Play()
{
	
	m_pSoundEngine->Play(this,0);
}


void Sound::SetVolume(int Volume)
{
	if(Volume>MIX_MAX_VOLUME)
		m_iVolume=MIX_MAX_VOLUME;
	else if(Volume<0)
		m_iVolume=0;
	else
		m_iVolume=Volume;
	Mix_VolumeChunk(m_pSound,m_iVolume);
}

int Sound::GetVolume()
{
	return m_iVolume;
}


Mix_Chunk *Sound::GetChunk()
{
	return m_pSound;
}
std::string Sound::GetName() 
{
	return m_sFileName;
}

} // namespace Isis
