
#include <isis/util.h>
#include <isis/ivm.h>

#include <isis/game/SoundEngine.h>
#include <isis/game/Sound.h>

namespace Isis {

void SoundEngine::FillExportTable()
{
    m_ExportTable["InitAudop"]				= FUNC_INITAUDIO;
    m_ExportTable["LoadSound"]				= FUNC_LOADSOUND;
    m_ExportTable["Play"]					= FUNC_PLAY;
    m_ExportTable["PlayMusic"]				= FUNC_PLAYMUSIC;
	m_ExportTable["HookMusicFinsihed"]		= FUNC_HOOKMUSICFINISHED;
	m_ExportTable["SetSoundVolume"]			= FUNC_SETSOUNDVOLUME;
	m_ExportTable["SetMusicVolume"]			= FUNC_SETMUSICVOLUME;
	m_ExportTable["GetSoundVolume"]			= FUNC_GETSOUNDVOLUME;
	m_ExportTable["GetMusicVolume"]			= FUNC_GETMUSICVOLUME;
}

bool SoundEngine::IVMContainsMethod(const std::string & funcname)
{
    return m_ExportTable.end() != m_ExportTable.find(funcname);
}

void SoundEngine::IVMFillExportTable(std::vector<std::string> & table)
{
    std::map<std::string,FuncType>::iterator iter;

    for (iter = m_ExportTable.begin(); iter != m_ExportTable.end(); iter++)
    {
        table.push_back(iter->first);
    }
}

IVM::Value SoundEngine::IVMCallMethod(const std::string & funcname,
                           const std::vector<IVM::Value> & parameters)
{   
    if (m_ExportTable.end() == m_ExportTable.find(funcname))
        throw IVM::IVMException("function not found in SoundEngine object");
    
    switch (m_ExportTable[funcname])
    {
    case FUNC_INITAUDIO:
        {
            if (parameters.size() != 0)
                throw IVM::IVMException(
                       "wrong number of parameters for CreateDisplay");

            InitAudio();

            return true;

        } break;
    case FUNC_LOADSOUND:
        {
            if (parameters.size() != 1)
                throw IVM::IVMException(
                         "wrong number of parameters for LoadSound");

            Sound *pSound=LoadSound(parameters[0].GetString());
			
            return pSound;

        } break;
    case FUNC_PLAY:
        {
            if (parameters.size() == 0 || parameters.size() > 2)
                throw IVM::IVMException(
                               "wrong number of parameters for Play");

			Sound * pSound = dynamic_cast<Sound *>
								(parameters[0].GetObjPtr());
			if(parameters.size()==2)
				Play(pSound,0);
			//else
			//	Play(pSound);

            return true;

        } break;
	case FUNC_SETSOUNDVOLUME:
        {
            if (parameters.size() != 1)
                throw IVM::IVMException(
                       "wrong number of parameters for SetSoundVolume");

			SetSoundVolume(parameters[0].GetInteger());
			
		} break;
		  case FUNC_SETMUSICVOLUME:
        {
            if (parameters.size() != 1)
                throw IVM::IVMException(
                       "wrong number of parameters for SetMusicVolume");

			SetMusicVolume(parameters[0].GetInteger());
			
		} break;
		  case FUNC_GETSOUNDVOLUME:
        {
            if (parameters.size() != 0)
                throw IVM::IVMException(
                       "wrong number of parameters for GetSoundVolume");
			int result;
			result=GetSoundVolume();
			return result;
			
		} break;
		case FUNC_GETMUSICVOLUME:
        {
            if (parameters.size() != 0)
                throw IVM::IVMException(
                       "wrong number of parameters for GetMusicVolume");
			int result;
			result=GetMusicVolume();
			return result;
			
		} break;
    case FUNC_PLAYMUSIC:
	{
		if (parameters.size() > 2|| parameters.size()==0)
			throw IVM::IVMException(
						   "wrong number of parameters for PlayMusic");
	//I can't figure out how to dynamic cast function pointers :(
		//		void (*)() * pHook = dynamic_cast<void (*)() *>
	//							(parameters[1].GetObjPtr());
		//if (parameters.size() == 2)
			//PlayMusic(parameters[0].GetString(),pHook);
		//else
			PlayMusic(parameters[0].GetString());
		
		return true;
		
	} break;
    case FUNC_HOOKMUSICFINISHED:
	{
		if (parameters.size() != 1)
			throw IVM::IVMException(
						   "wrong number of params for HookMusicFinished");

//		void (*)() pHook = dynamic_cast<void (*)()>
//							(parameters[1].GetObjPtr());
//		HookMusicFinished(pHook);
		return true;

	} break;
	}

	return true;
}

} // namespace Isis

