
#include <isis/util.h>
#include <isis/ivm.h>

#include <isis/game/Sound.h>

#include <isis/game/SoundEngine.h>

namespace Isis {

void Sound::FillExportTable()
{
    m_ExportTable["Init"]			= FUNC_INIT;
	m_ExportTable["Play"]			= FUNC_PLAY;
	m_ExportTable["SetVolume"]		= FUNC_SETVOLUME;
	m_ExportTable["GetVolume"]		= FUNC_GETVOLUME;
}

void Sound::IVMFillExportTable(std::vector<std::string> & table)
{
    std::map<std::string,FuncType>::iterator iter;

    for (iter = m_ExportTable.begin(); iter != m_ExportTable.end(); iter++)
    {
        table.push_back(iter->first);
    }
}

bool Sound::IVMContainsMethod(const std::string & funcname)
{
    return m_ExportTable.end() != m_ExportTable.find(funcname);
}

IVM::Value Sound::IVMCallMethod(const std::string & funcname,
                           const std::vector<IVM::Value> & parameters)
{   
    if (m_ExportTable.end() == m_ExportTable.find(funcname))
        throw IVM::IVMException("function not found in Sound object");
    
    switch (m_ExportTable[funcname])
    {
    case FUNC_INIT:
        {
            if (parameters.size() != 1)
                throw IVM::IVMException(
                       "wrong number of parameters for Init");

			if (parameters[0].GetType() == IVM::Value::String)
			{
	            Init(parameters[0].GetString());
			}
			else
			{
				throw IsisException("Wrong type passed to Sound Init");
			}
			
		} break;
    case FUNC_SETVOLUME:
        {
            if (parameters.size() != 1)
                throw IVM::IVMException(
                       "wrong number of parameters for SetVolume");

			if (parameters[0].GetType() == IVM::Value::Integer)
			{
				Sound::SetVolume(parameters[0].GetInteger());
			}else
			{
				throw IsisException("Wrong type passed to Sound SetVolume");
			}
			
		} break;
		
    case FUNC_GETVOLUME:
        {
            if (parameters.size() != 0)
                throw IVM::IVMException(
                       "wrong number of parameters for GetVolume");

			return (int)GetVolume();
			
		} break;
    case FUNC_PLAY:
        {
            if (parameters.size() != 0)
                throw IVM::IVMException(
                       "wrong number of parameters for Init");

			Play();
			
		} break; 
    }

	return true;
}

} // namespace Isis

