
#include <isis/util.h>
#include <isis/ivm.h>

#include <isis/game/GraphicsEngine.h>
#include <isis/game/VectorAnimation.h>
#include <isis/game/VectorImage.h>

namespace Isis {

void VectorAnimation::FillExportTable()
{
    m_ExportTable["GetNumFrames"]    = FUNC_GETNUMFRAMES;
    m_ExportTable["GetFrame"]        = FUNC_GETFRAME;
    m_ExportTable["GetFrameRate"]    = FUNC_GETFRAMERATE;
    m_ExportTable["GetName"]         = FUNC_GETNAME;
	m_ExportTable["Get"]			 = FUNC_GET;
}

void VectorAnimation::IVMFillExportTable(std::vector<std::string> & table)
{
    std::map<std::string,FuncType>::iterator iter;

    for (iter = m_ExportTable.begin(); iter != m_ExportTable.end(); iter++)
    {
        table.push_back(iter->first);
    }
}

bool VectorAnimation::IVMContainsMethod(const std::string & funcname)
{
    return m_ExportTable.end() != m_ExportTable.find(funcname);
}

IVM::Value VectorAnimation::IVMCallMethod(const std::string & funcname,
                           const std::vector<IVM::Value> & parameters)
{   
    if (m_ExportTable.end() == m_ExportTable.find(funcname))
        throw IVM::IVMException("function not found in VectorAnimation object");
    
    switch (m_ExportTable[funcname])
    {
    case FUNC_GETNUMFRAMES:
        {
            if (parameters.size() != 0)
                throw IVM::IVMException(
                         "wrong number of parameters for GetNumFrames");

            return GetNumFrames();

        } break;
	case FUNC_GET:
    case FUNC_GETFRAME:
        {
            if (parameters.size() != 1)
                throw IVM::IVMException(
                         "wrong number of parameters for GetFrame");

            return GetFrame(parameters[0].GetInteger());

        } break;
    case FUNC_GETFRAMERATE:
        {
            if (parameters.size() != 0)
                throw IVM::IVMException(
                         "wrong number of parameters for GetFrameRAte");

            return GetFramerate();

        } break;
    case FUNC_GETNAME:
        {
            if (parameters.size() != 0)
                throw IVM::IVMException(
                         "wrong number of parameters for GetName");

            return GetName();

        } break;
    }

	return true;
}

} // namespace Isis

