
#include <isis/util.h>
#include <isis/ivm.h>

#include <isis/game/GraphicsEngine.h>
#include <isis/game/VectorImage.h>

namespace Isis {

void VectorImage::FillExportTable()
{
    m_ExportTable["Draw"]            = FUNC_DRAW;
    m_ExportTable["GetLeft"]         = FUNC_GETLEFT;
    m_ExportTable["GetRight"]        = FUNC_GETRIGHT;
    m_ExportTable["GetTop"]          = FUNC_GETTOP;
    m_ExportTable["GetBottom"]       = FUNC_GETBOTTOM;
}

void VectorImage::IVMFillExportTable(std::vector<std::string> & table)
{
    std::map<std::string,FuncType>::iterator iter;

    for (iter = m_ExportTable.begin(); iter != m_ExportTable.end(); iter++)
    {
        table.push_back(iter->first);
    }
}

bool VectorImage::IVMContainsMethod(const std::string & funcname)
{
    return m_ExportTable.end() != m_ExportTable.find(funcname);
}

IVM::Value VectorImage::IVMCallMethod(const std::string & funcname,
                           const std::vector<IVM::Value> & parameters)
{   
    if (m_ExportTable.end() == m_ExportTable.find(funcname))
        throw IVM::IVMException("function not found in VectorImage object");
    
    switch (m_ExportTable[funcname])
    {
    case FUNC_DRAW:
        {
			switch (parameters.size())
			{
			case 0:
				{
					Draw();
				} break;
			case 2:
				{
					Draw(parameters[0].GetFloat(),
						 parameters[1].GetFloat());
				} break;
			case 3:
				{
					Draw(parameters[0].GetFloat(),
						 parameters[1].GetFloat(),
						 parameters[2].GetFloat());
				} break;
			case 4:
				{
					Draw(parameters[0].GetFloat(),
						 parameters[1].GetFloat(),
						 parameters[2].GetFloat(),
						 parameters[3].GetFloat());
				} break;
			case 5:
				{
					Draw(parameters[0].GetFloat(),
						 parameters[1].GetFloat(),
						 parameters[2].GetFloat(),
						 parameters[3].GetFloat(),
						 parameters[4].GetFloat());
				} break;
			case 6:
				{
					Draw(parameters[0].GetFloat(),
						 parameters[1].GetFloat(),
						 parameters[2].GetFloat(),
						 parameters[3].GetFloat(),
						 parameters[4].GetFloat(),
						 parameters[5].GetFloat());
				} break;
			default:
				{
					throw IVM::IVMException(
							"wrong number of parameters for Draw");
				} break;
			}

            return true;

        } break;
    case FUNC_GETLEFT:
        {
            if (parameters.size() != 0)
                throw IVM::IVMException(
                         "wrong number of parameters for GetLeft");

            return GetLeft();

        } break;
    case FUNC_GETRIGHT:
        {
            if (parameters.size() != 0)
                throw IVM::IVMException(
                         "wrong number of parameters for GetRight");

            return GetRight();

        } break;
	case FUNC_GETTOP:
        {
            if (parameters.size() != 0)
                throw IVM::IVMException(
                         "wrong number of parameters for GetTop");

            return GetTop();

        } break;
    case FUNC_GETBOTTOM:
        {
            if (parameters.size() != 0)
                throw IVM::IVMException(
                         "wrong number of parameters for GetBottom");

            return GetBottom();

        } break;
    }

	return true;
}

} // namespace Isis

