
#include <isis/util/errors.h>
#include <isis/util/RefCountObject.h>

namespace Isis {

RefCountObject::RefCountObject()
{
    m_iRefCount = 0;
}

RefCountObject::~RefCountObject()
{
    if (m_iRefCount != 0)
    {
        WARN_MSG("refcount not 0 in destructor: %d",m_iRefCount);
    }
}

void RefCountObject::AddRef()
{
    m_iRefCount++;

#ifdef _DEBUG
	m_iGlobalCount++;
#endif // _DEBUG

}

void RefCountObject::Release()
{
    if (m_iRefCount == 0)
    {
        WARN_MSG("refcount already 0 in Release()");
        return;
    }
    
    m_iRefCount--;

#ifdef _DEBUG
	m_iGlobalCount--;
#endif // _DEBUG

    if (m_iRefCount == 0)
    {
        delete this;
    }
}

void RefCountObject::SafeRelease()
{
    if (m_iRefCount == 0)
    {
        WARN_MSG("refcount already 0 in Release()");
        return;
    }
    
    m_iRefCount--;

#ifdef _DEBUG
	m_iGlobalCount--;
#endif // _DEBUG
}

int RefCountObject::GetRefCount()
{
    return m_iRefCount;
}

#ifdef _DEBUG

int RefCountObject::m_iGlobalCount = 0;

int RefCountObject::GetGlobalCount()
{
	return m_iGlobalCount;
}

#endif //_DEBUG

} // namespace Isis

