
#include <stdio.h>
#include <iostream>
#include <algorithm>

#include <isis/ivm/Script.h>
#include <isis/ivm/IVMException.h>
#include <isis/ivm/DataSet.h>

namespace IVM {

#ifdef _DEBUG
int DataSet::m_iGlobalRefCount = 0;
#endif // _DEBUG

std::map<std::string,DataSet::FuncType> DataSet::m_ExportTable;

DataSet::DataSet()
{
    if (m_ExportTable.empty())
        FillExportTable();

#ifdef _DEBUG
	m_iGlobalRefCount++;
#endif // _DEBUG
}

DataSet::~DataSet()
{
#ifdef _DEBUG
	m_iGlobalRefCount--;
#endif // _DEBUG
}

#ifdef _DEBUG
int DataSet::GetGlobalRefCount()
{
	return m_iGlobalRefCount;
}
#endif // _DEBUG

void DataSet::FillExportTable()
{
    m_ExportTable["Init"] =      FUNC_INIT;
    m_ExportTable["Get"] =       FUNC_GET;
    m_ExportTable["Set"] =       FUNC_SET;
    m_ExportTable["Size"] =      FUNC_SIZE;
    m_ExportTable["Clear"] =     FUNC_CLEAR;
    m_ExportTable["Save"] =      FUNC_SAVE;
    m_ExportTable["Load"] =      FUNC_LOAD;
}

bool DataSet::IVMContainsMethod(const std::string & funcname)
{
    return m_ExportTable.end() != m_ExportTable.find(funcname);
}
 
void DataSet::IVMFillExportTable(std::vector<std::string> & table)
{
    std::map<std::string,FuncType>::iterator iter;

    for (iter = m_ExportTable.begin(); iter != m_ExportTable.end(); iter++)
    {
        table.push_back(iter->first);
    }
}

Value DataSet::Get(const std::string & sFieldName)
{
    return m_Data[sFieldName];
}

void DataSet::Set(const std::string & sFieldName, Value value)
{
    m_Data[sFieldName] = value;
}

void DataSet::Clear()
{
    m_Data.clear();
}

int DataSet::Size()
{
    return (int)m_Data.size();
}

void DataSet::Save(ISaveLoad * pSaveLoad)
{
	if (NULL == pSaveLoad)
		throw IVMException("NULL ISaveLoad given to DataSet::Save()");

    std::map<std::string,Value>::iterator iter;

    pSaveLoad->Save(Size());

    for (iter = m_Data.begin(); iter != m_Data.end(); iter++)
    {
//		printf("saving: %s\n",iter->first.c_str());
        pSaveLoad->Save(iter->first);
        pSaveLoad->Save(iter->second);
    }
}

void DataSet::Load(ISaveLoad * pSaveLoad)
{
    int size = pSaveLoad->Load().GetInteger();

    Clear();

    for (int i = 0; i < size; i++)
    {
        std::string string = pSaveLoad->Load().GetString();
        Value value = pSaveLoad->Load();

        Set(string,value);
    }
}

Value DataSet::IVMCallMethod(const std::string & funcname, 
                             const std::vector<Value> & parameters)
{
    if (m_ExportTable.end() == m_ExportTable.find(funcname))
		throw IVMException("function not found in DataSet: ",
							funcname);

    switch (m_ExportTable[funcname])
    {
    case FUNC_INIT:
        {
			if (!parameters.empty())
				throw IVMException(
					"invalid number of parameters passed to DataSet()");

			return true;
		} break;
    case FUNC_GET:
        {
            if (parameters.size() != 1)
                throw IVMException("wrong number of parameters for Get");

            return Get(parameters[0].GetString());

        } break;
    case FUNC_SET:
        {
            if (parameters.size() != 2)
                throw IVMException("wrong number of parameters for Get");

			Set(parameters[0].GetString(),parameters[1]);

            return true;

        } break;
    case FUNC_SIZE:
        {
            if (parameters.size() != 0)
                throw IVMException("wrong number of parameters for Size");

            return Size();

        } break;
    case FUNC_CLEAR:
        {
            if (parameters.size() != 0)
                throw IVMException("wrong number of parameters for Clear");

            Clear();

            return true;
        } break;
    case FUNC_SAVE:
        {
            if (parameters.size() != 1)
                throw IVMException("wrong number of parameters for Save");

            Save(dynamic_cast<ISaveLoad*>(parameters[0].GetObjPtr()));

            return true;
        } break;
    case FUNC_LOAD:
        {
            if (parameters.size() != 1)
                throw IVMException("wrong number of parameters for Load");

            Load(dynamic_cast<ISaveLoad*>(parameters[0].GetObjPtr()));

            return true;
        } break;
    default:
        {
			throw IVMException("function not found in DataSet: ",funcname);
        } break;
    }
}

} // namespace IVM

