
#ifndef IVM_IVMSTDLIBPROVIDER_H_INCLUDED
#define IVM_IVMSTDLIBPROVIDER_H_INCLUDED

#include <map>
#include <string>

#include <isis/ivm/ObjectProvider.h>

namespace IVM {

class IVMStdLibProvider : public ObjectProvider, public IVMObject
{
public:

    IVMStdLibProvider();
    virtual ~IVMStdLibProvider() {}

    virtual void FillObjectTable(std::vector<std::string> & table);
    virtual IVMObject * CreateObject(const std::string & objname,
                                     const std::vector<Value> & InitParams);

    virtual void IVMFillExportTable(std::vector<std::string> & table);
    virtual bool IVMContainsMethod(const std::string & funcname);
    virtual IVM::Value IVMCallMethod(const std::string & funcname, 
                               const std::vector<Value> & parameters);

private:

    enum ObjectType
    {
        OBJECT_CONSOLE,
        OBJECT_ISISVM,
        OBJECT_VECTOR,
    OBJECT_DATASET
    };

    static std::map<std::string,ObjectType> m_ObjectTable;
    static void FillObjectTable();

    enum FuncType
    {
        FUNC_CHOPCHARACTER,
        FUNC_TOUPPER,
        FUNC_TOLOWER,
        FUNC_STRLEN,
        FUNC_ISNULL,
        FUNC_INT,
        FUNC_FLOAT,
        FUNC_STRING,
        FUNC_LISTFILES,
        FUNC_RAND,
		FUNC_SIN,
		FUNC_COS,
        FUNC_SQRT,
        FUNC_POW,
		FUNC_ABS
    };
    
    static std::map<std::string,FuncType> m_ExportTable;
    static void FillExportTable();
};

} // namespace IVM

#endif // IVM_IVMSTDLIBPROVIDER_H_INCLUDED

