
#include <isis/util.h>
#include <isis/ivm/IVMException.h>

#include "Instruction.h"
#include "ISEReader.h"

namespace IVM {


void Instruction::Load(ISEReader & reader)
{
    Type = (INSTRUCTION_TYPE)reader.ReadByte();;

    switch (Type)
    {
    case INSTR_MOV:
        {
            Operand op1;
            Operand op2;

            op1.LoadLValue(reader);
            OpList.push_back(op1);

            op2.LoadRValue(reader);
            OpList.push_back(op2);

        } break;
    case INSTR_ADD:
    case INSTR_SUB:
    case INSTR_MUL:
    case INSTR_DIV:
    case INSTR_MOD:
    case INSTR_LOGICAND:
    case INSTR_LOGICOR:
    case INSTR_BITAND:
    case INSTR_BITOR:
    case INSTR_XOR:
    case INSTR_SHL:
    case INSTR_SHR:
    case INSTR_CMPE:
    case INSTR_CMPNE:
    case INSTR_CMPG:
    case INSTR_CMPGE:
    case INSTR_CMPL:
    case INSTR_CMPLE:
    case INSTR_CONCAT:
        {
            Operand op1;
            Operand op2;
            Operand op3;

            op1.LoadLValue(reader);
            OpList.push_back(op1);

            op2.LoadRValue(reader);
            OpList.push_back(op2);

            op3.LoadRValue(reader);
            OpList.push_back(op3);

        } break;
    case INSTR_NEG:
    case INSTR_INC:
    case INSTR_DEC:
    case INSTR_NOT:
        {
            Operand op1;
            Operand op2;

            op1.LoadLValue(reader);
            OpList.push_back(op1);

            op2.LoadRValue(reader);
            OpList.push_back(op2);

        } break;
    case INSTR_POP:
        {
            Operand op;

            op.LoadLValue(reader);
            OpList.push_back(op);

        } break;
    case INSTR_PUSH:
        {
            Operand op;

            op.LoadRValue(reader);
            OpList.push_back(op);

        } break;
    case INSTR_GETCHAR:
    case INSTR_SETCHAR:
        {
            Operand op1;
            Operand op2;
            Operand op3;

            op1.LoadLValue(reader);
            OpList.push_back(op1);

            op2.LoadRValue(reader);
            OpList.push_back(op2);

            op3.LoadRValue(reader);
            OpList.push_back(op3);

        } break;
    case INSTR_JMP:
        {
            Operand op;

            op.LoadLabel(reader);
            OpList.push_back(op);

        } break;
    case INSTR_JZ:
    case INSTR_JNZ:
        {
            Operand op1;
            Operand op2;

            op1.LoadRValue(reader);
            OpList.push_back(op1);

            op2.LoadLabel(reader);
            OpList.push_back(op2);

        } break;
    case INSTR_CALL:
        {
            Operand func;
            Operand numparams;

            func.LoadCall(reader);
            OpList.push_back(func);

            numparams.LoadRValue(reader);
            OpList.push_back(numparams);

        } break;
    case INSTR_CALLHOST:
        {
            Operand hostfunc;
            Operand numparams;

            hostfunc.LoadCallHost(reader);
            OpList.push_back(hostfunc);

            numparams.LoadRValue(reader);
            OpList.push_back(numparams);

        } break;
    case INSTR_CALLOBJ:
        {
            Operand object;
            Operand funcname;
            Operand numparams;

            object.LoadLValue(reader);
            OpList.push_back(object);

            funcname.LoadCallObject(reader);
            OpList.push_back(funcname);

            numparams.LoadRValue(reader);
            OpList.push_back(numparams);

        } break;
    case INSTR_RET:
    case INSTR_THIS:
        {
        } break;
    default:
        {
            throw IVMException("Unknown instruction");
        } break;
    }
}


}
