
#include <sys/stat.h>

#include <isis/util.h>

#include <isis/ivm/IsisVM.h>

#include <isis/ivm/Script.h>
#include <isis/ivm/IVMException.h>
#include <isis/ivm/ObjectProvider.h>

namespace IVM {

IVMObject * IsisVM::NewObject(const std::string & name, 
                              const std::vector<Value> & InitParams)
{
    if (ObjectProviderExists(name))
    {
        ObjectProvider * provider = FindObjectProvider(name);

        return provider->CreateObject(name,InitParams);
    }
    else
    {
        struct stat statbuf;
		std::string prefix;
        std::string suffix;
		bool bFound = false;

		for (unsigned int i = 0; i < m_ScriptPrefixes.size(); i++)
		{
			prefix = m_ScriptPrefixes[i];

			if (-1 == stat((prefix + name + ".isz").c_str(),&statbuf))
			{
				if (-1 == stat((prefix + name + ".ism").c_str(),&statbuf))
				{
					if (-1 == stat((prefix + name + ".iss").c_str(),
								&statbuf))
					{
						continue;
					}
					else
					{
						bFound = true;
						suffix = ".iss";
						break;
					}
				}
				else
				{
					bFound = true;
					suffix = ".ism";
					break;
				}
			}
			else
			{
				bFound = true;
				suffix = ".isz";
				break;
			}
		}

		if (!bFound)
			throw IVMException("object not found:",name);

        return MakeScript(prefix + name + suffix,InitParams);
    }
}

}
