
#include <isis/util.h>

#include <isis/ivm/IVMException.h>

#include "Operand.h"
#include "ISEReader.h"

namespace IVM {


void Operand::LoadLValue(ISEReader & reader)
{
    Type = (OPERAND_TYPE)reader.ReadByte();

    switch (Type)
    {
    case OP_RETVAL:
        {
        } break;
    case OP_SCALAR:
        {
            iStackIndex = reader.ReadInt();
        } break;
    case OP_ARRAY:
        {
            iStackIndex = reader.ReadInt();
            iExtraInfo = reader.ReadInt();
        } break;
    default:
        {
            throw IVMException("unknown or invalid lvalue type");
        } break;
    }
}

void Operand::LoadRValue(ISEReader & reader)
{
    Type = (OPERAND_TYPE)reader.ReadByte();

    switch (Type)
    {
    case OP_STRING:
        {
            iStringIndex = reader.ReadInt();
        } break;
    case OP_INTEGER:
        {
            iValue = reader.ReadInt();
        } break;
    case OP_FLOAT:
        {
            fValue = reader.ReadFloat();
        } break;
    case OP_RETVAL:
        {
        } break;
    case OP_SCALAR:
        {
            iStackIndex = reader.ReadInt();
        } break;
    case OP_ARRAY:
        {
            iStackIndex = reader.ReadInt();
            iExtraInfo = reader.ReadInt();
        } break;
    default:
        {
            throw IVMException("unknown or invalid rvalue type");
        } break;
    }
}

void Operand::LoadLabel(ISEReader & reader)
{
    Type = OP_LABEL;
    iInstrNum = reader.ReadInt();
}

void Operand::LoadCall(ISEReader & reader)
{
    Type = OP_FUNCINDEX;
    iFuncIndex = reader.ReadInt();
}

void Operand::LoadCallHost(ISEReader & reader)
{
    Type = OP_HOSTFUNC;
    iHostFuncIndex = reader.ReadInt();
}

void Operand::LoadCallObject(ISEReader & reader)
{
    Type = OP_HOSTOBJ;
    iHostObjIndex = reader.ReadInt();
}


}

