
#include <isis/util.h>

#include <isis/ivm/Script.h>
#include <isis/ivm/IVMException.h>
#include <isis/ivm/IsisVM.h>

#include "ISEStruct.h"

namespace IVM {

#ifdef _DEBUG
int Script::m_iGlobalRefCount = 0;
#endif // _DEBUG

std::map<std::string,Script::FuncType> Script::m_ExportTable;

Script::Script()
{
    m_bLoaded = false;
	m_pISE = NULL;
    m_iCurrStackFrame = -1;
    m_iNextInstr = -1;
    m_iCurrInstr = -1;
	
    if (m_ExportTable.empty())
        FillExportTable();

#ifdef _DEBUG
	m_iGlobalRefCount++;
#endif // _DEBUG
}

Script::~Script()
{
    if (m_bLoaded)
		m_pParentVM->Release();

#ifdef _DEBUG
	m_iGlobalRefCount--;
#endif // _DEBUG
}

#ifdef _DEBUG
int Script::GetGlobalRefCount()
{
	return m_iGlobalRefCount;
}
#endif // _DEBUG

void Script::Load(IsisVM * pParentVM, const std::string & filename)
{
	if (NULL == pParentVM)
		throw IVMException("NULL parent VM in Load");
	
	m_pParentVM = pParentVM;
	m_pParentVM->AddRef();
	
    m_pISE = m_pParentVM->LoadISE(filename);
	
    for (int i = 0; i < m_pISE->iGlobalDataSize; i++)
    {
        Push(Value());
    }
	
    m_iCurrStackFrame = (int)m_Stack.size();
    m_bLoaded = true;
}

} // namespace IVM

