
#include <stdio.h>

#include <isis/util.h>

#include <isis/ivm/IVMException.h>
#include <isis/ivm/Script.h>

#include "ISEStruct.h"

namespace IVM {

void Script::DebugPrint()
{
    if (!m_bLoaded)
    {
        fprintf(stderr,"Script not loaded\n");
        return;
    }

    fprintf(stderr,"Script loaded.  ISE version: %d.%d\n",
            m_pISE->iMajorVersion,m_pISE->iMinorVersion);
    fprintf(stderr,"Global data size: %d\n",m_pISE->iGlobalDataSize);
    fprintf(stderr,"\n");

    DebugPrintInstructionTable();
    DebugPrintStringTable();
    DebugPrintFunctionTable();
    DebugPrintHostAPITable();
    DebugPrintHostObjTable();
}

void Script::DebugPrintInstructionTable()
{
    fprintf(stderr,"Instruction table:\n");

    for (unsigned int i = 0; i < m_pISE->InstructionTable.size(); i++)
    {
        fprintf(stderr," Instr %d  ",i);
        m_pISE->InstructionTable[i].DebugPrint();
    }

    fprintf(stderr,"\n");
}

void Script::DebugPrintStringTable()
{
    fprintf(stderr,"String table:\n");

    for (unsigned int i = 0; i < m_pISE->StringTable.size(); i++)
    {
        fprintf(stderr," String %d: %s\n",i,m_pISE->StringTable[i].c_str());
    }

    fprintf(stderr,"\n");
}

void Script::DebugPrintFunctionTable()
{
    fprintf(stderr,"Function table:\n");

    for (unsigned int i = 0; i < m_pISE->FunctionTable.size(); i++)
    {
        fprintf(stderr," Func %d  \n",i);
        m_pISE->FunctionTable[i].DebugPrint();
    }

    fprintf(stderr,"\n");
}

void Script::DebugPrintHostAPITable()
{
    fprintf(stderr,"HostAPI table:\n");

    for (unsigned int i = 0; i < m_pISE->HostAPITable.size(); i++)
    {
        fprintf(stderr," HostAPI %d: %s\n",i,m_pISE->HostAPITable[i].c_str());
    }

    fprintf(stderr,"\n");
}

void Script::DebugPrintHostObjTable()
{
    fprintf(stderr,"HostObj table:\n");

    for (unsigned int i = 0; i < m_pISE->HostObjTable.size(); i++)
    {
        fprintf(stderr," HostObj %d: %s\n",i,m_pISE->HostObjTable[i].c_str());
    }

    fprintf(stderr,"\n");
}

} // namespace IVM

