
#include <stdio.h>

#include <isis/util.h>

#include <isis/ivm/Value.h>

#include <isis/ivm/IVMObject.h>
#include <isis/ivm/IVMException.h>

namespace IVM {

#ifdef _DEBUG
int Value::m_iGlobalRefCount = 0;
#endif // _DEBUG

Value::Value()
{
    Type = None;
    iValue = 0;

#ifdef _DEBUG
	m_iGlobalRefCount++;
#endif // _DEBUG
}

Value::Value(int value)
{
    Type = Integer;
    iValue = value;

#ifdef _DEBUG
	m_iGlobalRefCount++;
#endif // _DEBUG
}

Value::Value(float value)
{
    Type = Float;
    fValue = value;

#ifdef _DEBUG
	m_iGlobalRefCount++;
#endif // _DEBUG
}

Value::Value(const std::string & value)
{
    Type = String;
    
    psValue = new std::string(value);

#ifdef _DEBUG
	m_iGlobalRefCount++;
#endif // _DEBUG
}

Value::Value(IVMObject * object)
{
    Type = ObjPtr;

    pObject = object;

    if (pObject)
        pObject->AddRef();

#ifdef _DEBUG
	m_iGlobalRefCount++;
#endif // _DEBUG
}

Value::Value(const Value & v)
{
    Type = v.GetType();

    switch (Type)
    {
    case None:     iValue      = 0;                              break;
    case Integer:  iValue      = v.GetInteger();                 break;
    case Float:    fValue      = v.GetFloat();                   break;
    case String:   psValue     = new std::string(v.GetString()); break;
    case ObjPtr:   
        {
            pObject = v.GetObjPtr();                         

            if (NULL != pObject)
            {
                pObject->AddRef();
            }
        } break;
    }

#ifdef _DEBUG
	m_iGlobalRefCount++;
#endif // _DEBUG
}


Value::~Value()
{
    Free();

#ifdef _DEBUG
	m_iGlobalRefCount--;
#endif // _DEBUG
}


#ifdef _DEBUG
int Value::GetGlobalRefCount()
{
	return m_iGlobalRefCount;
}
#endif // _DEBUG


Value & Value::operator=(const Value & v)
{
	if (this == &v)
		return *this;

    Free();

    Type = v.GetType();

    switch (Type)
    {
    case None:     iValue      = 0;                              break;
    case Integer:  iValue      = v.GetInteger();                 break;
    case Float:    fValue      = v.GetFloat();                   break;
    case String:   psValue     = new std::string(v.GetString()); break;
    case ObjPtr:   
        {
            pObject = v.GetObjPtr();                         

            if (NULL != pObject)
            {
                pObject->AddRef();
            }
        } break;
    }

    return *this;
}

void Value::Free()
{
    switch (Type)
    {
    case String:
        {
            if (NULL != psValue)
                delete psValue;
        } break;
    case ObjPtr:
        {
            if (NULL != pObject)
                pObject->Release();
        } break;
    default:
        break;
    }

	Type = None;
	iValue = 0;
}

void Value::DebugPrint() const
{
    switch (Type)
    {
    case Integer:  fprintf(stderr,"  int: %d\n",iValue); break;
    case Float:    fprintf(stderr,"  float: %f\n",fValue); break;
    case String:   fprintf(stderr,"  string: %s\n",
                                   psValue->c_str()); break;
    case ObjPtr:   fprintf(stderr,"  objptr: %p\n",pObject); break;
    case None:     fprintf(stderr,"None\n"); break;
    }
}


}

