
#include <stdlib.h>

#include <iostream>
#include <sstream>
#include <string>

#include <isis/util.h>

#include <isis/ivm/Value.h>

#include <isis/ivm/IVMObject.h>
#include <isis/ivm/IVMException.h>

namespace IVM {

Value::ValueType Value::GetType() const
{
    return Type;
}

int Value::GetInteger() const
{
    switch (Type)
    {
    case None:
        {
            return 0;
        } break;
    case Integer:
        {
            return iValue;
        } break;
    case Float:
        {
            return (int)fValue;
        } break;
    case String:
        {
            return atoi(psValue->c_str());
        } break;
    case ObjPtr:
        {
            WARN_MSG("coercing object pointer to int");
            return (int)(pObject-(IVMObject*)NULL);
        } break;
    }

    return 0;
}

float Value::GetFloat() const
{
    switch (Type)
    {
    case None:
        {
            return 0.0f;
        } break;
    case Integer:
        {
            return (float)iValue;
        } break;
    case Float:
        {
            return fValue;
        } break;
    case String:
        {
            return (float)atof(psValue->c_str());
        } break;
    case ObjPtr:
        {
            WARN_MSG("coercing object pointer to float");
            return (float)((int)(pObject - (IVMObject *)NULL));
        } break;
    }

    return 0.0f;
}

std::string Value::GetString() const
{
    switch (Type)
    {
    case None:
        {
            return "";
        } break;
    case Integer:
        {
            std::stringstream s;
            s << iValue;
            return s.str();
        } break;
    case Float:
        {
            std::stringstream s;
            s << fValue;
            return s.str();
        } break;
    case String:
        {
            return *psValue;
        } break;
    case ObjPtr:
        {
            WARN_MSG("coercing object pointer to string");
            std::stringstream s;
            s << "<OBJECT: " << pObject << ">";
            return s.str();
        } break;
    }

    return "";
}


IVMObject * Value::GetObjPtr() const
{
    switch (Type)
    {
    case None:
        {
            throw IVMException("use of an uninitialized variable as an object");
        }
    case Integer:
        {
			if (0 == iValue)
				return NULL;

            throw IVMException("cannot convert integer to object pointer");
        }
    case Float:
        {
            throw IVMException("cannot convert float to object pointer");
        }
    case String:
        {
            throw IVMException("cannot convert string to object pointer");
        }
    case ObjPtr:
        {
            return pObject;
        }
    default:
        {
            throw IVMException("ERROR - unknown Value type detected");
        }
    }
}


} // namespace IVM

