
#include <stdio.h>
#include <math.h>

#include <isis/util.h>

#include <isis/ivm/Value.h>
#include <isis/ivm/IVMException.h>

namespace IVM {

const Value operator+(const Value & left, const Value & right)
{
    switch (left.GetType())
    {
    case Value::None:
    case Value::Integer:
    case Value::String:
        {
			if (Value::Float == right.GetType())
	            return (left.GetFloat() + right.GetFloat());
			else
	            return (left.GetInteger() + right.GetInteger());
        }
    case Value::Float:
        {
            return (left.GetFloat() + right.GetFloat());
        }
    case Value::ObjPtr:
        {
            throw IVMException("adding something to an objptr");
        }
    }

    return 0;
}

const Value operator-(const Value & left, const Value & right)
{
    switch (left.GetType())
    {
    case Value::None:
    case Value::Integer:
    case Value::String:
        {
			if (Value::Float == right.GetType())
	            return (left.GetFloat() - right.GetFloat());
			else
	            return (left.GetInteger() - right.GetInteger());
        }
    case Value::Float:
        {
            return (left.GetFloat() - right.GetFloat());
        }
    case Value::ObjPtr:
        {
            throw IVMException("subtracting something from an objptr");
        }
    }

    return 0;
}

const Value operator*(const Value & left, const Value & right)
{
    switch (left.GetType())
    {
    case Value::None:
    case Value::Integer:
        {
			if (Value::Float == right.GetType())
	            return (left.GetFloat() * right.GetFloat());
			else
	            return (left.GetInteger() * right.GetInteger());
        }
    case Value::Float:
    case Value::String:
        {
            return (left.GetFloat() * right.GetFloat());
        }
    case Value::ObjPtr:
        {
            throw IVMException("multiplying something with an objptr");
        }
    }

    return 0;
}

const Value operator/(const Value & left, const Value & right)
{
    switch (left.GetType())
    {
    case Value::None:
    case Value::Integer:
        {
			if (Value::Float == right.GetType())
	            return (left.GetFloat() / right.GetFloat());
			else
			{
	            if (0 == right.GetInteger() && 0 != left.GetInteger())
					throw IVMException("Divide by zero error!");

				return (left.GetInteger() / right.GetInteger());
			}
        }
    case Value::Float:
    case Value::String:
        {
            return (left.GetFloat() / right.GetFloat());
        }
    case Value::ObjPtr:
        {
            throw IVMException("dividing an objptr by something");
        }
    }

    return 0;
}

const Value operator%(const Value & left, const Value & right)
{
    switch (left.GetType())
    {
    case Value::None:
    case Value::Integer:
    case Value::Float:
    case Value::String:
        {
            if (0 == right.GetInteger())
                throw IVMException("Modding by zero!");

            return (left.GetInteger() % right.GetInteger());
        }
    case Value::ObjPtr:
        {
            throw IVMException("modding an objptr by something");
        }
    }

    return 0;
}

const Value operator-(const Value & value)
{
    switch (value.GetType())
    {
    case Value::None:
    case Value::Integer:
        {
            return (-value.GetInteger());
        }
    case Value::Float:
    case Value::String:
        {
            return (-value.GetFloat());
        }
    case Value::ObjPtr:
        {
            throw IVMException("negating an objptr");
        }
    }

    return 0;
}

const Value operator&(const Value & left, const Value & right)
{
    switch (left.GetType())
    {
    case Value::None:
    case Value::Integer:
    case Value::Float:
    case Value::String:
        {
            return (left.GetInteger() & right.GetInteger());
        }
    case Value::ObjPtr:
        {
            throw IVMException("anding an objptr with something");
        }
    }

    return 0;
}

const Value operator|(const Value & left, const Value & right)
{
    switch (left.GetType())
    {
    case Value::None:
    case Value::Integer:
    case Value::Float:
    case Value::String:
        {
            return (left.GetInteger() | right.GetInteger());
        }
    case Value::ObjPtr:
        {
            throw IVMException("oring an objptr with something");
        }
    }

    return 0;
}

const Value operator&&(const Value & left, const Value & right)
{
    switch (left.GetType())
    {
    case Value::None:
    case Value::Integer:
    case Value::Float:
    case Value::String:
        {
            return (left.GetInteger() && right.GetInteger());
        }
    case Value::ObjPtr:
        {
            throw IVMException("anding an objptr with something");
        }
    }

    return 0;
}

const Value operator||(const Value & left, const Value & right)
{
    switch (left.GetType())
    {
    case Value::None:
    case Value::Integer:
    case Value::Float:
    case Value::String:
        {
            return (left.GetInteger() || right.GetInteger());
        }
    case Value::ObjPtr:
        {
            throw IVMException("oring an objptr with something");
        }
    }

    return 0;
}

const Value operator^(const Value & left, const Value & right)
{
    switch (left.GetType())
    {
    case Value::None:
    case Value::Integer:
    case Value::Float:
    case Value::String:
        {
            return (left.GetInteger() ^ right.GetInteger());
        }
    case Value::ObjPtr:
        {
            throw IVMException("xoring an objptr with something");
        }
    }

    return 0;
}

const Value operator!(const Value & value)
{
    switch (value.GetType())
    {
    case Value::None:
    case Value::Integer:
    case Value::Float:
    case Value::String:
        {
            return (!value.GetInteger());
        }
    case Value::ObjPtr:
        {
            throw IVMException("notting an objptr");
        }
    }

    return 0;
}

const Value operator<<(const Value & left, const Value & right)
{
    switch (left.GetType())
    {
    case Value::None:
    case Value::Integer:
    case Value::Float:
    case Value::String:
        {
            return (left.GetInteger() << right.GetInteger());
        }
    case Value::ObjPtr:
        {
            throw IVMException("shifting an objptr");
        }
    }

    return 0;
}

const Value operator>>(const Value & left, const Value & right)
{
    switch (left.GetType())
    {
    case Value::None:
    case Value::Integer:
    case Value::Float:
    case Value::String:
        {
            return (left.GetInteger() >> right.GetInteger());
        }
    case Value::ObjPtr:
        {
            throw IVMException("shifting an objptr");
        }
    }

    return 0;
}


} // namespace IVM

