
#include <stdio.h>
#include <math.h>

#include <isis/util.h>

#include <isis/ivm/Value.h>
#include <isis/ivm/IVMException.h>

namespace IVM {

bool operator==(const Value & left, const Value & right)
{
    switch (left.GetType())
    {
    case Value::None:
    case Value::Integer:
        {
			if (right.GetType() == Value::ObjPtr && left.GetInteger() == 0)
				return NULL == right.GetObjPtr();
			else
	            return (left.GetInteger() == right.GetInteger());
        }
    case Value::Float:
        {
            return (fabs(left.GetFloat() - right.GetFloat()) < 0.00001f);
        }
    case Value::String:
        {
            return (left.GetString() == right.GetString());
        }
    case Value::ObjPtr:
        {
            return (left.GetObjPtr() == right.GetObjPtr());
        }
    }

    return false;
}

bool operator!=(const Value & left, const Value & right)
{
    return (!(left == right));
}


bool operator>(const Value & left, const Value & right)
{
    switch (left.GetType())
    {
    case Value::None:
    case Value::Integer:
        {
            return (left.GetInteger() > right.GetInteger());
        }
    case Value::Float:
        {
            return (left.GetFloat() > right.GetFloat());
        }
    case Value::String:
        {
            return (left.GetString() > right.GetString());
        }
    case Value::ObjPtr:
        {
            return (left.GetObjPtr() > right.GetObjPtr());
        }
    }

    return false;
}

bool operator<=(const Value & left, const Value & right)
{
    return (!(left > right));
}



bool operator<(const Value & left, const Value & right)
{
    switch (left.GetType())
    {
    case Value::None:
    case Value::Integer:
        {
            return (left.GetInteger() < right.GetInteger());
        }
    case Value::Float:
        {
            return (left.GetFloat() < right.GetFloat());
        }
    case Value::String:
        {
            return (left.GetString() < right.GetString());
        }
    case Value::ObjPtr:
        {
            return (left.GetObjPtr() < right.GetObjPtr());
        }
    }

    return false;
}

bool operator>=(const Value & left, const Value & right)
{
    return (!(left < right));
}


} // namespace IVM

