#region Usings

using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Text;
using System.Drawing.Imaging;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.IO;
using System.Runtime.InteropServices;

using ICSharpCode.SharpZipLib.GZip;

#endregion

namespace ImageMaker
{
	public class ImageMaker : System.Windows.Forms.Form
	{
		#region UI Controls
		private System.ComponentModel.IContainer components;
		private System.Windows.Forms.ToolBar toolBar1;
		private System.Windows.Forms.StatusBar statusBar1;
		private System.Windows.Forms.Panel panel1;
		private System.Windows.Forms.Panel panel2;
		private System.Windows.Forms.Panel ImagePanel;
		private System.Windows.Forms.Button AddImage;
		private System.Windows.Forms.Button RemoveImage;
		private System.Windows.Forms.StatusBarPanel SizePanel;
		private System.Windows.Forms.StatusBarPanel MousePosPanel;
		private System.Windows.Forms.CheckBox NeedBorderCheck;
		private System.Windows.Forms.RadioButton RepeatRadio;
		private System.Windows.Forms.RadioButton MirrorRadio;
		private System.Windows.Forms.RadioButton EmptyRadio;
		private System.Windows.Forms.ImageList imageList1;
		private System.Windows.Forms.GroupBox SIGroup;
		private System.Windows.Forms.Label SISizeLabel;
		private System.Windows.Forms.Label SIPosLabel;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.Button MoveFrontButton;
		private System.Windows.Forms.Button MoveBackButton;
		private System.Windows.Forms.Label SINumber;
		private System.Windows.Forms.Label SISize;
		private System.Windows.Forms.Label SIPos;
		private System.Windows.Forms.CheckBox ShowNumbersCheck;
		private System.Windows.Forms.ToolBarButton NewButton;
		private System.Windows.Forms.ToolBarButton OpenButton;
		private System.Windows.Forms.ToolBarButton SaveButton;
		private System.Windows.Forms.ToolBarButton SaveAsButton;
		private System.Windows.Forms.StatusBarPanel SaveProgressPanel;
		private System.Windows.Forms.Button RenderFontButton;
		private System.Windows.Forms.Button ReplaceImage;
		#endregion

		#region Subtypes

		public enum BorderType
		{
			None = 0,
			Repeat = 1,
			Mirror = 2,
			Empty = 3
		}

		public class SubImage
		{
			public Bitmap bitmap;
			public Point position;
			public BorderType borders;

			public bool HasBorders
			{ get { return (borders != BorderType.None); } }
			public Int32 Left 
			{ get { return position.X - (HasBorders ? 1 : 0); } }
			public Int32 Right 
			{ get { return position.X + bitmap.Width - 1 + (HasBorders ? 1 : 0); } }
			public Int32 Top 
			{ get { return position.Y - (HasBorders ? 1 : 0); } }
			public Int32 Bottom 
			{ get { return position.Y + bitmap.Height - 1 + (HasBorders ? 1 : 0); } }
			public Rectangle Rect
			{ get { return new Rectangle(Left,Top,Right-Left+1,Bottom-Top+1); } }
		}

		#endregion

		#region Fields

		ArrayList m_ImageList = new ArrayList();
		SubImage m_SelectedSI;
		Point m_LastMousePos;
		Point m_MouseDownPos;
		Point m_MouseSIPos;
		bool m_bMouseDown;
		bool m_bDragging;
		static string m_sFilename = "";
		float m_fSaveProgress;

		#endregion

		#region Loading/Disposing

		public ImageMaker()
		{
			InitializeComponent();
		}

		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		private void Form1_Load(object sender, System.EventArgs e)
		{
			SizeUpdated();

			if (m_sFilename.Length > 0)
			{
				Open();

				SizeUpdated();
				ImagePanel.Invalidate();
			}
		}

		#endregion

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(ImageMaker));
			this.toolBar1 = new System.Windows.Forms.ToolBar();
			this.NewButton = new System.Windows.Forms.ToolBarButton();
			this.OpenButton = new System.Windows.Forms.ToolBarButton();
			this.SaveButton = new System.Windows.Forms.ToolBarButton();
			this.SaveAsButton = new System.Windows.Forms.ToolBarButton();
			this.imageList1 = new System.Windows.Forms.ImageList(this.components);
			this.statusBar1 = new System.Windows.Forms.StatusBar();
			this.SizePanel = new System.Windows.Forms.StatusBarPanel();
			this.MousePosPanel = new System.Windows.Forms.StatusBarPanel();
			this.SaveProgressPanel = new System.Windows.Forms.StatusBarPanel();
			this.panel1 = new System.Windows.Forms.Panel();
			this.ReplaceImage = new System.Windows.Forms.Button();
			this.RenderFontButton = new System.Windows.Forms.Button();
			this.ShowNumbersCheck = new System.Windows.Forms.CheckBox();
			this.SIGroup = new System.Windows.Forms.GroupBox();
			this.SIPos = new System.Windows.Forms.Label();
			this.SISize = new System.Windows.Forms.Label();
			this.SINumber = new System.Windows.Forms.Label();
			this.MoveBackButton = new System.Windows.Forms.Button();
			this.MoveFrontButton = new System.Windows.Forms.Button();
			this.label1 = new System.Windows.Forms.Label();
			this.SIPosLabel = new System.Windows.Forms.Label();
			this.SISizeLabel = new System.Windows.Forms.Label();
			this.EmptyRadio = new System.Windows.Forms.RadioButton();
			this.MirrorRadio = new System.Windows.Forms.RadioButton();
			this.RepeatRadio = new System.Windows.Forms.RadioButton();
			this.NeedBorderCheck = new System.Windows.Forms.CheckBox();
			this.RemoveImage = new System.Windows.Forms.Button();
			this.AddImage = new System.Windows.Forms.Button();
			this.panel2 = new System.Windows.Forms.Panel();
			this.ImagePanel = new System.Windows.Forms.Panel();
			((System.ComponentModel.ISupportInitialize)(this.SizePanel)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.MousePosPanel)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.SaveProgressPanel)).BeginInit();
			this.panel1.SuspendLayout();
			this.SIGroup.SuspendLayout();
			this.panel2.SuspendLayout();
			this.SuspendLayout();
			// 
			// toolBar1
			// 
			this.toolBar1.Appearance = System.Windows.Forms.ToolBarAppearance.Flat;
			this.toolBar1.Buttons.AddRange(new System.Windows.Forms.ToolBarButton[] {
																						this.NewButton,
																						this.OpenButton,
																						this.SaveButton,
																						this.SaveAsButton});
			this.toolBar1.ButtonSize = new System.Drawing.Size(80, 22);
			this.toolBar1.Divider = false;
			this.toolBar1.DropDownArrows = true;
			this.toolBar1.ImageList = this.imageList1;
			this.toolBar1.Name = "toolBar1";
			this.toolBar1.ShowToolTips = true;
			this.toolBar1.Size = new System.Drawing.Size(768, 23);
			this.toolBar1.TabIndex = 0;
			this.toolBar1.TextAlign = System.Windows.Forms.ToolBarTextAlign.Right;
			this.toolBar1.ButtonClick += new System.Windows.Forms.ToolBarButtonClickEventHandler(this.toolBar1_ButtonClick);
			// 
			// NewButton
			// 
			this.NewButton.ImageIndex = 0;
			this.NewButton.Text = "New";
			// 
			// OpenButton
			// 
			this.OpenButton.ImageIndex = 1;
			this.OpenButton.Text = "Open";
			// 
			// SaveButton
			// 
			this.SaveButton.ImageIndex = 2;
			this.SaveButton.Text = "Save";
			// 
			// SaveAsButton
			// 
			this.SaveAsButton.ImageIndex = 2;
			this.SaveAsButton.Text = "Save As...";
			// 
			// imageList1
			// 
			this.imageList1.ColorDepth = System.Windows.Forms.ColorDepth.Depth8Bit;
			this.imageList1.ImageSize = new System.Drawing.Size(16, 16);
			this.imageList1.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imageList1.ImageStream")));
			this.imageList1.TransparentColor = System.Drawing.Color.Transparent;
			// 
			// statusBar1
			// 
			this.statusBar1.Location = new System.Drawing.Point(0, 608);
			this.statusBar1.Name = "statusBar1";
			this.statusBar1.Panels.AddRange(new System.Windows.Forms.StatusBarPanel[] {
																						  this.SizePanel,
																						  this.MousePosPanel,
																						  this.SaveProgressPanel});
			this.statusBar1.ShowPanels = true;
			this.statusBar1.Size = new System.Drawing.Size(768, 22);
			this.statusBar1.TabIndex = 1;
			// 
			// SizePanel
			// 
			this.SizePanel.Text = "Texture size: 256,256";
			this.SizePanel.Width = 130;
			// 
			// MousePosPanel
			// 
			this.MousePosPanel.Text = "Mouse: ";
			// 
			// SaveProgressPanel
			// 
			this.SaveProgressPanel.Width = 200;
			// 
			// panel1
			// 
			this.panel1.Controls.AddRange(new System.Windows.Forms.Control[] {
																				 this.ReplaceImage,
																				 this.RenderFontButton,
																				 this.ShowNumbersCheck,
																				 this.SIGroup,
																				 this.RemoveImage,
																				 this.AddImage});
			this.panel1.Dock = System.Windows.Forms.DockStyle.Right;
			this.panel1.Location = new System.Drawing.Point(632, 23);
			this.panel1.Name = "panel1";
			this.panel1.Size = new System.Drawing.Size(136, 585);
			this.panel1.TabIndex = 2;
			// 
			// ReplaceImage
			// 
			this.ReplaceImage.Enabled = false;
			this.ReplaceImage.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.ReplaceImage.Location = new System.Drawing.Point(24, 72);
			this.ReplaceImage.Name = "ReplaceImage";
			this.ReplaceImage.Size = new System.Drawing.Size(88, 23);
			this.ReplaceImage.TabIndex = 5;
			this.ReplaceImage.Text = "Replace image";
			this.ReplaceImage.Click += new System.EventHandler(this.ReplaceImage_Click);
			// 
			// RenderFontButton
			// 
			this.RenderFontButton.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.RenderFontButton.Location = new System.Drawing.Point(24, 384);
			this.RenderFontButton.Name = "RenderFontButton";
			this.RenderFontButton.Size = new System.Drawing.Size(88, 23);
			this.RenderFontButton.TabIndex = 4;
			this.RenderFontButton.Text = "Create font...";
			this.RenderFontButton.Click += new System.EventHandler(this.RenderFontButton_Click);
			// 
			// ShowNumbersCheck
			// 
			this.ShowNumbersCheck.Checked = true;
			this.ShowNumbersCheck.CheckState = System.Windows.Forms.CheckState.Checked;
			this.ShowNumbersCheck.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.ShowNumbersCheck.Location = new System.Drawing.Point(16, 112);
			this.ShowNumbersCheck.Name = "ShowNumbersCheck";
			this.ShowNumbersCheck.Size = new System.Drawing.Size(104, 16);
			this.ShowNumbersCheck.TabIndex = 3;
			this.ShowNumbersCheck.Text = "Show numbers";
			this.ShowNumbersCheck.CheckedChanged += new System.EventHandler(this.ShowNumbersCheck_CheckedChanged);
			// 
			// SIGroup
			// 
			this.SIGroup.Anchor = ((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right);
			this.SIGroup.Controls.AddRange(new System.Windows.Forms.Control[] {
																				  this.SIPos,
																				  this.SISize,
																				  this.SINumber,
																				  this.MoveBackButton,
																				  this.MoveFrontButton,
																				  this.label1,
																				  this.SIPosLabel,
																				  this.SISizeLabel,
																				  this.EmptyRadio,
																				  this.MirrorRadio,
																				  this.RepeatRadio,
																				  this.NeedBorderCheck});
			this.SIGroup.Enabled = false;
			this.SIGroup.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.SIGroup.Location = new System.Drawing.Point(8, 136);
			this.SIGroup.Name = "SIGroup";
			this.SIGroup.Size = new System.Drawing.Size(120, 232);
			this.SIGroup.TabIndex = 2;
			this.SIGroup.TabStop = false;
			this.SIGroup.Text = "Selected image";
			// 
			// SIPos
			// 
			this.SIPos.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.SIPos.Location = new System.Drawing.Point(56, 48);
			this.SIPos.Name = "SIPos";
			this.SIPos.Size = new System.Drawing.Size(48, 16);
			this.SIPos.TabIndex = 11;
			// 
			// SISize
			// 
			this.SISize.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.SISize.Location = new System.Drawing.Point(56, 32);
			this.SISize.Name = "SISize";
			this.SISize.Size = new System.Drawing.Size(48, 16);
			this.SISize.TabIndex = 10;
			// 
			// SINumber
			// 
			this.SINumber.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.SINumber.Location = new System.Drawing.Point(56, 16);
			this.SINumber.Name = "SINumber";
			this.SINumber.Size = new System.Drawing.Size(48, 16);
			this.SINumber.TabIndex = 9;
			// 
			// MoveBackButton
			// 
			this.MoveBackButton.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.MoveBackButton.Location = new System.Drawing.Point(12, 104);
			this.MoveBackButton.Name = "MoveBackButton";
			this.MoveBackButton.Size = new System.Drawing.Size(96, 23);
			this.MoveBackButton.TabIndex = 8;
			this.MoveBackButton.Text = "Move Backward";
			this.MoveBackButton.Click += new System.EventHandler(this.MoveBackButton_Click);
			// 
			// MoveFrontButton
			// 
			this.MoveFrontButton.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.MoveFrontButton.Location = new System.Drawing.Point(16, 72);
			this.MoveFrontButton.Name = "MoveFrontButton";
			this.MoveFrontButton.Size = new System.Drawing.Size(88, 23);
			this.MoveFrontButton.TabIndex = 7;
			this.MoveFrontButton.Text = "Move Forward";
			this.MoveFrontButton.Click += new System.EventHandler(this.MoveFrontButton_Click);
			// 
			// label1
			// 
			this.label1.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.label1.Location = new System.Drawing.Point(8, 16);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(48, 16);
			this.label1.TabIndex = 6;
			this.label1.Text = "Number:";
			// 
			// SIPosLabel
			// 
			this.SIPosLabel.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.SIPosLabel.Location = new System.Drawing.Point(8, 48);
			this.SIPosLabel.Name = "SIPosLabel";
			this.SIPosLabel.Size = new System.Drawing.Size(48, 16);
			this.SIPosLabel.TabIndex = 5;
			this.SIPosLabel.Text = "Pos:";
			// 
			// SISizeLabel
			// 
			this.SISizeLabel.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.SISizeLabel.Location = new System.Drawing.Point(8, 32);
			this.SISizeLabel.Name = "SISizeLabel";
			this.SISizeLabel.Size = new System.Drawing.Size(48, 16);
			this.SISizeLabel.TabIndex = 4;
			this.SISizeLabel.Text = "Size: ";
			// 
			// EmptyRadio
			// 
			this.EmptyRadio.Enabled = false;
			this.EmptyRadio.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.EmptyRadio.Location = new System.Drawing.Point(16, 208);
			this.EmptyRadio.Name = "EmptyRadio";
			this.EmptyRadio.Size = new System.Drawing.Size(96, 16);
			this.EmptyRadio.TabIndex = 3;
			this.EmptyRadio.Text = "Empty Border";
			this.EmptyRadio.CheckedChanged += new System.EventHandler(this.EmptyRadio_CheckedChanged);
			// 
			// MirrorRadio
			// 
			this.MirrorRadio.Enabled = false;
			this.MirrorRadio.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.MirrorRadio.Location = new System.Drawing.Point(16, 184);
			this.MirrorRadio.Name = "MirrorRadio";
			this.MirrorRadio.Size = new System.Drawing.Size(96, 16);
			this.MirrorRadio.TabIndex = 2;
			this.MirrorRadio.Text = "Mirror Border";
			this.MirrorRadio.CheckedChanged += new System.EventHandler(this.MirrorRadio_CheckedChanged);
			// 
			// RepeatRadio
			// 
			this.RepeatRadio.Checked = true;
			this.RepeatRadio.Enabled = false;
			this.RepeatRadio.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.RepeatRadio.Location = new System.Drawing.Point(16, 160);
			this.RepeatRadio.Name = "RepeatRadio";
			this.RepeatRadio.Size = new System.Drawing.Size(96, 16);
			this.RepeatRadio.TabIndex = 1;
			this.RepeatRadio.TabStop = true;
			this.RepeatRadio.Text = "Repeat border";
			this.RepeatRadio.CheckedChanged += new System.EventHandler(this.RepeatRadio_CheckedChanged);
			// 
			// NeedBorderCheck
			// 
			this.NeedBorderCheck.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.NeedBorderCheck.Location = new System.Drawing.Point(8, 136);
			this.NeedBorderCheck.Name = "NeedBorderCheck";
			this.NeedBorderCheck.Size = new System.Drawing.Size(96, 16);
			this.NeedBorderCheck.TabIndex = 0;
			this.NeedBorderCheck.Text = "Needs border";
			this.NeedBorderCheck.CheckedChanged += new System.EventHandler(this.NeedBorderCheck_CheckedChanged);
			// 
			// RemoveImage
			// 
			this.RemoveImage.Anchor = System.Windows.Forms.AnchorStyles.Top;
			this.RemoveImage.Enabled = false;
			this.RemoveImage.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.RemoveImage.Location = new System.Drawing.Point(20, 40);
			this.RemoveImage.Name = "RemoveImage";
			this.RemoveImage.Size = new System.Drawing.Size(96, 24);
			this.RemoveImage.TabIndex = 1;
			this.RemoveImage.Text = "Remove image";
			this.RemoveImage.Click += new System.EventHandler(this.RemoveImage_Click);
			// 
			// AddImage
			// 
			this.AddImage.Anchor = System.Windows.Forms.AnchorStyles.Top;
			this.AddImage.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.AddImage.Location = new System.Drawing.Point(20, 8);
			this.AddImage.Name = "AddImage";
			this.AddImage.Size = new System.Drawing.Size(96, 24);
			this.AddImage.TabIndex = 0;
			this.AddImage.Text = "Add image";
			this.AddImage.Click += new System.EventHandler(this.AddImage_Click);
			// 
			// panel2
			// 
			this.panel2.AutoScroll = true;
			this.panel2.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.panel2.Controls.AddRange(new System.Windows.Forms.Control[] {
																				 this.ImagePanel});
			this.panel2.Dock = System.Windows.Forms.DockStyle.Fill;
			this.panel2.Location = new System.Drawing.Point(0, 23);
			this.panel2.Name = "panel2";
			this.panel2.Size = new System.Drawing.Size(632, 585);
			this.panel2.TabIndex = 3;
			// 
			// ImagePanel
			// 
			this.ImagePanel.BackColor = System.Drawing.Color.Fuchsia;
			this.ImagePanel.Name = "ImagePanel";
			this.ImagePanel.Size = new System.Drawing.Size(256, 256);
			this.ImagePanel.TabIndex = 0;
			this.ImagePanel.MouseUp += new System.Windows.Forms.MouseEventHandler(this.UpMouse);
			this.ImagePanel.Paint += new System.Windows.Forms.PaintEventHandler(this.ImagePanel_Paint);
			this.ImagePanel.MouseMove += new System.Windows.Forms.MouseEventHandler(this.MoveMouse);
			this.ImagePanel.MouseLeave += new System.EventHandler(this.MouseOut);
			this.ImagePanel.MouseDown += new System.Windows.Forms.MouseEventHandler(this.DownMouse);
			// 
			// Form1
			// 
			this.AcceptButton = this.AddImage;
			this.AllowDrop = true;
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(768, 630);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.panel2,
																		  this.panel1,
																		  this.statusBar1,
																		  this.toolBar1});
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.Name = "Form1";
			this.Text = "Isis Image Maker";
			this.Load += new System.EventHandler(this.Form1_Load);
			this.DragDrop += new System.Windows.Forms.DragEventHandler(this.FileDragged);
			this.DragEnter += new System.Windows.Forms.DragEventHandler(this.DragEntered);
			((System.ComponentModel.ISupportInitialize)(this.SizePanel)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.MousePosPanel)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.SaveProgressPanel)).EndInit();
			this.panel1.ResumeLayout(false);
			this.SIGroup.ResumeLayout(false);
			this.panel2.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		#region Main
		
		[STAThread] 
		static void Main(string [] args) 
		{
			Application.EnableVisualStyles();
			Application.DoEvents();

			if (args.Length > 0)
				m_sFilename = args[0];

			Application.Run(new ImageMaker());
		}

		#endregion

		#region Properties

		public Int32 TexWidth
		{
			get
			{
				int size = 1;

				foreach (SubImage si in m_ImageList)
				{
					while (si.position.X + si.bitmap.Width > size)
						size *= 2;
				}

				return size;
			}
		}

		public Int32 TexHeight
		{
			get
			{
				int size = 1;

				foreach (SubImage si in m_ImageList)
				{
					while (si.position.Y + si.bitmap.Height > size)
						size *= 2;
				}

				return size;
			}
		}

		public float SaveProgress
		{
			get { return m_fSaveProgress; }

			set
			{
				m_fSaveProgress = value;

				if (value < 100.0f)
					SaveProgressPanel.Text = "Saving & compressing: " + ((int)value).ToString() + "%";
				else
					SaveProgressPanel.Text = "";
			}
		}

		public float OpenProgress
		{
			get { return m_fSaveProgress; }

			set
			{
				m_fSaveProgress = value;

				if (value < 100.0f)
					SaveProgressPanel.Text = "Decompressing & opening: " + ((int)value).ToString() + "%";
				else
					SaveProgressPanel.Text = "";
			}
		}

		#endregion

		#region Toolbar handlers

		private void toolBar1_ButtonClick(object sender, System.Windows.Forms.ToolBarButtonClickEventArgs e)
		{
			if (e.Button == NewButton)
			{
				SelectedSIChanged(null);
				m_ImageList.Clear();
				SizeUpdated();
				ImagePanel.Invalidate();
				m_sFilename = "";
			}
			else if (e.Button == SaveAsButton)
			{
				SaveAs();
			}
			else if (e.Button == SaveButton)
			{
				if (m_sFilename.Length > 0)
					Save();
				else 
					SaveAs();
			}
			else if (e.Button == OpenButton)
			{
				OpenFileDialog ofg = new OpenFileDialog();
				ofg.CheckFileExists = true;
				ofg.CheckPathExists = true;
				ofg.Filter = "Isis image files (*.igz)|*.igz";
				ofg.Title = "Open Isis image file";
				ofg.Multiselect = false;

				if (DialogResult.OK == ofg.ShowDialog(this))
				{
					SelectedSIChanged(null);
					m_ImageList.Clear();

					m_sFilename = ofg.FileName;
					Open();

					SizeUpdated();
					ImagePanel.Invalidate();
				}
			}
		}

		private void SaveAs()
		{
			SaveFileDialog sfg = new SaveFileDialog();
			sfg.CreatePrompt = false;
			sfg.CheckFileExists = false;
			sfg.CheckPathExists = true;
			sfg.OverwritePrompt = true;
			sfg.Filter = "Isis image files (*.igz)|*.igz";
			sfg.Title = "Save Isis image file";

			if (DialogResult.OK == sfg.ShowDialog(this))
			{
				m_sFilename = sfg.FileName;
				Save();
			}		
		}

		private void Open()
		{
			try
			{
				OpenProgress = 0.0f;

				Stream s = new FileStream(m_sFilename,FileMode.Open,FileAccess.Read);
				s = new BufferedStream(s);
				s = new GZipInputStream(s);
				s = new BufferedStream(s);

				OpenProgress += 3.0f;

				int width = ReadInt(s);

				if (width < 1 || width > 4096)
					throw new Exception("Invalid IGZ file");

				int height = ReadInt(s);

				int NumImages;

				if (height < 64)
				{
					if (DialogResult.No == MessageBox.Show(this,"Is this image an old-style IGZ file?","Old-style IGZ possibly detected",MessageBoxButtons.YesNo,MessageBoxIcon.Question))
					{
						if (height < 1 || height > 4096)
							throw new Exception("Invalid IGZ file");

						NumImages = ReadInt(s);
					}
					else
					{
						NumImages = height;
						height = width;
					}
				}
				else
				{
					if (height < 1 || height > 4096)
						throw new Exception("Invalid IGZ file");

					NumImages = ReadInt(s);
				}
	
				if (NumImages < 0 || NumImages > 4096)
					throw new Exception("Invalid IGZ file");

				OpenProgress += 5.0f;

				float ImageProgress = 0.0f;

				if (NumImages > 0)
					ImageProgress = 20.0f / (float)NumImages;
				else
					OpenProgress += 20.0f;

				for (int i = 0; i < NumImages; i++)
				{
					int x = ReadInt(s);
					int y = ReadInt(s);
					int image_width = ReadInt(s);
					int image_height = ReadInt(s);
					int border = ReadInt(s);

					SubImage si = new SubImage();
					si.position = new Point(x,y);
					si.bitmap = new Bitmap(image_width,image_height);
					si.borders = (BorderType)border;

					m_ImageList.Add(si);

					OpenProgress += ImageProgress;
				}

				Bitmap bitmap = new Bitmap(width,height);

				float LineProgress = 70.0f / (float)height;

				for (int y = 0; y < height; y++)
				{
					for (int x = 0; x < width; x++)
					{
						int r = s.ReadByte();
						int g = s.ReadByte();
						int b = s.ReadByte();
						int a = s.ReadByte();

						Color c = Color.FromArgb(a,r,g,b);

						bitmap.SetPixel(x,y,c);
					}

					OpenProgress += LineProgress;
				}

				foreach (SubImage si in m_ImageList)
				{
					Graphics g = Graphics.FromImage(si.bitmap);
					g.DrawImage(bitmap,new Rectangle(0,0,si.bitmap.Width,si.bitmap.Height),
						si.position.X,si.position.Y,si.bitmap.Width,si.bitmap.Height,GraphicsUnit.Pixel);
				}

				OpenProgress++;

				bitmap.Dispose();
				s.Close();

				OpenProgress = 101.0f;
			}
			catch (Exception e)
			{
				MessageBox.Show(this,e.Message,"An error occurred while opening the file.");
			}
		}

		private void Save()
		{
			try
			{
				SaveProgress = 0.0f;

				Stream s = new FileStream(m_sFilename,FileMode.Create,FileAccess.Write);
				s = new BufferedStream(s);
				s = new GZipOutputStream(s);
				s = new BufferedStream(s);

				SaveProgress += 3.0f;

				int width = TexWidth;
				int height = TexHeight;

				WriteInt(s,width);
				WriteInt(s,height);

				WriteInt(s,m_ImageList.Count);

				Bitmap b = new Bitmap(width,height);
				b.MakeTransparent();

				SaveProgress += 5.0f;

				float ImageProgress = 20.0f / (float)m_ImageList.Count;

				foreach (SubImage si in m_ImageList)
				{
					WriteInt(s,si.position.X);
					WriteInt(s,si.position.Y);
					WriteInt(s,si.bitmap.Width);
					WriteInt(s,si.bitmap.Height);
					WriteInt(s,(int)si.borders);
				
					Graphics g = Graphics.FromImage(b);
					g.DrawImage(si.bitmap,si.position);
					g.Dispose();

					switch (si.borders)
					{
						case BorderType.None: break;
						case BorderType.Empty: break;
						case BorderType.Mirror:
						case BorderType.Repeat:
						{
							for (int x = 0; x < si.bitmap.Width; x++)
							{
								if (si.position.X+x >= 0 && si.position.X+x < width)
								{
									Color c1 = si.bitmap.GetPixel(x,si.bitmap.Height-1);
									Color c2 = si.bitmap.GetPixel(x,0);
									Color c3,c4;

									if (si.borders == BorderType.Mirror)
									{
										c3 = c1;
										c4 = c2;
									}
									else
									{
										c3 = c2;
										c4 = c1;
									}

									if (si.position.Y-1 >= 0 && si.position.Y-1 < height)
										b.SetPixel(si.position.X+x,si.position.Y-1,c3);

									if (si.position.Y+si.bitmap.Height >= 0 && si.position.Y+si.bitmap.Height < height)
										b.SetPixel(si.position.X+x,si.position.Y+si.bitmap.Height,c4);
								}
							}

							for (int y = 0; y < si.bitmap.Height; y++)
							{
								if (si.position.Y+y >= 0 && si.position.Y+y < height)
								{
									Color c1 = si.bitmap.GetPixel(si.bitmap.Width-1,y);
									Color c2 = si.bitmap.GetPixel(0,y);
									Color c3,c4;

									if (si.borders == BorderType.Mirror)
									{
										c3 = c1;
										c4 = c2;
									}
									else
									{
										c3 = c2;
										c4 = c1;
									}

									if (si.position.X-1 >= 0 && si.position.X-1 < width)
										b.SetPixel(si.position.X-1,si.position.Y+y,c3);

									if (si.position.X+si.bitmap.Width >= 0 && si.position.X+si.bitmap.Width < width)
										b.SetPixel(si.position.X+si.bitmap.Width,si.position.Y+y,c4);
								}
							}
						} break;
					}

					SaveProgress += ImageProgress;
				}

				float LineProgress = 70.0f / (float)b.Height;

				for (int y = 0; y < b.Height; y++)
				{
					for (int x = 0; x < b.Width; x++)
					{
						Color c = b.GetPixel(x,y);
						s.WriteByte(c.R);
						s.WriteByte(c.G);
						s.WriteByte(c.B);
						s.WriteByte(c.A);
					}

					SaveProgress += LineProgress;
				}

				s.Close();
				b.Dispose();

				SaveProgress = 101.0f;
			}
			catch (Exception e)
			{
				MessageBox.Show(this,e.Message,"An error occurred while saving the file.");
			}
		}

		#endregion

		#region Add/Remove image handlers

		private void ReplaceImage_Click(object sender, System.EventArgs e)
		{
			OpenFileDialog ofg = new OpenFileDialog();
			ofg.CheckFileExists = true;
			ofg.CheckPathExists = true;
			ofg.Filter = "Image files (*.*)|*.*";
			ofg.Multiselect = true;
			ofg.Title = "Open image file";

			if (DialogResult.OK == ofg.ShowDialog(this))
			{
				try
				{
					SubImage si = m_SelectedSI;

					int index = 0;
					Point pos = new Point(0,0);
					if (null != si)
					{
						index = m_ImageList.IndexOf(si);
						pos = si.position;
						m_ImageList.Remove(si);
					}

					foreach (string sFilename in ofg.FileNames)
					{
						AddNewImage(sFilename,index,pos);
					}

					SizeUpdated();

					ImagePanel.Invalidate();
				}
				catch (Exception)
				{
					MessageBox.Show(this,"An error occurred opening that file:\n\n" + e.ToString());
				}
			}
		}

		private void AddImage_Click(object sender, System.EventArgs e)
		{
			OpenFileDialog ofg = new OpenFileDialog();
			ofg.CheckFileExists = true;
			ofg.CheckPathExists = true;
			ofg.Filter = "Image files (*.*)|*.*";
			ofg.Multiselect = true;
			ofg.Title = "Open image file";

			if (DialogResult.OK == ofg.ShowDialog(this))
			{
				try
				{
					foreach (string sFilename in ofg.FileNames)
					{
						AddNewImage(sFilename);
					}

					SizeUpdated();

					ImagePanel.Invalidate();
				}
				catch (Exception)
				{
					MessageBox.Show(this,"An error occurred opening that file:\n\n" + e.ToString());
				}
			}
		}

		private void AddNewImage(string sFilename)
		{
			Stream file = new FileStream(sFilename,FileMode.Open,FileAccess.Read);
			Bitmap b = new Bitmap(file);
			file.Close();

			b.SetResolution(96,96);

			SubImage si = new SubImage();
			si.bitmap = b;
			si.position = new Point(0,0);

			m_ImageList.Add(si);
		}

		private void AddNewImage(string sFilename, int index, Point pos)
		{
			Stream file = new FileStream(sFilename,FileMode.Open,FileAccess.Read);
			Bitmap b = new Bitmap(file);
			file.Close();

			b.SetResolution(96,96);

			SubImage si = new SubImage();
			si.bitmap = b;
			si.position = pos;

			m_ImageList.Insert(index,si);
		}

		private void RemoveImage_Click(object sender, System.EventArgs e)
		{
			if (null != m_SelectedSI)
			{
				SubImage si = m_SelectedSI;

				SelectedSIChanged(null);

				m_ImageList.Remove(si);

				SizeUpdated();
				ImagePanel.Invalidate();
			}
		}

		#endregion

		#region Paint handler

		private void ImagePanel_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
		{
			int iNum = 0;
			foreach (SubImage si in m_ImageList)
			{
				if (!m_bDragging)
				{
					e.Graphics.DrawImageUnscaled(si.bitmap,si.position.X,si.position.Y);

					if (m_SelectedSI == si)
					{
						e.Graphics.FillRectangle(
							new HatchBrush(HatchStyle.Weave,Color.FromArgb(64,128,255),Color.Transparent),
							si.position.X,si.position.Y,si.bitmap.Width,si.bitmap.Height);
					}

					if (si.HasBorders)
					{
						Color c = (m_SelectedSI == si ? Color.FromArgb(0,0,255) : Color.FromArgb(0,255,0));

						e.Graphics.DrawRectangle(new Pen(c,1),
							si.Left,si.Top,si.Right-si.Left,si.Bottom-si.Top);
					}

					if (ShowNumbersCheck.Checked)
					{
						e.Graphics.DrawString(iNum.ToString(),Font,new SolidBrush(Color.Black),si.position);
					}
				}
				else
				{
					Color c = (m_SelectedSI == si ? Color.FromArgb(0,0,255) : Color.FromArgb(0,255,0));

					e.Graphics.FillRectangle(
						new HatchBrush(HatchStyle.Cross,c,Color.Transparent),
						si.position.X,si.position.Y,si.bitmap.Width,si.bitmap.Height);

					c = (m_SelectedSI == si ? Color.FromArgb(0,255,0) : Color.FromArgb(0,0,255));

					if (si.HasBorders)
					{
						e.Graphics.DrawRectangle(new Pen(c,1),
							si.Left,si.Top,si.Right-si.Left,si.Bottom-si.Top);
					}
				}

				iNum++;
			}
		}

		#endregion

		#region Mouse handlers

		private void DownMouse(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if (null != m_SelectedSI)
				ImagePanel.Invalidate(m_SelectedSI.Rect);

			SelectedSIChanged(SIFromPoint(e.X,e.Y));

			m_LastMousePos = new Point(e.X,e.Y);
			m_MouseDownPos = m_LastMousePos;

			if (null != m_SelectedSI)
			{
				ImagePanel.Invalidate(m_SelectedSI.Rect);
				m_bMouseDown = true;
				m_MouseSIPos = m_SelectedSI.position;
			}
		}

		private void MoveMouse(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			MousePosPanel.Text = "Mouse: " + e.X.ToString() + "," + e.Y.ToString();

			if (m_bMouseDown && !m_bDragging)
			{
				ImagePanel.Invalidate();
				m_bDragging = true;
			}

			if (null != m_SelectedSI && m_bDragging)
			{
				Rectangle rect = m_SelectedSI.Rect;

				int BorderWidth = (m_SelectedSI.HasBorders ? 1 : 0);

				Point NewPos = m_MouseSIPos + (new Size(e.X,e.Y)) - (new Size(m_MouseDownPos));
				Point RectPos = NewPos;
				Size RectSize = m_SelectedSI.bitmap.Size;

				Size diff = new Size(BorderWidth,BorderWidth);
				RectPos -= diff;
				RectSize += diff;

				Rectangle NewRect = new Rectangle(RectPos,RectSize);

				if (Keys.Control != (Control.ModifierKeys & Keys.Control))
				{
					foreach (SubImage si in m_ImageList)
					{
						if (m_SelectedSI != si)
						{
							int border = (si.HasBorders ? 1 : 0);
							Size BorderSize = new Size(border,border);
							Rectangle OtherRect = new Rectangle(si.position - BorderSize,
								si.bitmap.Size + BorderSize + BorderSize);

							Rectangle rect1 = new Rectangle(NewRect.X-6,
								NewRect.Y-6,
								NewRect.Width+12,
								NewRect.Height+12);
							Rectangle rect2 = new Rectangle(OtherRect.X-6,
								OtherRect.Y-6,
								OtherRect.Width+12,
								OtherRect.Height+12);

							if (!rect1.IntersectsWith(rect2))
								continue;

							if (Math.Abs(OtherRect.Left - NewRect.Right) < 6)
								NewPos.X = OtherRect.Left - NewRect.Width;
							if (Math.Abs(OtherRect.Right - NewRect.Left) < 6)
								NewPos.X = OtherRect.Right + BorderWidth;
							if (Math.Abs(OtherRect.Top - NewRect.Bottom) < 6)
								NewPos.Y = OtherRect.Top - NewRect.Height;
							if (Math.Abs(OtherRect.Bottom - NewRect.Top) < 6)
								NewPos.Y = OtherRect.Bottom + BorderWidth;
						}
					}

					int width = TexWidth;
					int height = TexHeight;
                    Rectangle MaxRect = new Rectangle(0,0,width,height);

					if (Math.Abs(NewRect.Right - MaxRect.Right) < 6)
						NewPos.X = MaxRect.Right - NewRect.Width;
					if (Math.Abs(NewRect.Left - 0) < 6)
						NewPos.X = (m_SelectedSI.HasBorders ? 1 : 0);
					if (Math.Abs(NewRect.Bottom - MaxRect.Bottom) < 6)
						NewPos.Y = MaxRect.Bottom - NewRect.Height;
					if (Math.Abs(NewRect.Top - 0) < 6)
						NewPos.Y = (m_SelectedSI.HasBorders ? 1 : 0);
				}

				m_SelectedSI.position = NewPos;

				SIPos.Text = m_SelectedSI.position.X.ToString() + 
					"," + m_SelectedSI.position.Y.ToString();
				SISize.Text = m_SelectedSI.bitmap.Width.ToString() +
					"," + m_SelectedSI.bitmap.Height.ToString();

				SizeUpdated();

				ImagePanel.Invalidate(Rectangle.Union(rect,m_SelectedSI.Rect));
			}

			m_LastMousePos = new Point(e.X,e.Y);
		}

		private void UpMouse(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if (m_bDragging)
				ImagePanel.Invalidate();

			m_bMouseDown = false;
			m_bDragging = false;
		}

		private void MouseOut(object sender, System.EventArgs e)
		{
			MousePosPanel.Text = "Mouse: ";
		}

		#endregion

		#region Helpers

		private void WriteInt(Stream s, int i)
		{
			s.WriteByte((byte)(i & 0x000000FF));
			s.WriteByte((byte)((i & 0x0000FF00) >> 8));
			s.WriteByte((byte)((i & 0x00FF0000) >> 16));
			s.WriteByte((byte)((i & 0xFF000000) >> 24));
		}

		private int ReadInt(Stream s)
		{
			byte b1,b2,b3,b4;
			b1 = (byte)s.ReadByte();
			b2 = (byte)s.ReadByte();
			b3 = (byte)s.ReadByte();
			b4 = (byte)s.ReadByte();

			return b1 | b2<<8 | b3<<16 | b4<<24;
		}

		SubImage SIFromPoint(int x, int y)
		{
			for (int i = m_ImageList.Count - 1; i >= 0; i--)
			{
				SubImage si = m_ImageList[i] as SubImage;

				if (x > si.position.X && x < si.position.X + si.bitmap.Width &&
					y > si.position.Y && y < si.position.Y + si.bitmap.Height)
				{
					return si;
				}
			}

			return null;
		}

		private void SelectedSIChanged(SubImage si)
		{
			m_SelectedSI = si;

			if (m_SelectedSI == null)
			{
				SIGroup.Enabled = false;
				RemoveImage.Enabled = false;
				ReplaceImage.Enabled = false;

				SIPos.Text = "";
				SISize.Text = "";
				SINumber.Text = "";
			}
			else
			{
				SIGroup.Enabled = true;
				RemoveImage.Enabled = true;
				ReplaceImage.Enabled = true;

				SIPos.Text = m_SelectedSI.position.X.ToString() + 
					"," + m_SelectedSI.position.Y.ToString();
				SISize.Text = m_SelectedSI.bitmap.Width.ToString() +
					"," + m_SelectedSI.bitmap.Height.ToString();

				int iNum = 0;
				foreach (SubImage s in m_ImageList)
				{
					if (si == s)
					{
						SINumber.Text = iNum.ToString();
						break;
					}

					iNum++;
				}

				switch (m_SelectedSI.borders)
				{
					case BorderType.None:
					{
						NeedBorderCheck.Checked = false;
					} break;
					case BorderType.Repeat:
					{
						NeedBorderCheck.Checked = true;
						RepeatRadio.Checked = true;
					} break;
					case BorderType.Mirror:
					{
						NeedBorderCheck.Checked = true;
						MirrorRadio.Checked = true;
					} break;
					case BorderType.Empty:
					{
						NeedBorderCheck.Checked = true;
						EmptyRadio.Checked = true;
					} break;
				}
			}
		}

		private void SizeUpdated()
		{
			int NewWidth = TexWidth;
			int NewHeight = TexHeight;

			if (NewWidth != ImagePanel.Width || NewHeight != ImagePanel.Height)
			{
				ImagePanel.Size = new Size(NewWidth,NewHeight);
				SizePanel.Text = "Texture size: " + NewWidth.ToString() + "," + NewHeight.ToString();
			}
		}

		#endregion

		#region SelectedSI UI handlers

		private void NeedBorderCheck_CheckedChanged(object sender, System.EventArgs e)
		{
			if (null == m_SelectedSI)
				return;

			if (NeedBorderCheck.Checked)
			{
				RepeatRadio.Enabled = true;
				MirrorRadio.Enabled = true;
				EmptyRadio.Enabled = true;

				if (RepeatRadio.Checked)
					m_SelectedSI.borders = BorderType.Repeat;
				else if (MirrorRadio.Checked)
					m_SelectedSI.borders = BorderType.Mirror;
				else if (MirrorRadio.Checked)
					m_SelectedSI.borders = BorderType.Empty;
			}
			else
			{
				RepeatRadio.Enabled = false;
				MirrorRadio.Enabled = false;
				EmptyRadio.Enabled = false;

				m_SelectedSI.borders = BorderType.None;
			}

			ImagePanel.Invalidate();
		}

		private void RepeatRadio_CheckedChanged(object sender, System.EventArgs e)
		{
			if (null != m_SelectedSI && RepeatRadio.Checked)
			{
				m_SelectedSI.borders = BorderType.Repeat;
			}
		}

		private void MirrorRadio_CheckedChanged(object sender, System.EventArgs e)
		{
			if (null != m_SelectedSI && MirrorRadio.Checked)
			{
				m_SelectedSI.borders = BorderType.Mirror;
			}
		}

		private void EmptyRadio_CheckedChanged(object sender, System.EventArgs e)
		{
			if (null != m_SelectedSI && EmptyRadio.Checked)
			{
				m_SelectedSI.borders = BorderType.Empty;
			}
		}

		private void MoveFrontButton_Click(object sender, System.EventArgs e)
		{
			if (null != m_SelectedSI)
			{
				SubImage si = m_SelectedSI;

				int index = m_ImageList.IndexOf(si);

				if (index == m_ImageList.Count - 1)
					return;

				m_ImageList.Remove(si);
				index++;
				m_ImageList.Insert(index,si);

				SelectedSIChanged(si);

				ImagePanel.Invalidate();
			}
		}

		private void MoveBackButton_Click(object sender, System.EventArgs e)
		{
			if (null != m_SelectedSI)
			{
				SubImage si = m_SelectedSI;

				int index = m_ImageList.IndexOf(si);

				if (index == 0)
					return;

				m_ImageList.Remove(si);
				index--;
				m_ImageList.Insert(index,si);

				SelectedSIChanged(si);

				ImagePanel.Invalidate();
			}
		}

		private void ShowNumbersCheck_CheckedChanged(object sender, System.EventArgs e)
		{
			ImagePanel.Invalidate();
		}

		#endregion
		#region Font rendering stuff

		[StructLayout(LayoutKind.Sequential)]
		public struct SIZE
		{
			public int cx;
			public int cy;
		}
		[DllImport("gdi32.dll")]
		public static extern int GetTextExtentPoint32(IntPtr hdc, 
			String str, int len, ref SIZE size);

		[DllImport("gdi32.dll")]
		public static extern int SelectObject(IntPtr hdc, IntPtr hobj);

		private void RenderFontButton_Click(object sender, System.EventArgs e)
		{
			FontDialog fd = new FontDialog();
			fd.AllowVerticalFonts = false;
			fd.AllowScriptChange = false;
			fd.FontMustExist = true;
			fd.ScriptsOnly = true;
			fd.ShowApply = false;
			fd.ShowColor = false;
			fd.ShowHelp = false;
			fd.Font = new Font("Times New Roman",12);
			
			if (fd.ShowDialog(this) == DialogResult.OK)
			{
				SelectedSIChanged(null);
				m_ImageList.Clear();

				RenderFont(fd.Font);

				SizeUpdated();
				ImagePanel.Invalidate();
			}
		}

		private void RenderFont(Font font)
		{
			int NewSize = 256;
			if (font.SizeInPoints > 20) NewSize = 512;
			if (font.SizeInPoints > 40) NewSize = 1024;

			Bitmap b = new Bitmap(NewSize,NewSize);
			b.MakeTransparent();

			Graphics g = Graphics.FromImage(b);
			g.TextRenderingHint = TextRenderingHint.AntiAliasGridFit;
			g.CompositingMode = CompositingMode.SourceOver;
			g.CompositingQuality = CompositingQuality.HighQuality;

			int x = 1;
			int y = 1;

			for (char c = ' '; c <= '~'; c++)
			{
				SubImage si = new SubImage();
				SizeF sizef = g.MeasureString(new String(c,1),font);
				Size size = new Size();
				size.Width = (int)(Math.Ceiling(sizef.Width)+.001f);
				size.Height = (int)(Math.Ceiling(sizef.Height)+.001f);
				si.borders = BorderType.Empty;

				IntPtr hfont = font.ToHfont();
				IntPtr hdc = g.GetHdc();
				SelectObject(hdc,hfont);
				SIZE Csize = new SIZE();
				GetTextExtentPoint32(hdc,new String(c,1),1,ref Csize);
				g.ReleaseHdc(hdc);
				si.bitmap = new Bitmap(Csize.cx+4,Csize.cy+1);

				int DrawOffset = (int)(Math.Ceiling((sizef.Width - (float)Csize.cx)/2)+.001f)-2;
				if (x + si.bitmap.Width >= NewSize)
				{
					x = 1;
					y += si.bitmap.Height+1;
				}

				si.position = new Point(x,y);
				m_ImageList.Add(si);
				
				g.DrawString(new String(c,1),font,Brushes.White,x-DrawOffset,y);

				x += si.bitmap.Width+1;
			}

			g.Dispose();

			foreach (SubImage si in m_ImageList)
			{
				g = Graphics.FromImage(si.bitmap);
				
				g.DrawImage(b,new Rectangle(0,0,si.bitmap.Width,si.bitmap.Height),
					si.position.X,si.position.Y,si.bitmap.Width,si.bitmap.Height,GraphicsUnit.Pixel);

				g.Dispose();
			}
			b.Dispose();
		}

		#endregion
		#region DragDrop stuff

		private void FileDragged(object sender, System.Windows.Forms.DragEventArgs e)
		{
			try
			{
				foreach (string sFilename in (e.Data.GetData(DataFormats.FileDrop) as String[]))
				{
					AddNewImage(sFilename);
				}
				SizeUpdated();
				ImagePanel.Invalidate();
			}
			catch (Exception ex)
			{
				MessageBox.Show(this,"An error occurred opening that file:\n\n" + ex.ToString());
			}
		}

		private void DragEntered(object sender, System.Windows.Forms.DragEventArgs e)
		{
			if (e.Data.GetDataPresent(DataFormats.FileDrop))
			{
				e.Effect = DragDropEffects.Copy;
			}
		}

		#endregion
	}
}
