using System;
using System.IO;
using System.Drawing.Imaging;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Runtime.InteropServices;

using ICSharpCode.SharpZipLib.GZip;

namespace VectorEditor
{
	/// <summary>
	/// Summary description for VectorEditor.
	/// </summary>
	public class VectorEditor : System.Windows.Forms.Form
	{
		#region Extra classes

		class Animation
		{
			public string name;
			public float framerate;
			public bool syncframes;
			public ArrayList frames = new ArrayList();

			public Animation(string name, float framerate, bool syncframes)
			{
				this.name = name;
				this.framerate = framerate;
				this.syncframes = syncframes;
			}

			public Animation(string name)
			{
				this.name = name;
				this.framerate = 30;
				this.syncframes = false;
			}
		}

		class Frame
		{
			public Metafile metafile;
			public MemoryStream stream;
			public Point center;
			public Rectangle bounds;
			public int zoom;

			public Frame(MemoryStream stream, Metafile metafile)
			{
				this.stream = stream;
				this.metafile = metafile;

				center = new Point();
				center.X = metafile.Width/2;
				center.Y = metafile.Height/2;

				bounds = new Rectangle();
				bounds.X = -metafile.Width/2;
				bounds.Y = -metafile.Height/2;
				bounds.Width = metafile.Width;
				bounds.Height = metafile.Height;

				zoom = 100;
			}
		}

		public class BufferPanel : System.Windows.Forms.Panel
		{
			public BufferPanel()
			{
				this.SetStyle(
					ControlStyles.AllPaintingInWmPaint |
					ControlStyles.UserPaint |
					ControlStyles.DoubleBuffer,true);
			}
		}
		
		#endregion

		#region GUI Elements

		private System.ComponentModel.IContainer components;
		private System.Windows.Forms.StatusBar StatusBar;
		private System.Windows.Forms.StatusBarPanel MousePosPanel;
		private System.Windows.Forms.StatusBarPanel SaveProgressPanel;
		private System.Windows.Forms.ToolBar ToolBar;
		private System.Windows.Forms.ImageList ToolIconList;
		private System.Windows.Forms.ToolBarButton OpenButton;
		private System.Windows.Forms.ToolBarButton NewButton;
		private System.Windows.Forms.ToolBarButton SaveAsButton;
		private System.Windows.Forms.ToolBarButton SaveButton;
		private System.Windows.Forms.TrackBar FrameTrackBar;
		private System.Windows.Forms.Panel panel1;
		private System.Windows.Forms.Panel panel2;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.Label TotalFrames;
		private System.Windows.Forms.Label label2;
		private System.Windows.Forms.Label FrameNumber;
		private System.Windows.Forms.TabControl AnimationTabs;
		private System.Windows.Forms.TabPage untitled;
		private VectorEditor.BufferPanel ImagePanel;
		private System.Windows.Forms.GroupBox groupBox1;
		private System.Windows.Forms.GroupBox groupBox2;
		private System.Windows.Forms.Button NewAnimationButton;
		private System.Windows.Forms.Label label3;
		private System.Windows.Forms.Button DeleteAnimationButton;
		private System.Windows.Forms.Button AddFramesButton;
		private System.Windows.Forms.Button DeleteFrameButton;
		private System.Windows.Forms.Button MoveForwardButton;
		private System.Windows.Forms.Button MoveBackwardButton;
		private System.Windows.Forms.TextBox AnimationName;
		private System.Windows.Forms.ImageList MiscIcons;
		private System.Windows.Forms.Label label4;
		private System.Windows.Forms.Label label5;
		private System.Windows.Forms.Label label6;
		private System.Windows.Forms.TextBox FrameZoom;
		private System.Windows.Forms.TrackBar ZoomBar;
		private System.Windows.Forms.Label label7;
		private System.Windows.Forms.TextBox FrameRateTextBox;
		private System.Windows.Forms.CheckBox SyncFrames;
		private System.Windows.Forms.Label FrameBounds;
		private System.Windows.Forms.Label FrameSize;

		#endregion

		enum Records : int
		{
			Color = 0,
			Poly = 1,
			End = 2,
			PolyPoly = 3,
		}

		Frame m_CurFrame;
		float m_DpiX;
		float m_DpiY;
		float m_HorzRes;
		float m_VertRes;
		static string m_sFilename = "";
		Stream m_SaveFile;
		bool m_bMouseDown;
		Point m_ClickPos = new Point();
		float m_fSaveProgress = 100;
		BinaryWriter m_CurWriter;
		BinaryReader m_CurReader;

		ArrayList animations = new ArrayList();

		#region Constructor / dispose

		public VectorEditor()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			//
			// TODO: Add any constructor code after InitializeComponent call
			//
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#endregion

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(VectorEditor));
			this.StatusBar = new System.Windows.Forms.StatusBar();
			this.MousePosPanel = new System.Windows.Forms.StatusBarPanel();
			this.SaveProgressPanel = new System.Windows.Forms.StatusBarPanel();
			this.ToolBar = new System.Windows.Forms.ToolBar();
			this.NewButton = new System.Windows.Forms.ToolBarButton();
			this.OpenButton = new System.Windows.Forms.ToolBarButton();
			this.SaveButton = new System.Windows.Forms.ToolBarButton();
			this.SaveAsButton = new System.Windows.Forms.ToolBarButton();
			this.ToolIconList = new System.Windows.Forms.ImageList(this.components);
			this.FrameTrackBar = new System.Windows.Forms.TrackBar();
			this.panel1 = new System.Windows.Forms.Panel();
			this.label1 = new System.Windows.Forms.Label();
			this.TotalFrames = new System.Windows.Forms.Label();
			this.label2 = new System.Windows.Forms.Label();
			this.FrameNumber = new System.Windows.Forms.Label();
			this.panel2 = new System.Windows.Forms.Panel();
			this.groupBox1 = new System.Windows.Forms.GroupBox();
			this.SyncFrames = new System.Windows.Forms.CheckBox();
			this.ZoomBar = new System.Windows.Forms.TrackBar();
			this.FrameZoom = new System.Windows.Forms.TextBox();
			this.label4 = new System.Windows.Forms.Label();
			this.AddFramesButton = new System.Windows.Forms.Button();
			this.DeleteFrameButton = new System.Windows.Forms.Button();
			this.MoveForwardButton = new System.Windows.Forms.Button();
			this.MoveBackwardButton = new System.Windows.Forms.Button();
			this.label5 = new System.Windows.Forms.Label();
			this.FrameBounds = new System.Windows.Forms.Label();
			this.FrameSize = new System.Windows.Forms.Label();
			this.label6 = new System.Windows.Forms.Label();
			this.groupBox2 = new System.Windows.Forms.GroupBox();
			this.AnimationName = new System.Windows.Forms.TextBox();
			this.label3 = new System.Windows.Forms.Label();
			this.NewAnimationButton = new System.Windows.Forms.Button();
			this.DeleteAnimationButton = new System.Windows.Forms.Button();
			this.label7 = new System.Windows.Forms.Label();
			this.FrameRateTextBox = new System.Windows.Forms.TextBox();
			this.AnimationTabs = new System.Windows.Forms.TabControl();
			this.untitled = new System.Windows.Forms.TabPage();
			this.ImagePanel = new VectorEditor.BufferPanel();
			this.MiscIcons = new System.Windows.Forms.ImageList(this.components);
			((System.ComponentModel.ISupportInitialize)(this.MousePosPanel)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.SaveProgressPanel)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.FrameTrackBar)).BeginInit();
			this.panel1.SuspendLayout();
			this.panel2.SuspendLayout();
			this.groupBox1.SuspendLayout();
			((System.ComponentModel.ISupportInitialize)(this.ZoomBar)).BeginInit();
			this.groupBox2.SuspendLayout();
			this.AnimationTabs.SuspendLayout();
			this.SuspendLayout();
			// 
			// StatusBar
			// 
			this.StatusBar.Location = new System.Drawing.Point(0, 616);
			this.StatusBar.Name = "StatusBar";
			this.StatusBar.Panels.AddRange(new System.Windows.Forms.StatusBarPanel[] {
																						 this.MousePosPanel,
																						 this.SaveProgressPanel});
			this.StatusBar.ShowPanels = true;
			this.StatusBar.Size = new System.Drawing.Size(744, 22);
			this.StatusBar.TabIndex = 0;
			// 
			// MousePosPanel
			// 
			this.MousePosPanel.Width = 150;
			// 
			// SaveProgressPanel
			// 
			this.SaveProgressPanel.Width = 200;
			// 
			// ToolBar
			// 
			this.ToolBar.Appearance = System.Windows.Forms.ToolBarAppearance.Flat;
			this.ToolBar.Buttons.AddRange(new System.Windows.Forms.ToolBarButton[] {
																					   this.NewButton,
																					   this.OpenButton,
																					   this.SaveButton,
																					   this.SaveAsButton});
			this.ToolBar.ButtonSize = new System.Drawing.Size(80, 22);
			this.ToolBar.Divider = false;
			this.ToolBar.DropDownArrows = true;
			this.ToolBar.ImageList = this.ToolIconList;
			this.ToolBar.Location = new System.Drawing.Point(0, 0);
			this.ToolBar.Name = "ToolBar";
			this.ToolBar.ShowToolTips = true;
			this.ToolBar.Size = new System.Drawing.Size(744, 26);
			this.ToolBar.TabIndex = 1;
			this.ToolBar.TextAlign = System.Windows.Forms.ToolBarTextAlign.Right;
			this.ToolBar.ButtonClick += new System.Windows.Forms.ToolBarButtonClickEventHandler(this.ToolBar_ButtonClick);
			// 
			// NewButton
			// 
			this.NewButton.ImageIndex = 0;
			this.NewButton.Text = "New";
			// 
			// OpenButton
			// 
			this.OpenButton.ImageIndex = 1;
			this.OpenButton.Text = "Open";
			// 
			// SaveButton
			// 
			this.SaveButton.ImageIndex = 2;
			this.SaveButton.Text = "Save";
			// 
			// SaveAsButton
			// 
			this.SaveAsButton.ImageIndex = 2;
			this.SaveAsButton.Text = "Save As..";
			// 
			// ToolIconList
			// 
			this.ToolIconList.ImageSize = new System.Drawing.Size(16, 16);
			this.ToolIconList.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("ToolIconList.ImageStream")));
			this.ToolIconList.TransparentColor = System.Drawing.Color.Transparent;
			// 
			// FrameTrackBar
			// 
			this.FrameTrackBar.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Left | System.Windows.Forms.AnchorStyles.Right)));
			this.FrameTrackBar.LargeChange = 1;
			this.FrameTrackBar.Location = new System.Drawing.Point(144, 1);
			this.FrameTrackBar.Maximum = 0;
			this.FrameTrackBar.Name = "FrameTrackBar";
			this.FrameTrackBar.Size = new System.Drawing.Size(464, 45);
			this.FrameTrackBar.TabIndex = 2;
			this.FrameTrackBar.TickStyle = System.Windows.Forms.TickStyle.TopLeft;
			this.FrameTrackBar.ValueChanged += new System.EventHandler(this.FrameValueChanged);
			// 
			// panel1
			// 
			this.panel1.Controls.Add(this.FrameTrackBar);
			this.panel1.Controls.Add(this.label1);
			this.panel1.Controls.Add(this.TotalFrames);
			this.panel1.Controls.Add(this.label2);
			this.panel1.Controls.Add(this.FrameNumber);
			this.panel1.Dock = System.Windows.Forms.DockStyle.Bottom;
			this.panel1.Location = new System.Drawing.Point(0, 568);
			this.panel1.Name = "panel1";
			this.panel1.Size = new System.Drawing.Size(608, 48);
			this.panel1.TabIndex = 3;
			// 
			// label1
			// 
			this.label1.Location = new System.Drawing.Point(8, 8);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(72, 16);
			this.label1.TabIndex = 3;
			this.label1.Text = "Total frames:";
			// 
			// TotalFrames
			// 
			this.TotalFrames.Location = new System.Drawing.Point(96, 8);
			this.TotalFrames.Name = "TotalFrames";
			this.TotalFrames.Size = new System.Drawing.Size(48, 16);
			this.TotalFrames.TabIndex = 3;
			this.TotalFrames.Text = "0";
			// 
			// label2
			// 
			this.label2.Location = new System.Drawing.Point(8, 24);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(88, 16);
			this.label2.TabIndex = 3;
			this.label2.Text = "Current Frame:";
			// 
			// FrameNumber
			// 
			this.FrameNumber.Location = new System.Drawing.Point(96, 24);
			this.FrameNumber.Name = "FrameNumber";
			this.FrameNumber.Size = new System.Drawing.Size(48, 16);
			this.FrameNumber.TabIndex = 3;
			this.FrameNumber.Text = "0";
			// 
			// panel2
			// 
			this.panel2.Controls.Add(this.groupBox1);
			this.panel2.Controls.Add(this.groupBox2);
			this.panel2.Dock = System.Windows.Forms.DockStyle.Right;
			this.panel2.Location = new System.Drawing.Point(608, 48);
			this.panel2.Name = "panel2";
			this.panel2.Size = new System.Drawing.Size(136, 568);
			this.panel2.TabIndex = 4;
			// 
			// groupBox1
			// 
			this.groupBox1.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)));
			this.groupBox1.Controls.Add(this.FrameBounds);
			this.groupBox1.Controls.Add(this.SyncFrames);
			this.groupBox1.Controls.Add(this.ZoomBar);
			this.groupBox1.Controls.Add(this.FrameZoom);
			this.groupBox1.Controls.Add(this.label4);
			this.groupBox1.Controls.Add(this.AddFramesButton);
			this.groupBox1.Controls.Add(this.DeleteFrameButton);
			this.groupBox1.Controls.Add(this.MoveForwardButton);
			this.groupBox1.Controls.Add(this.MoveBackwardButton);
			this.groupBox1.Controls.Add(this.label5);
			this.groupBox1.Controls.Add(this.FrameSize);
			this.groupBox1.Controls.Add(this.label6);
			this.groupBox1.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.groupBox1.Location = new System.Drawing.Point(8, 168);
			this.groupBox1.Name = "groupBox1";
			this.groupBox1.Size = new System.Drawing.Size(120, 392);
			this.groupBox1.TabIndex = 0;
			this.groupBox1.TabStop = false;
			this.groupBox1.Text = "Frame";
			// 
			// SyncFrames
			// 
			this.SyncFrames.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left)));
			this.SyncFrames.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.SyncFrames.Location = new System.Drawing.Point(8, 368);
			this.SyncFrames.Name = "SyncFrames";
			this.SyncFrames.Size = new System.Drawing.Size(104, 16);
			this.SyncFrames.TabIndex = 4;
			this.SyncFrames.Text = "Sync all frames";
			this.SyncFrames.CheckedChanged += new System.EventHandler(this.SyncFrames_CheckedChanged);
			// 
			// ZoomBar
			// 
			this.ZoomBar.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)));
			this.ZoomBar.Cursor = System.Windows.Forms.Cursors.Default;
			this.ZoomBar.LargeChange = 10;
			this.ZoomBar.Location = new System.Drawing.Point(40, 216);
			this.ZoomBar.Maximum = 300;
			this.ZoomBar.Name = "ZoomBar";
			this.ZoomBar.Orientation = System.Windows.Forms.Orientation.Vertical;
			this.ZoomBar.Size = new System.Drawing.Size(45, 152);
			this.ZoomBar.TabIndex = 3;
			this.ZoomBar.TickFrequency = 10;
			this.ZoomBar.TickStyle = System.Windows.Forms.TickStyle.Both;
			this.ZoomBar.Value = 100;
			this.ZoomBar.ValueChanged += new System.EventHandler(this.ZoomChanged);
			// 
			// FrameZoom
			// 
			this.FrameZoom.Location = new System.Drawing.Point(48, 184);
			this.FrameZoom.Name = "FrameZoom";
			this.FrameZoom.Size = new System.Drawing.Size(56, 20);
			this.FrameZoom.TabIndex = 2;
			this.FrameZoom.Text = "100";
			this.FrameZoom.TextChanged += new System.EventHandler(this.FrameZoom_TextChanged);
			// 
			// label4
			// 
			this.label4.Location = new System.Drawing.Point(8, 152);
			this.label4.Name = "label4";
			this.label4.Size = new System.Drawing.Size(32, 16);
			this.label4.TabIndex = 1;
			this.label4.Text = "Size:";
			// 
			// AddFramesButton
			// 
			this.AddFramesButton.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.AddFramesButton.Location = new System.Drawing.Point(8, 24);
			this.AddFramesButton.Name = "AddFramesButton";
			this.AddFramesButton.Size = new System.Drawing.Size(104, 23);
			this.AddFramesButton.TabIndex = 0;
			this.AddFramesButton.Text = "Add frames...";
			this.AddFramesButton.Click += new System.EventHandler(this.AddFramesButton_Click);
			// 
			// DeleteFrameButton
			// 
			this.DeleteFrameButton.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.DeleteFrameButton.Location = new System.Drawing.Point(8, 56);
			this.DeleteFrameButton.Name = "DeleteFrameButton";
			this.DeleteFrameButton.Size = new System.Drawing.Size(104, 23);
			this.DeleteFrameButton.TabIndex = 0;
			this.DeleteFrameButton.Text = "Delete frame";
			this.DeleteFrameButton.Click += new System.EventHandler(this.DeleteFrameButton_Click);
			// 
			// MoveForwardButton
			// 
			this.MoveForwardButton.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.MoveForwardButton.Location = new System.Drawing.Point(8, 88);
			this.MoveForwardButton.Name = "MoveForwardButton";
			this.MoveForwardButton.Size = new System.Drawing.Size(104, 23);
			this.MoveForwardButton.TabIndex = 0;
			this.MoveForwardButton.Text = "Move forward";
			this.MoveForwardButton.Click += new System.EventHandler(this.MoveForwardButton_Click);
			// 
			// MoveBackwardButton
			// 
			this.MoveBackwardButton.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.MoveBackwardButton.Location = new System.Drawing.Point(8, 120);
			this.MoveBackwardButton.Name = "MoveBackwardButton";
			this.MoveBackwardButton.Size = new System.Drawing.Size(104, 23);
			this.MoveBackwardButton.TabIndex = 0;
			this.MoveBackwardButton.Text = "Move backward";
			this.MoveBackwardButton.Click += new System.EventHandler(this.MoveBackwardButton_Click);
			// 
			// label5
			// 
			this.label5.Location = new System.Drawing.Point(8, 168);
			this.label5.Name = "label5";
			this.label5.Size = new System.Drawing.Size(48, 16);
			this.label5.TabIndex = 1;
			this.label5.Text = "Bound:";
			// 
			// FrameBounds
			// 
			this.FrameBounds.Location = new System.Drawing.Point(48, 168);
			this.FrameBounds.Name = "FrameBounds";
			this.FrameBounds.Size = new System.Drawing.Size(64, 16);
			this.FrameBounds.TabIndex = 1;
			this.FrameBounds.Text = "0x0";
			// 
			// FrameSize
			// 
			this.FrameSize.Location = new System.Drawing.Point(48, 152);
			this.FrameSize.Name = "FrameSize";
			this.FrameSize.Size = new System.Drawing.Size(64, 16);
			this.FrameSize.TabIndex = 1;
			this.FrameSize.Text = "0x0";
			// 
			// label6
			// 
			this.label6.Location = new System.Drawing.Point(8, 184);
			this.label6.Name = "label6";
			this.label6.Size = new System.Drawing.Size(40, 20);
			this.label6.TabIndex = 1;
			this.label6.Text = "Zoom:";
			this.label6.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// groupBox2
			// 
			this.groupBox2.Controls.Add(this.AnimationName);
			this.groupBox2.Controls.Add(this.label3);
			this.groupBox2.Controls.Add(this.NewAnimationButton);
			this.groupBox2.Controls.Add(this.DeleteAnimationButton);
			this.groupBox2.Controls.Add(this.label7);
			this.groupBox2.Controls.Add(this.FrameRateTextBox);
			this.groupBox2.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.groupBox2.Location = new System.Drawing.Point(8, 8);
			this.groupBox2.Name = "groupBox2";
			this.groupBox2.Size = new System.Drawing.Size(120, 152);
			this.groupBox2.TabIndex = 0;
			this.groupBox2.TabStop = false;
			this.groupBox2.Text = "Animation";
			// 
			// AnimationName
			// 
			this.AnimationName.Location = new System.Drawing.Point(48, 24);
			this.AnimationName.Name = "AnimationName";
			this.AnimationName.Size = new System.Drawing.Size(64, 20);
			this.AnimationName.TabIndex = 2;
			this.AnimationName.Text = "untitled";
			this.AnimationName.TextChanged += new System.EventHandler(this.AnimationName_TextChanged);
			// 
			// label3
			// 
			this.label3.Location = new System.Drawing.Point(8, 24);
			this.label3.Name = "label3";
			this.label3.Size = new System.Drawing.Size(40, 16);
			this.label3.TabIndex = 1;
			this.label3.Text = "Name:";
			// 
			// NewAnimationButton
			// 
			this.NewAnimationButton.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.NewAnimationButton.Location = new System.Drawing.Point(8, 112);
			this.NewAnimationButton.Name = "NewAnimationButton";
			this.NewAnimationButton.Size = new System.Drawing.Size(104, 23);
			this.NewAnimationButton.TabIndex = 0;
			this.NewAnimationButton.Text = "New animation";
			this.NewAnimationButton.Click += new System.EventHandler(this.NewAnimationButton_Click);
			// 
			// DeleteAnimationButton
			// 
			this.DeleteAnimationButton.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.DeleteAnimationButton.Location = new System.Drawing.Point(8, 80);
			this.DeleteAnimationButton.Name = "DeleteAnimationButton";
			this.DeleteAnimationButton.Size = new System.Drawing.Size(104, 23);
			this.DeleteAnimationButton.TabIndex = 0;
			this.DeleteAnimationButton.Text = "Delete animation";
			this.DeleteAnimationButton.Click += new System.EventHandler(this.DeleteAnimationButton_Click);
			// 
			// label7
			// 
			this.label7.Location = new System.Drawing.Point(8, 48);
			this.label7.Name = "label7";
			this.label7.Size = new System.Drawing.Size(64, 16);
			this.label7.TabIndex = 1;
			this.label7.Text = "Framerate:";
			// 
			// FrameRateTextBox
			// 
			this.FrameRateTextBox.Location = new System.Drawing.Point(72, 48);
			this.FrameRateTextBox.Name = "FrameRateTextBox";
			this.FrameRateTextBox.Size = new System.Drawing.Size(40, 20);
			this.FrameRateTextBox.TabIndex = 2;
			this.FrameRateTextBox.Text = "30";
			this.FrameRateTextBox.TextChanged += new System.EventHandler(this.FrameRateTextBox_TextChanged);
			// 
			// AnimationTabs
			// 
			this.AnimationTabs.Controls.Add(this.untitled);
			this.AnimationTabs.Dock = System.Windows.Forms.DockStyle.Top;
			this.AnimationTabs.Location = new System.Drawing.Point(0, 26);
			this.AnimationTabs.Name = "AnimationTabs";
			this.AnimationTabs.SelectedIndex = 0;
			this.AnimationTabs.Size = new System.Drawing.Size(744, 22);
			this.AnimationTabs.TabIndex = 6;
			this.AnimationTabs.SelectedIndexChanged += new System.EventHandler(this.AnimationTabs_SelectedIndexChanged);
			// 
			// untitled
			// 
			this.untitled.Location = new System.Drawing.Point(4, 22);
			this.untitled.Name = "untitled";
			this.untitled.Size = new System.Drawing.Size(736, 0);
			this.untitled.TabIndex = 0;
			this.untitled.Text = "untitled";
			// 
			// ImagePanel
			// 
			this.ImagePanel.AutoScroll = true;
			this.ImagePanel.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.ImagePanel.Dock = System.Windows.Forms.DockStyle.Fill;
			this.ImagePanel.Location = new System.Drawing.Point(0, 48);
			this.ImagePanel.Name = "ImagePanel";
			this.ImagePanel.Size = new System.Drawing.Size(608, 520);
			this.ImagePanel.TabIndex = 7;
			this.ImagePanel.Resize += new System.EventHandler(this.OnImagePanelResize);
			this.ImagePanel.MouseUp += new System.Windows.Forms.MouseEventHandler(this.OnMouseUp);
			this.ImagePanel.Paint += new System.Windows.Forms.PaintEventHandler(this.OnImagePanelPaint);
			this.ImagePanel.MouseMove += new System.Windows.Forms.MouseEventHandler(this.OnMouseMove);
			this.ImagePanel.MouseLeave += new System.EventHandler(this.OnMouseLeave);
			this.ImagePanel.MouseDown += new System.Windows.Forms.MouseEventHandler(this.OnMouseDown);
			// 
			// MiscIcons
			// 
			this.MiscIcons.ColorDepth = System.Windows.Forms.ColorDepth.Depth24Bit;
			this.MiscIcons.ImageSize = new System.Drawing.Size(16, 16);
			this.MiscIcons.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("MiscIcons.ImageStream")));
			this.MiscIcons.TransparentColor = System.Drawing.Color.Transparent;
			// 
			// VectorEditor
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(744, 638);
			this.Controls.Add(this.ImagePanel);
			this.Controls.Add(this.panel1);
			this.Controls.Add(this.panel2);
			this.Controls.Add(this.StatusBar);
			this.Controls.Add(this.AnimationTabs);
			this.Controls.Add(this.ToolBar);
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.Name = "VectorEditor";
			this.Text = "Isis Vector Art Editor";
			this.Load += new System.EventHandler(this.VectorEditor_Load);
			((System.ComponentModel.ISupportInitialize)(this.MousePosPanel)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.SaveProgressPanel)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.FrameTrackBar)).EndInit();
			this.panel1.ResumeLayout(false);
			this.panel2.ResumeLayout(false);
			this.groupBox1.ResumeLayout(false);
			((System.ComponentModel.ISupportInitialize)(this.ZoomBar)).EndInit();
			this.groupBox2.ResumeLayout(false);
			this.AnimationTabs.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args) 
		{
			Application.EnableVisualStyles();
			Application.DoEvents();

			if (args.Length > 0)
				m_sFilename = args[0];

			Application.Run(new VectorEditor());
		}

		#region Properties

		public float SaveProgress
		{
			get { return m_fSaveProgress; }

			set
			{
				m_fSaveProgress = value;

				if (value < 100.0f)
					SaveProgressPanel.Text = "Saving & compressing: " + ((int)value).ToString() + "%";
				else
					SaveProgressPanel.Text = "";
			}
		}

		public float OpenProgress
		{
			get { return m_fSaveProgress; }

			set
			{
				m_fSaveProgress = value;

				if (value < 100.0f)
					SaveProgressPanel.Text = "Decompressing & opening: " + ((int)value).ToString() + "%";
				else
					SaveProgressPanel.Text = "";
			}
		}

		#endregion

		private bool MetafileCallback(
			EmfPlusRecordType recordType,
			int flags,
			int dataSize,
			IntPtr data,
			PlayRecordCallback callbackData)
		{  
			switch (recordType)
			{
				case EmfPlusRecordType.WmfCreateBrushIndirect:
				case EmfPlusRecordType.WmfPolygon:
				case EmfPlusRecordType.WmfPolyPolygon:
				{
					byte[] dataArray = null;
					if (data != IntPtr.Zero)
					{
						dataArray = new byte[dataSize];
						Marshal.Copy(data, dataArray, 0, dataSize);
					}

					WriteRecord(m_SaveFile,recordType,dataArray);

				} break;
				default:
					break;
			}

			return true;
		}

		private void WriteRecord(Stream file, EmfPlusRecordType type, byte[] data)
		{
			switch (type)
			{
				case EmfPlusRecordType.WmfPolygon:
				{
					WriteInt(file,(int)Records.Poly);

					int num = data[0] + (data[1]<<8);

					WriteInt(file,num);

					for (int i = 0; i < num; i++)
					{
						int x = data[i*4+2] + (data[i*4+3]<<8);
						int y = data[i*4+4] + (data[i*4+5]<<8);

						float fx = (float)(x - m_CurFrame.center.X)*m_CurFrame.zoom/100.0f*m_DpiX/m_HorzRes;
						float fy = (float)(y - m_CurFrame.center.Y)*m_CurFrame.zoom/100.0f*m_DpiY/m_VertRes;

						WriteFloat(file,fx);
						WriteFloat(file,fy);
					}
				} break;
				case EmfPlusRecordType.WmfPolyPolygon:
				{
					WriteInt(file,(int)Records.PolyPoly);

					int num = data[0] + (data[1]<<8);

					WriteInt(file,num);

//					MessageBox.Show(this,"Polypolygon data:\n\n" +
//						"0: " + data[0] +"\n" +
//						"1: " + data[1] +"\n" +
//						"2: " + data[2] +"\n" +
//						"3: " + data[3] +"\n" +
//						"4: " + data[4] +"\n" +
//						"5: " + data[5] +"\n" +
//						"6: " + data[6] +"\n" +
//						"7: " + data[7] +"\n" +
//						"8: " + data[8] +"\n" +
//						"9: " + data[9] +"\n" +
//						"0: " + data[10] +"\n");

					int offset = 2*(num+1);
					for (int i = 0; i < num; i++)
					{
						int n = data[2*(i+1)] + (data[2*(i+1)+1]<<8);
						
						WriteInt(file,n);

						for (int j = 0; j < n; j++)
						{
							int x = data[j*4+offset] + (data[j*4+1+offset]<<8);
							int y = data[j*4+2+offset] + (data[j*4+3+offset]<<8);

							float fx = (float)(x - m_CurFrame.center.X)*m_CurFrame.zoom/100.0f*m_DpiX/m_HorzRes;
							float fy = (float)(y - m_CurFrame.center.Y)*m_CurFrame.zoom/100.0f*m_DpiY/m_VertRes;

							WriteFloat(file,fx);
							WriteFloat(file,fy);
						}

						offset += 4*n;
					}
				} break;
				case EmfPlusRecordType.WmfCreateBrushIndirect:
				{
					WriteInt(file,(int)Records.Color);
					file.WriteByte(data[2]);
					file.WriteByte(data[3]);
					file.WriteByte(data[4]);
				} break;
				default:
					break;
			}
		}

		#region Helper methods
		
		void WriteInt(Stream s, int i)
		{
			s.WriteByte((byte)(i & 0x000000FF));
			s.WriteByte((byte)((i & 0x0000FF00) >> 8));
			s.WriteByte((byte)((i & 0x00FF0000) >> 16));
			s.WriteByte((byte)((i & 0xFF000000) >> 24));
		}

		void WriteString(Stream s, String text)
		{
			WriteInt(s,text.Length);

			for (int i = 0; i < text.Length; i++)
				s.WriteByte((byte)text[i]);
		}

		void WriteFloat(Stream s, float f)
		{
			m_CurWriter.Write(f);
		}

		int ReadInt(Stream s)
		{
			byte b1,b2,b3,b4;
			b1 = (byte)s.ReadByte();
			b2 = (byte)s.ReadByte();
			b3 = (byte)s.ReadByte();
			b4 = (byte)s.ReadByte();

			return b1 | b2<<8 | b3<<16 | b4<<24;
		}

		string ReadString(Stream s)
		{
			int length = ReadInt(s);

			char[] text = new char[length];

			for (int i = 0; i < length; i++)
				text[i] = (char)s.ReadByte();

			return new string(text);
		}

		float ReadFloat(Stream s)
		{
			return m_CurReader.ReadSingle();
		}

		#endregion

		private void FrameValueChanged(object sender, System.EventArgs e)
		{
			FrameNumber.Text = FrameTrackBar.Value.ToString();

			Graphics g = ImagePanel.CreateGraphics();

			Frame f = ((Frame)((Animation)animations[AnimationTabs.SelectedIndex]).frames[FrameTrackBar.Value]);
			FrameZoom.Text = f.zoom.ToString();
			FrameSize.Text = ((int)(f.metafile.Width*f.zoom/100*g.DpiX/f.metafile.HorizontalResolution)).ToString() + "x" +
				((int)(f.metafile.Height*f.zoom/100*g.DpiY/f.metafile.VerticalResolution)).ToString();
			FrameBounds.Text = ((int)(f.bounds.Width*f.zoom/100*g.DpiX/f.metafile.HorizontalResolution)).ToString() + "x" +
				((int)(f.bounds.Height*f.zoom/100*g.DpiY/f.metafile.VerticalResolution)).ToString();
			
			ImagePanel.Invalidate();
		}

		private void OnImagePanelPaint(object sender, System.Windows.Forms.PaintEventArgs e)
		{
			Pen p = new Pen(Brushes.Black);
			e.Graphics.DrawLine(p,ImagePanel.Width/2,0,ImagePanel.Width/2,ImagePanel.Height-1);
			e.Graphics.DrawLine(p,0,ImagePanel.Height/2,ImagePanel.Width-1,ImagePanel.Height/2);

			if (AnimationTabs.TabPages.Count > 0 && AnimationTabs.SelectedIndex >= 0)
			{
				Animation anim = (Animation)animations[AnimationTabs.SelectedIndex];
				
				if (anim.frames.Count > 0)
				{
					Frame frame = (Frame)anim.frames[FrameTrackBar.Value];
					Metafile mf = frame.metafile;

					e.Graphics.DrawImage(frame.metafile,(float)ImagePanel.Width/2-(float)frame.zoom/100.0f*frame.center.X*e.Graphics.DpiX/mf.HorizontalResolution,
														(float)ImagePanel.Height/2-(float)frame.zoom/100.0f*frame.center.Y*e.Graphics.DpiY/mf.VerticalResolution,
														(float)frame.zoom/100.0f*frame.metafile.Width*e.Graphics.DpiX/mf.HorizontalResolution,
														(float)frame.zoom/100.0f*frame.metafile.Height*e.Graphics.DpiY/mf.VerticalResolution);

					Pen p2 = new Pen(Brushes.Magenta,1);
					e.Graphics.DrawRectangle(p2,(float)ImagePanel.Width/2+(float)frame.zoom/100.0f*frame.bounds.X*e.Graphics.DpiX/mf.HorizontalResolution,
												(float)ImagePanel.Height/2+(float)frame.zoom/100.0f*frame.bounds.Y*e.Graphics.DpiY/mf.VerticalResolution,
												(float)frame.zoom/100.0f*frame.bounds.Width*e.Graphics.DpiX/mf.HorizontalResolution,
												(float)frame.zoom/100.0f*frame.bounds.Height*e.Graphics.DpiY/mf.VerticalResolution);

					e.Graphics.DrawImage(MiscIcons.Images[0],(ImagePanel.Width-MiscIcons.Images[0].Width)/2,
															 (ImagePanel.Height-MiscIcons.Images[0].Height)/2);
				}
			}
		}

		private void OnImagePanelResize(object sender, System.EventArgs e)
		{
			ImagePanel.Invalidate();
		}

		private void ToolBar_ButtonClick(object sender, System.Windows.Forms.ToolBarButtonClickEventArgs e)
		{
			if (e.Button == NewButton)
				New();
			if (e.Button == SaveAsButton)
				SaveAs();
			else if (e.Button == SaveButton)
			{
				if (m_sFilename.Length > 0)
					Save();
				else
					SaveAs();
			}
			else if (e.Button == OpenButton)
			{
				OpenFileDialog ofd = new OpenFileDialog();
				ofd.CheckFileExists = true;
				ofd.CheckPathExists = true;
				ofd.DefaultExt = "vgz";
				ofd.DereferenceLinks = true;
				ofd.Filter = "Isis Vector Art Files (*.vgz)|*.vgz";
				ofd.Multiselect = false;
				ofd.Title = "Open VGZ file...";
				ofd.ValidateNames = true;

				if (DialogResult.OK == ofd.ShowDialog(this))
				{
					m_sFilename = ofd.FileName;
					Open();
				}
			}
		}

		void New()
		{
			m_sFilename = "";
			AnimationTabs.TabPages.Clear();

			animations.Clear();
			animations.Add(new Animation("untitled0"));

			AnimationsChanged();
			SelectAnimation(0);
		}

		void SaveAs()
		{
			SaveFileDialog sfd = new SaveFileDialog();
			sfd.AddExtension = true;
			sfd.CheckPathExists = true;
			sfd.CreatePrompt = false;
			sfd.DefaultExt = "vgz";
			sfd.DereferenceLinks = true;
			sfd.Filter = "Isis Vector Art Files (*.vgz)|*.vgz";
			sfd.OverwritePrompt = true;
			sfd.Title = "Save VGZ file...";

			if (DialogResult.OK == sfd.ShowDialog(this))
			{
				m_sFilename = sfd.FileName;
				Save();
			}
		}

		void Save()
		{
			try
			{
				Graphics g = ImagePanel.CreateGraphics();
				m_DpiX = g.DpiX;
				m_DpiY = g.DpiY;

				SaveProgress = 0;

				Stream s = new FileStream(m_sFilename,FileMode.Create,FileAccess.Write);
				s = new BufferedStream(s);
				s = new GZipOutputStream(s);
				s = new BufferedStream(s);
				m_SaveFile = s;

				m_CurWriter = new BinaryWriter(s);

				SaveProgress += 3;

				WriteInt(m_SaveFile,animations.Count);

				int total = 0;
				for (int i = 0; i < animations.Count; i++)
					total += ((Animation)animations[i]).frames.Count;

				for (int i = 0; i < animations.Count; i++)
				{
					Animation anim = (Animation)animations[i];

					WriteString(m_SaveFile,anim.name);
					WriteFloat(m_SaveFile,anim.framerate);
					WriteInt(m_SaveFile,(anim.syncframes?1:0));
					WriteInt(m_SaveFile,anim.frames.Count);
					
					for (int j = 0; j < anim.frames.Count; j++)
					{
						Frame frame = (Frame)anim.frames[j];

						WriteInt(m_SaveFile,frame.center.X);
						WriteInt(m_SaveFile,frame.center.Y);

						WriteInt(m_SaveFile,frame.bounds.X);
						WriteInt(m_SaveFile,frame.bounds.Y);
						WriteInt(m_SaveFile,frame.bounds.Width);
						WriteInt(m_SaveFile,frame.bounds.Height);

						WriteInt(m_SaveFile,frame.zoom);

						WriteInt(m_SaveFile,(int)frame.stream.Length);
						m_SaveFile.Write(frame.stream.ToArray(),0,(int)frame.stream.Length);

						float left = (float)(frame.bounds.Left)*frame.zoom/100.0f*g.DpiX/frame.metafile.HorizontalResolution;
						float right = (float)(frame.bounds.Right)*frame.zoom/100.0f*g.DpiX/frame.metafile.HorizontalResolution;
						float top = (float)(frame.bounds.Top)*frame.zoom/100.0f*g.DpiY/frame.metafile.VerticalResolution;
						float bottom = (float)(frame.bounds.Bottom)*frame.zoom/100.0f*g.DpiY/frame.metafile.VerticalResolution;

						WriteFloat(m_SaveFile,left);
						WriteFloat(m_SaveFile,right);
						WriteFloat(m_SaveFile,top);
						WriteFloat(m_SaveFile,bottom);

						m_CurFrame = frame;
						m_HorzRes = frame.metafile.HorizontalResolution;
						m_VertRes = frame.metafile.VerticalResolution;

						g.EnumerateMetafile(frame.metafile, new Point(0,0), new Graphics.EnumerateMetafileProc(MetafileCallback));

						WriteInt(m_SaveFile,(int)Records.End);

						SaveProgress += 100.0f/total;
					}
				}

				m_CurFrame = null;

				m_CurWriter = null;
				m_SaveFile.Close();
			}
			catch (Exception ex)
			{
				if (m_SaveFile != null)
					m_SaveFile.Close();

				MessageBox.Show(this,"An error occurred while saving the file:\n\n" + 
					m_sFilename + "\n\nError message:\n\n" + ex.Message,"Error saving VGZ file",
					MessageBoxButtons.OK,MessageBoxIcon.Error);
			}
		}

		void Open()
		{
			try
			{
				OpenProgress = 0;

				Stream s = new FileStream(m_sFilename,FileMode.Open,FileAccess.Read);
				s = new BufferedStream(s);
				s = new GZipInputStream(s);
				s = new BufferedStream(s);
				m_CurReader = new BinaryReader(s);

				OpenProgress += 3.0f;

				animations.Clear();

				int num_anims = ReadInt(s);

				for (int i = 0; i < num_anims; i++)
				{
					Animation anim = new Animation(ReadString(s),ReadFloat(s),ReadInt(s) != 0);
					animations.Add(anim);

					int num_frames = ReadInt(s);

					for (int j = 0; j < num_frames; j++)
					{
						Point p = new Point(ReadInt(s),ReadInt(s));
						Rectangle r = new Rectangle(ReadInt(s),ReadInt(s),ReadInt(s),ReadInt(s));
						int z = ReadInt(s);

						int l = ReadInt(s);
						byte[] buff = new byte[l];

						int read = 0;
						while (read != l)
							read += s.Read(buff,read,l-read);

						MemoryStream ms = new MemoryStream(buff);

						Frame frame = new Frame(ms,new Metafile(ms));
						frame.bounds = r;
						frame.center = p;
						frame.zoom = z;
						anim.frames.Add(frame);

						ReadFloat(s);
						ReadFloat(s);
						ReadFloat(s);
						ReadFloat(s);

						Records rec = (Records)ReadInt(s);

						while (rec != Records.End)
						{
							switch (rec)
							{
								case Records.Color:
								{
									s.ReadByte();
									s.ReadByte();
									s.ReadByte();
								} break;
								case Records.Poly:
								{
									int num = ReadInt(s);

									for (int k = 0; k < num; k++)
									{
										ReadInt(s);
										ReadInt(s);
									}
								} break;
								case Records.PolyPoly:
								{
									int num = ReadInt(s);

									for (int k = 0; k < num; k++)
									{
										int num2 = ReadInt(s);

										for (int m = 0; m < num2; m++)
										{
											ReadInt(s);
											ReadInt(s);
										}
									}
								} break;
							}

							if (!s.CanRead)
								throw new Exception("EOF occurred while reading file");

							rec = (Records)ReadInt(s);
						}

						OpenProgress += 100.0f/num_anims/num_frames;
					}
				}

				s.Close();
			}
			catch (Exception ex)
			{
				MessageBox.Show(this,"An error occurred while opening the file:\n\n" + 
					m_sFilename + "\n\nError message:\n\n" + ex.Message,"Error opening VGZ file",
					MessageBoxButtons.OK,MessageBoxIcon.Error);

				New();
			}

			AnimationsChanged();
			SelectAnimation(0);
		}

		void AnimationsChanged()
		{
			AnimationTabs.TabPages.Clear();

			foreach (Animation anim in animations)
				AnimationTabs.TabPages.Add(new TabPage(anim.name));

			DeleteAnimationButton.Enabled = (animations.Count > 1);
		}

		void SelectAnimation(int num)
		{
			if (AnimationTabs.SelectedIndex != num)
				AnimationTabs.SelectedIndex = num;

			Animation anim = (Animation)animations[num];

			int max = anim.frames.Count-1;

			if (max < 0)
				max = 0;

			FrameTrackBar.Maximum = max;
			FrameTrackBar.Value = 0;

			TotalFrames.Text = anim.frames.Count.ToString();
			FrameNumber.Text = FrameTrackBar.Value.ToString();

			AnimationName.Text = anim.name;
			FrameRateTextBox.Text = anim.framerate.ToString();
			SyncFrames.Checked = anim.syncframes;

			DeleteFrameButton.Enabled = (anim.frames.Count != 0);
			MoveForwardButton.Enabled = DeleteFrameButton.Enabled;
			MoveBackwardButton.Enabled = DeleteFrameButton.Enabled;

			if (anim.frames.Count > 0)
			{
				FrameZoom.Enabled = true;
				ZoomBar.Enabled = true;
				FrameSize.Enabled = true;
				FrameBounds.Enabled = true;

				Graphics g = ImagePanel.CreateGraphics();

				Frame f = ((Frame)anim.frames[0]);
				FrameZoom.Text = f.zoom.ToString();
				FrameSize.Text = ((int)(f.metafile.Width*f.zoom/100*g.DpiX/f.metafile.HorizontalResolution)).ToString() + "x" +
					((int)(f.metafile.Height*f.zoom/100*g.DpiY/f.metafile.VerticalResolution)).ToString();
				FrameBounds.Text = ((int)(f.bounds.Width*f.zoom/100*g.DpiX/f.metafile.HorizontalResolution)).ToString() + "x" +
					((int)(f.bounds.Height*f.zoom/100*g.DpiY/f.metafile.VerticalResolution)).ToString();
			}
			else
			{
				FrameZoom.Text = "100";
				FrameZoom.Enabled = false;

				ZoomBar.Value = 100;
				ZoomBar.Maximum = 300;
				ZoomBar.Minimum = 0;
				ZoomBar.Enabled = false;

				FrameSize.Text = "0x0";
				FrameSize.Enabled = false;

				FrameBounds.Text = "0x0";
				FrameBounds.Enabled = false;
			}

			ImagePanel.Invalidate();
		}

		private void AnimationTabs_SelectedIndexChanged(object sender, System.EventArgs e)
		{
			if (AnimationTabs.TabPages.Count > 0 && -1 != AnimationTabs.SelectedIndex)
				SelectAnimation(AnimationTabs.SelectedIndex);
		}

		private void NewAnimationButton_Click(object sender, System.EventArgs e)
		{
			animations.Add(new Animation("untitled" + animations.Count));
			AnimationsChanged();
			SelectAnimation(animations.Count-1);
		}

		private void VectorEditor_Load(object sender, System.EventArgs e)
		{
			if (m_sFilename.Length > 0)
				Open();
			else
				New();
		}

		private void AnimationName_TextChanged(object sender, System.EventArgs e)
		{
			if (-1 != AnimationTabs.SelectedIndex)
			{
				((Animation)animations[AnimationTabs.SelectedIndex]).name = AnimationName.Text;
				AnimationTabs.SelectedTab.Text = AnimationName.Text;
			}
		}

		private void SyncFrames_CheckedChanged(object sender, System.EventArgs e)
		{
			if (-1 != AnimationTabs.SelectedIndex)
			{
				Animation anim = (Animation)animations[AnimationTabs.SelectedIndex];
				anim.syncframes = SyncFrames.Checked;

				if (anim.frames.Count > 0)
				{
					Frame f = ((Frame)((Animation)animations[AnimationTabs.SelectedIndex]).frames[FrameTrackBar.Value]);

					foreach (Frame frame in anim.frames)
						frame.zoom = f.zoom;
				}
			}
		}

		private void DeleteAnimationButton_Click(object sender, System.EventArgs e)
		{
			int old = AnimationTabs.SelectedIndex;
			animations.RemoveAt(old);
			AnimationsChanged();

			if (old > 0)
				old--;

			SelectAnimation(old);		
		}

		private void AddFramesButton_Click(object sender, System.EventArgs e)
		{
			OpenFileDialog ofd = new OpenFileDialog();
			ofd.CheckFileExists = true;
			ofd.CheckPathExists = true;
			ofd.DefaultExt = "wmf";
			ofd.DereferenceLinks = true;
			ofd.Filter = "WMF Files (*.wmf)|*.wmf";
			ofd.Multiselect = true;
			ofd.Title = "Open WMF Files...";
			ofd.ValidateNames = true;

			if (DialogResult.OK != ofd.ShowDialog(this))
				return;

			Animation anim = (Animation)animations[AnimationTabs.SelectedIndex];

			foreach (string filename in ofd.FileNames)
			{
				Metafile mf;
				MemoryStream ms;

				try
				{
					Stream s = File.OpenRead(filename);
					byte[] buff = new byte[s.Length];
					s.Read(buff,0,(int)s.Length);
					ms = new MemoryStream(buff);
					mf = new Metafile(ms);
					s.Close();
				}
				catch (Exception ex)
				{
					MessageBox.Show(this,"An error occurred while opening the file:\n\n" + 
									filename + "\n\nError message:\n\n" + ex.Message,
									"Error opening metafile",MessageBoxButtons.OK,MessageBoxIcon.Error);
					continue;
				}

/*				Graphics g = ImagePanel.CreateGraphics();
				MessageBox.Show(this,"file: " + filename + "\n\nwidth: " + mf.Width + 
									"\n\nheight: " + mf.Height + "\n\nvertres: " + 
									mf.VerticalResolution + "\n\nhorzres: " + mf.HorizontalResolution + 
									"\n\ndpi.x: " + g.DpiX + "\n\ndpi.y: " + g.DpiY);*/

				anim.frames.Add(new Frame(ms,mf));
			}

			SelectAnimation(AnimationTabs.SelectedIndex);
		}

		private void DeleteFrameButton_Click(object sender, System.EventArgs e)
		{
			Animation anim = (Animation)animations[AnimationTabs.SelectedIndex];

			int old = FrameTrackBar.Value;

			anim.frames.RemoveAt(FrameTrackBar.Value);

			SelectAnimation(AnimationTabs.SelectedIndex);

			if (old >= anim.frames.Count && old > 0)
				old--;

			FrameTrackBar.Value = old;
		}

		private void MoveForwardButton_Click(object sender, System.EventArgs e)
		{
			Animation anim = (Animation)animations[AnimationTabs.SelectedIndex];

			if (anim.frames.Count < 2)
				return;

			int old_value = FrameTrackBar.Value;
			int new_value = old_value + 1;

			if (new_value > anim.frames.Count-1)
				new_value = 0;

			Frame temp = (Frame)anim.frames[new_value];
			anim.frames[new_value] = anim.frames[old_value];
			anim.frames[old_value] = temp;

			FrameTrackBar.Value = new_value;
		}

		private void MoveBackwardButton_Click(object sender, System.EventArgs e)
		{
			Animation anim = (Animation)animations[AnimationTabs.SelectedIndex];

			if (anim.frames.Count < 2)
				return;

			int old_value = FrameTrackBar.Value;
			int new_value = old_value - 1;

			if (new_value < 0)
				new_value = anim.frames.Count-1;

			Frame temp = (Frame)anim.frames[new_value];
			anim.frames[new_value] = anim.frames[old_value];
			anim.frames[old_value] = temp;

			FrameTrackBar.Value = new_value;
		}

		private void OnMouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if (((Animation)animations[AnimationTabs.SelectedIndex]).frames.Count <= 0)
				return;

			Frame f = ((Frame)((Animation)animations[AnimationTabs.SelectedIndex]).frames[FrameTrackBar.Value]);
			Metafile m = f.metafile;
			Graphics g = ImagePanel.CreateGraphics();

			PointF p = new PointF();
			p.X = (e.X - ImagePanel.Width/2);
			p.Y = (e.Y- ImagePanel.Height/2);

			p.X *= m.HorizontalResolution;
			p.X /= g.DpiX;
			p.X /= (float)f.zoom/100.0f;

			p.Y *= m.VerticalResolution;
			p.Y /= g.DpiY;
			p.Y /= (float)f.zoom/100.0f;

			if (e.Button == MouseButtons.Right) 
			{
				int x = (int)p.X;
				int y = (int)p.Y;

				f.center.X += x;
				f.bounds.X -= x;

				f.center.Y += y;
				f.bounds.Y -= y;
			}
			else if (e.Button == MouseButtons.Left)
			{
				m_bMouseDown = true;

				f.bounds.X = (int)p.X;
				f.bounds.Y = (int)p.Y;

				f.bounds.Width = 0;
				f.bounds.Height = 0;

				FrameBounds.Text = ((int)(f.bounds.Width*f.zoom/100*g.DpiX/f.metafile.HorizontalResolution)).ToString() + "x" +
					((int)(f.bounds.Height*f.zoom/100*g.DpiY/f.metafile.VerticalResolution)).ToString();
			}

			ImagePanel.Invalidate();
		}

		private void OnMouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			MousePosPanel.Text = (e.X-ImagePanel.Width/2).ToString() + "," + (e.Y-ImagePanel.Height/2).ToString();

			if (m_bMouseDown)
			{
				Frame f = ((Frame)((Animation)animations[AnimationTabs.SelectedIndex]).frames[FrameTrackBar.Value]);
				Metafile m = f.metafile;
				Graphics g = ImagePanel.CreateGraphics();

				PointF p = new PointF();
				p.X = (e.X - ImagePanel.Width/2);
				p.Y = (e.Y - ImagePanel.Height/2);

				p.X *= m.HorizontalResolution;
				p.X /= g.DpiX;
				p.X /= (float)f.zoom/100.0f;

				p.Y *= m.VerticalResolution;
				p.Y /= g.DpiY;
				p.Y /= (float)f.zoom/100.0f;

				int x = (int)p.X;
				int y = (int)p.Y;

				bool bChanged = false;

				if (x - f.bounds.Left > f.bounds.Width)
				{
					f.bounds.Width = x - f.bounds.Left;
					bChanged = true;
				}
				else if (x < f.bounds.X)
				{
					f.bounds.Width += f.bounds.X - x;
					f.bounds.X = x;
					bChanged = true;
				}

				if (y - f.bounds.Top > f.bounds.Height)
				{
					f.bounds.Height = y - f.bounds.Top;
					bChanged = true;
				}
				else if (y < f.bounds.Y)
				{
					f.bounds.Height += f.bounds.Y - y;
					f.bounds.Y = y;
					bChanged = true;
				}

				if (bChanged)
				{
					FrameBounds.Text = ((int)(f.bounds.Width*f.zoom/100*g.DpiX/f.metafile.HorizontalResolution)).ToString() + "x" +
						((int)(f.bounds.Height*f.zoom/100*g.DpiY/f.metafile.VerticalResolution)).ToString();

					ImagePanel.Invalidate();
				}
			}
		}

		private void OnMouseUp(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if (m_bMouseDown)
				m_bMouseDown = false;
		}

		private void ZoomChanged(object sender, System.EventArgs e)
		{
			if (FrameZoom.Text != ZoomBar.Value.ToString())
				FrameZoom.Text = ZoomBar.Value.ToString();

			Animation anim = (Animation)animations[AnimationTabs.SelectedIndex];

			if (anim.frames.Count > 0)
			{
				Graphics g = ImagePanel.CreateGraphics();
				Frame f = ((Frame)anim.frames[FrameTrackBar.Value]);
				f.zoom = ZoomBar.Value;
				FrameSize.Text = ((int)(f.metafile.Width*f.zoom/100*g.DpiX/f.metafile.HorizontalResolution)).ToString() + "x" +
					((int)(f.metafile.Height*f.zoom/100*g.DpiY/f.metafile.VerticalResolution)).ToString();
				FrameBounds.Text = ((int)(f.bounds.Width*f.zoom/100*g.DpiX/f.metafile.HorizontalResolution)).ToString() + "x" +
					((int)(f.bounds.Height*f.zoom/100*g.DpiY/f.metafile.VerticalResolution)).ToString();

				ImagePanel.Invalidate();

				if (anim.syncframes)
				{
					foreach (Frame frame in anim.frames)
						frame.zoom = f.zoom;
				}
			}
		}

		private void FrameZoom_TextChanged(object sender, System.EventArgs e)
		{
			int num;

			try
			{
				num = int.Parse(FrameZoom.Text);
			}
			catch (Exception)
			{
				num = 0;
			}

			if (FrameZoom.Text != num.ToString())
        		FrameZoom.Text = num.ToString();

			if (ZoomBar.Value != num)
			{
				if (num < ZoomBar.Minimum)
					ZoomBar.Minimum = num;
				if (num > ZoomBar.Maximum)
					ZoomBar.Maximum = num;

				ZoomBar.Value = num;
			}
		}

		private void OnMouseLeave(object sender, System.EventArgs e)
		{
			MousePosPanel.Text = "";
		}

		private void FrameRateTextBox_TextChanged(object sender, System.EventArgs e)
		{
			if (-1 != AnimationTabs.SelectedIndex)
			{
				float f = 0;

				try
				{
					f = float.Parse(FrameRateTextBox.Text);
				}
				catch
				{
					f = 1.0f;
				}

				if (f < 0)
					f = 1;

				((Animation)animations[AnimationTabs.SelectedIndex]).framerate = f;
			}
		}
	}
}
